MODULE Class_Statistic
  IMPLICIT NONE
  INTEGER,PRIVATE, PARAMETER :: r8=SELECTED_REAL_KIND(15,307)
  INTEGER,PRIVATE, PARAMETER :: r4 = SELECTED_REAL_KIND(6)  ! Kind for 32-bits Real Numbers

  INTERFACE AVE
     MODULE PROCEDURE Media1D, Media2D
  END INTERFACE

  INTERFACE SD
     MODULE PROCEDURE StandardDeviation1D, StandardDeviation2D
  END INTERFACE

  INTERFACE Variability
     MODULE PROCEDURE Variability1D, Variability2D
  END INTERFACE

  INTERFACE RemoveRegrLinear
     MODULE PROCEDURE RemoveRegrLinear1, RemoveRegrLinear2
  END INTERFACE

CONTAINS

  FUNCTION Adimensionalizacao_Torrence_Compo(VAR_Y,alphacore) RESULT(z2)
    IMPLICIT NONE
    REAL(KIND=r8) , INTENT(IN   )   :: VAR_Y (:) 
    REAL(KIND=r8) , INTENT(OUT  )   :: alphacore
    REAL(KIND=r8)    :: VAR_X (SIZE(VAR_Y,dim=1))
    INTEGER          :: I
    INTEGER          :: nData
    INTEGER          :: ndata1
    INTEGER          :: ndata2
    REAL(KIND=r8)    :: avgx, avgy,avgz, stdx, stdy,stdz, sum1, sum2
    REAL(KIND=r8)    :: StandardDeviation
    REAL(KIND=r8)    :: SXiXi (SIZE(VAR_Y,1))
    REAL(KIND=r8)    :: SYkYk (SIZE(VAR_Y,1))

    REAL(KIND=r8)    :: Z1 (SIZE(VAR_Y,1))
    REAL(KIND=r8)    :: Z2 (SIZE(VAR_Y,1))
    REAL(KIND=r8)    :: zlag1 (SIZE(VAR_Y,1))
    REAL(KIND=r8)    :: zlag2 (SIZE(VAR_Y,1))    
    REAL(KIND=r8)    :: ave   (SIZE(VAR_Y,dim=1))
    REAL(KIND=r8)    :: SXY
    REAL(KIND=r8)    :: SXX
    REAL(KIND=r8)    :: betha
    REAL(KIND=r8)    :: alfa
    REAL(KIND=r8)    :: r2,rr1,rr2

    nData=SIZE(VAR_Y,dim=1)
    
    ! Vetor com os instantes de tempo.
    
    VAR_X=(/(I,I=1,nData)/)
    
    ! PASSO 1: remocao da tendencia.
    !
    !           ----N
    !          \         
    !           \    Yi
    !           /        
    !          /
    !           ----i=1
    ! avgy= ---------------------- 
    !                   N
    !
    avgy = Media1D(VAR_Y)
    !           ----N
    !          \         
    !           \    Xi
    !           /        
    !          /
    !           ----i=1
    ! avgx= ---------------------- 
    !                   N
    avgx = Media1D(VAR_X)    
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Yi - Y   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdy=StandardDeviation1D(VAR_Y)
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdx=StandardDeviation1D(VAR_X)
    !
    !           
    !             --       --  
    !SYkYk=      | Yi - Ymed | 
    !             --       --  
    !           
    !
    SYkYk=Variability(VAR_Y)

    !
    !           
    !             --       --  
    !SXkXk=      | Xi - Xmed | 
    !             --       --  
    !           
    SXiXi=Variability(VAR_X)
    !
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Yi - Ymed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXY  = --------------------------
    !
    !                  N
    !
    SXY = SUM(SXiXi*SYkYk)/SIZE(VAR_Y,dim=1)
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Xi - Xmed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXX  = --------------------------
    !
    !                  N
    !  
    SXX = SUM(SXiXi*SXiXi)/SIZE(VAR_Y,dim=1)
    !        SXY   
    !   b = -----
    !        SXX
    betha = SXY/SXX
    !
    !   a = Ymed - b Xmed
    !
    alfa = avgy - betha* avgx
  
    print *, 'aa=',alfa,' bb=',betha

    !
    !   Y=a+bX
    !  
    !
    ! COEFICIENTE DE CORRELAO LINEAR DE PEARSON r2
    !
    ! Razao entre a covariancia de duas variaveis 
    ! pelo produto dos desvios pelo produto dos desvios padrao de cada padrao de cada
    ! uma delas
    !
    r2 = (SXY/REAL(nData-1))/(stdx*stdy)

    DO i = 1, nData
         !Z    = Y        -   Yregresao 
         Z1(i) = VAR_Y(i) - (alfa + betha*VAR_X(i)) + avgy
    END DO
    !
    ! PASSO 2: ADIMENSIONALIZACAO.
    !
    !
    !           ----N
    !          \         
    !           \    Z1i
    !           /        
    !          /
    !           ----i=1
    ! avgz= ---------------------- 
    !                   N
    !   
    avgz = Media1D(Z1)      
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --     _ -- 2    |
    !      |    \ | Z1i - Z1   |    |
    !      |    /  --       --      |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdz=StandardDeviation1D(Z1)
    print *, 'avgz=',avgz, ' stdz=',stdz

      do i = 1, nData
         z2(i) = (z1(i) - avgz)/stdz
      enddo
!
! PASSO 3: CALCULO DO PARAMETRO ALPHA DE TORRENCE E COMPO (1998).
! LAG 1
      zlag1=0.0
      ndata1 = ndata - 1
      do i = 1, ndata1
         zlag1(i) = z2(i+1)
      enddo
      
     ave=RegrLinear2(zlag1(1:ndata1),z2(1:ndata1),rr1)     
     print *, 'rr_lag1=',rr1

! LAG 2      
      zlag2=0.0
      ndata2 = ndata - 2
      do i = 1, ndata2
         zlag2(i) = z2(i+2)
      enddo
     ave=RegrLinear2(zlag2(1:ndata2),z2(1:ndata2),rr2)
     print *, 'rr_lag2=',rr2

! ALPHA
      rr1 = abs(rr1)
      rr2 = abs(rr2)
      alphacore = (rr1 + sqrt(rr2))/2.0
      print *, 'alpha=',alphacore 

! ESCRITA DA NOVA SERIE TEMPORAL.
      open (11,file='processed_data.txt',status='unknown')
      do i = 1, ndata
         write (11,*) z2(i)
      enddo
      close (11)

  END FUNCTION Adimensionalizacao_Torrence_Compo

  FUNCTION RegrLinear2(VAR_X,VAR_Y,r2) RESULT(ave)
    IMPLICIT NONE
    !
    !   Y=a+bX
    !
    !        SXY   
    !   b = -----
    !        SXX
    !
    !
    !   a = Ymed - b Xmed
    !
    !
    REAL(KIND=r8), INTENT(IN   )    :: VAR_X(:)     
    REAL(KIND=r8), INTENT(IN   )    :: VAR_Y(:)
    REAL(KIND=r8), INTENT(OUT  )    :: r2
    REAL(KIND=r8)    :: ave  (SIZE(VAR_Y,dim=1))
    REAL(KIND=r8)    :: SXY
    REAL(KIND=r8)    :: SXX
    REAL(KIND=r8)    :: Ymed
    REAL(KIND=r8)    :: Xmed
    REAL(KIND=r8)    :: betha
    REAL(KIND=r8)    :: alfa
    REAL(KIND=r8)    :: stdy 
    REAL(KIND=r8)    :: stdx

    REAL(KIND=r8) :: SXiXi (SIZE(VAR_Y,1))
    REAL(KIND=r8) :: SYkYk (SIZE(VAR_Y,1))
    INTEGER :: i
    !
    !           
    !             --       --  
    !SYkYk=      | Yi - Ymed | 
    !             --       --  
    !           
    !
    SYkYk=Variability(VAR_Y)

    !
    !           
    !             --       --  
    !SXkXk=      | Xi - Xmed | 
    !             --       --  
    !           
    SXiXi=Variability(VAR_X)
    !
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Yi - Ymed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXY  = --------------------------
    !
    !                  N -1
    !
    SXY = SUM(SXiXi*SYkYk)/(SIZE(VAR_Y,dim=1) -1)
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Xi - Xmed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXX  = --------------------------
    !
    !                  N - 1
    !  
    SXX = SUM(SXiXi*SXiXi)/(SIZE(VAR_Y,dim=1)-1)
    !
    !           ----N
    !          \         
    !           \    Yi
    !           /        
    !          /
    !           ----i=1
    ! Ymed= ---------------------- 
    !                   N
    !
    Ymed = Media1D(VAR_Y)
    !           ----N
    !          \         
    !           \    Xi
    !           /        
    !          /
    !           ----i=1
    ! Xmed= ---------------------- 
    !                   N

    Xmed = Media1D(VAR_X)
    !        SXY   
    !   b = -----
    !        SXX
    betha = SXY/SXX
    !
    !   a = Ymed - b Xmed
    !
    alfa = Ymed - betha* Xmed
    !
    !   Y=a+bX
    !
    ave = alfa+betha*VAR_X
    !
    !   Y=a+bX
    !  
    !
    ! COEFICIENTE DE CORRELAO LINEAR DE PEARSON r2
    !
    ! Razao entre a covariancia de duas variaveis 
    ! pelo produto dos desvios pelo produto dos desvios padrao de cada padrao de cada
    ! uma delas
    !
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Yi - Y   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdy=StandardDeviation1D(VAR_Y)
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdx=StandardDeviation1D(VAR_X)

    r2 = (SXY)/(stdx*stdy)

  END FUNCTION RegrLinear2

  FUNCTION RemoveRegrLinear2(VAR_X,VAR_Y,r2) RESULT(ave)
    IMPLICIT NONE
    !
    !   Y=a+bX
    !
    !        SXY   
    !   b = -----
    !        SXX
    !
    !
    !   a = Ymed - b Xmed
    !
    !
    REAL(KIND=r8), INTENT(IN   )    :: VAR_X(:)     
    REAL(KIND=r8), INTENT(IN   )    :: VAR_Y(:)
    REAL(KIND=r8), INTENT(OUT  )    :: r2
    REAL(KIND=r8)    :: ave  (SIZE(VAR_Y,dim=1))
    REAL(KIND=r8)    :: SXY
    REAL(KIND=r8)    :: SXX
    REAL(KIND=r8)    :: Ymed
    REAL(KIND=r8)    :: Xmed
    REAL(KIND=r8)    :: betha
    REAL(KIND=r8)    :: alfa
    REAL(KIND=r8)    :: stdy 
    REAL(KIND=r8)    :: stdx

    REAL(KIND=r8) :: SXiXi (SIZE(VAR_Y,1))
    REAL(KIND=r8) :: SYkYk (SIZE(VAR_Y,1))
    INTEGER :: i
    !
    !           
    !             --       --  
    !SYkYk=      | Yi - Ymed | 
    !             --       --  
    !           
    !
    SYkYk=Variability(VAR_Y)

    !
    !           
    !             --       --  
    !SXkXk=      | Xi - Xmed | 
    !             --       --  
    !           
    SXiXi=Variability(VAR_X)
    !
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Yi - Ymed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXY  = --------------------------
    !
    !                  N -1
    !
    SXY = SUM(SXiXi*SYkYk)/(SIZE(VAR_Y,dim=1) -1)
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Xi - Xmed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXX  = --------------------------
    !
    !                  N - 1
    !  
    SXX = SUM(SXiXi*SXiXi)/(SIZE(VAR_Y,dim=1)-1)
    !
    !           ----N
    !          \         
    !           \    Yi
    !           /        
    !          /
    !           ----i=1
    ! Ymed= ---------------------- 
    !                   N
    !
    Ymed = Media1D(VAR_Y)
    !           ----N
    !          \         
    !           \    Xi
    !           /        
    !          /
    !           ----i=1
    ! Xmed= ---------------------- 
    !                   N

    Xmed = Media1D(VAR_X)
    !        SXY   
    !   b = -----
    !        SXX
    betha = SXY/SXX
    !
    !   a = Ymed - b Xmed
    !
    alfa = Ymed - betha* Xmed
    !
    !   Y=a+bX
    !
    ave = alfa+betha*VAR_X
    !
    !   Y=a+bX
    !  
    !
    ! COEFICIENTE DE CORRELAO LINEAR DE PEARSON r2
    !
    ! Razao entre a covariancia de duas variaveis 
    ! pelo produto dos desvios pelo produto dos desvios padrao de cada padrao de cada
    ! uma delas
    !
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Yi - Y   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdy=StandardDeviation1D(VAR_Y)
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdx=StandardDeviation1D(VAR_X)

    r2 = (SXY)/(stdx*stdy)

  END FUNCTION RemoveRegrLinear2


  FUNCTION RemoveRegrLinear1(VAR_Y) RESULT(ave)
    IMPLICIT NONE
    !
    !   Y=a+bX
    !
    !        SXY   
    !   b = -----
    !        SXX
    !
    !
    !   a = Ymed - b Xmed
    !
    !
    INTEGER :: q
    REAL(KIND=r8)    :: VAR_Y(:) 
    REAL(KIND=r8)    :: ave  (SIZE(VAR_Y,dim=1))
    REAL(KIND=r8)    :: VAR_X(SIZE(VAR_Y,dim=1))
    REAL(KIND=r8)    :: SXY
    REAL(KIND=r8)    :: SXX
    REAL(KIND=r8)    :: Ymed
    REAL(KIND=r8)    :: Xmed
    REAL(KIND=r8)    :: betha
    REAL(KIND=r8)    :: alfa
    REAL(KIND=r8)    :: stdy 
    REAL(KIND=r8)    :: stdx
    REAL(KIND=r8)    :: r2

    REAL(KIND=r8) :: SXiXi (SIZE(VAR_Y,1))
    REAL(KIND=r8) :: SYkYk (SIZE(VAR_Y,1))
    INTEGER :: i
    DO i=1,(SIZE(VAR_Y,dim=1))
       VAR_X(i)=i
    END DO
    !
    !           
    !             --       --  
    !SYkYk=      | Yi - Ymed | 
    !             --       --  
    !           
    !
    SYkYk=Variability(VAR_Y)

    !
    !           
    !             --       --  
    !SXkXk=      | Xi - Xmed | 
    !             --       --  
    !           
    SXiXi=Variability(VAR_X)
    !
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Yi - Ymed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXY  = --------------------------
    !
    !                  N
    !
    SXY = SUM(SXiXi*SYkYk)/SIZE(VAR_Y,dim=1)
    !
    !           ----  N
    !          \     --       --     --       --  
    !           \   | Xi - Xmed | * | Xi - Xmed | 
    !           /    --       --     --       --  
    !          /
    !           ----  i=1  
    !
    !SXX  = --------------------------
    !
    !                  N
    !  
    SXX = SUM(SXiXi*SXiXi)/SIZE(VAR_Y,dim=1)
    !
    !           ----N
    !          \         
    !           \    Yi
    !           /        
    !          /
    !           ----i=1
    ! Ymed= ---------------------- 
    !                   N
    !
    Ymed = Media1D(VAR_Y)
    !           ----N
    !          \         
    !           \    Xi
    !           /        
    !          /
    !           ----i=1
    ! Xmed= ---------------------- 
    !                   N

    Xmed = Media1D(VAR_X)
    !        SXY   
    !   b = -----
    !        SXX
    betha = SXY/SXX
    !
    !   a = Ymed - b Xmed
    !
    alfa = Ymed - betha* Xmed
    !
    !   Y=a+bX
    !
    ave = alfa+betha*VAR_X
    !
    !   Y=a+bX
    !  
    !
    ! COEFICIENTE DE CORRELAO LINEAR DE PEARSON r2
    !
    ! Razao entre a covariancia de duas variaveis 
    ! pelo produto dos desvios pelo produto dos desvios padrao de cada padrao de cada
    ! uma delas
    !
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Yi - Y   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdy=StandardDeviation1D(VAR_Y)
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    stdx=StandardDeviation1D(VAR_X)

    r2 = (SXY/REAL((SIZE(VAR_Y,dim=1)) -1 ))/(stdx*stdy)

  END FUNCTION RemoveRegrLinear1

  FUNCTION NormalizedSerieTemporal(var) RESULT(ave)
    IMPLICIT NONE
    INTEGER :: q
    REAL(KIND=r8)    :: var(:) 
    REAL(KIND=r8)    :: sum
    REAL(KIND=r8)    :: ave(SIZE(var,dim=1))
    REAL(KIND=r8)    :: VariabilityN(SIZE(var,dim=1))
    REAL(KIND=r8)    :: StandardDeviation
    INTEGER :: i
    INTEGER :: r
    !
    !               --       --
    !  Var(i)=     | Xi - Xmed |
    !               --       --
    !          
    !
    VariabilityN=Variability1D(var)
    !
    !
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i=1               |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    StandardDeviation=StandardDeviation1D(var)
    !
    !                            --           --
    !                           |    Var(i)     |
    ! NormalizedSerieTemporal = | ------------  |
    !                           |     SD        |
    !                            --           --
    !
    !
    ave=VariabilityN/StandardDeviation

    !  Xi = SD * NormalizedSerieTemporal + Xmed

  END FUNCTION NormalizedSerieTemporal


  FUNCTION CycleDiurnalAverage(var,FirstYear,LastYear,&
       FirstMonth,LastMonth,&
       FirstDay,LastDay) RESULT(DIURNALCYCLE)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(in   ) :: var(:)
    INTEGER      , INTENT(in   ) :: FirstYear,LastYear
    INTEGER      , INTENT(in   ) :: FirstMonth,LastMonth
    INTEGER      , INTENT(in   ) :: FirstDay,LastDay
    REAL(KIND=r8)    :: SUMA(24)
    REAL(KIND=r8)    :: COUNT(24)
    REAL(KIND=r8)    :: DIURNALCYCLE(24)
    INTEGER          :: ntime
    INTEGER          :: iday
    INTEGER          :: ihour
    INTEGER          :: imonth
    INTEGER          :: iyear
    INTEGER          :: nDaybyMonth
    INTEGER          :: nFirstDay,nLastDay
    INTEGER          :: nFirstMonth,nLastMonth
    LOGICAL          :: First
    REAL(KIND=r8), PARAMETER    :: undef =-999.0
    INTEGER, PARAMETER :: LengthMonth(12)=(/31,28,31,30,31,30,31,31,30,31,30,31/)
    COUNT=0.0
    SUMA=0.0
    DIURNALCYCLE=0.0
    nFirstDay=FirstDay
    nLastDay=LastDay
    nFirstMonth=FirstMonth
    nLastMonth=LastMonth
    First=.FALSE.
    ntime=0
    DO iyear=FirstYear,LastYear
       IF(iyear > FirstYear)THEN
          nFirstMonth=1
          nLastMonth=12
       END IF
       IF(FirstYear < LastYear)nLastMonth=12
       IF(iyear == LastYear)nLastMonth=LastMonth
       DO imonth=nFirstMonth,nLastMonth
          nDaybyMonth=LengthMonth(12)
          IF(MOD(REAL(iyear),4.0) == 0.0 .AND. imonth==2)nDaybyMonth=29
          IF(First)nLastDay=nDaybyMonth
          DO iday=nFirstDay,nLastDay
             DO ihour=1,24
                ntime=ntime+1
                IF(var(iday) /= undef)THEN
                   SUMA(ihour)  = SUMA (ihour) + var(ntime)
                   COUNT(ihour) = COUNT(ihour) + 1
                END IF
             END DO
          END DO
          First=.TRUE.
          nFirstDay=1
       END DO
    END DO
    DO ihour=1,24
       IF(COUNT(ihour) /= 0.0_r8)THEN
          DIURNALCYCLE(ihour)=SUMA(ihour)/COUNT(ihour)
       ELSE
          DIURNALCYCLE(ihour)=0.0_r8
       END IF
    END DO


  END FUNCTION CycleDiurnalAverage

  FUNCTION RemoveCycleDiurnalAverage(var,FirstYear,LastYear,&
       FirstMonth,LastMonth,&
       FirstDay,LastDay) RESULT(varfilter)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(in   ) :: var(:)
    INTEGER      , INTENT(in   ) :: FirstYear,LastYear
    INTEGER      , INTENT(in   ) :: FirstMonth,LastMonth
    INTEGER      , INTENT(in   ) :: FirstDay,LastDay
    REAL(KIND=r8)    :: varfilter(SIZE(var,dim=1))
    REAL(KIND=r8)    :: SUMA(24)
    REAL(KIND=r8)    :: COUNT(24)
    REAL(KIND=r8)    :: DIURNALCYCLE(24)
    INTEGER          :: ntime
    INTEGER          :: iday
    INTEGER          :: ihour
    INTEGER          :: imonth
    INTEGER          :: iyear
    INTEGER          :: nDaybyMonth
    INTEGER          :: nFirstDay,nLastDay
    INTEGER          :: nFirstMonth,nLastMonth
    LOGICAL          :: First
    REAL(KIND=r8), PARAMETER    :: undef =-999.0
    INTEGER, PARAMETER :: LengthMonth(12)=(/31,28,31,30,31,30,31,31,30,31,30,31/)

    DIURNALCYCLE=CycleDiurnalAverage(var,FirstYear,LastYear,&
         FirstMonth,LastMonth,&
         FirstDay,LastDay)
    nFirstDay=FirstDay
    nLastDay=LastDay
    nFirstMonth=FirstMonth
    nLastMonth=LastMonth
    First=.FALSE.
    ntime=0
    DO iyear=FirstYear,LastYear
       IF(iyear > FirstYear)THEN
          nFirstMonth=1
          nLastMonth=12
       END IF
       IF(FirstYear < LastYear)nLastMonth=12
       IF(iyear == LastYear)nLastMonth=LastMonth
       DO imonth=nFirstMonth,nLastMonth
          nDaybyMonth=LengthMonth(12)
          IF(MOD(REAL(iyear),4.0) == 0.0 .AND. imonth==2)nDaybyMonth=29
          IF(First)nLastDay=nDaybyMonth
          DO iday=nFirstDay,nLastDay
             DO ihour=1,24
                ntime=ntime+1
                varfilter(ntime)=   var(ntime) - DIURNALCYCLE(ihour)
             END DO
          END DO
          First=.TRUE.
          nFirstDay=1
       END DO
    END DO

  END FUNCTION RemoveCycleDiurnalAverage


  FUNCTION ComposeCycleDiurnalAverage(var,DIURNALCYCLE,FirstYear,LastYear,&
       FirstMonth,LastMonth,&
       FirstDay,LastDay) RESULT(varfilter)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(in   ) :: var(:)
    REAL(KIND=r8), INTENT(in   ) :: DIURNALCYCLE(24)
    INTEGER      , INTENT(in   ) :: FirstYear,LastYear
    INTEGER      , INTENT(in   ) :: FirstMonth,LastMonth
    INTEGER      , INTENT(in   ) :: FirstDay,LastDay
    REAL(KIND=r8)    :: varfilter(SIZE(var,dim=1))
    REAL(KIND=r8)    :: SUMA(24)
    REAL(KIND=r8)    :: COUNT(24)
    INTEGER          :: ntime
    INTEGER          :: iday
    INTEGER          :: ihour
    INTEGER          :: imonth
    INTEGER          :: iyear
    INTEGER          :: nDaybyMonth
    INTEGER          :: nFirstDay,nLastDay
    INTEGER          :: nFirstMonth,nLastMonth
    LOGICAL          :: First
    REAL(KIND=r8), PARAMETER    :: undef =-999.0
    INTEGER, PARAMETER :: LengthMonth(12)=(/31,28,31,30,31,30,31,31,30,31,30,31/)

    nFirstDay=FirstDay
    nLastDay=LastDay
    nFirstMonth=FirstMonth
    nLastMonth=LastMonth
    First=.FALSE.
    ntime=0
    DO iyear=FirstYear,LastYear
       IF(iyear > FirstYear)THEN
          nFirstMonth=1
          nLastMonth=12
       END IF
       IF(FirstYear < LastYear)nLastMonth=12
       IF(iyear == LastYear)nLastMonth=LastMonth
       DO imonth=nFirstMonth,nLastMonth
          nDaybyMonth=LengthMonth(12)
          IF(MOD(REAL(iyear),4.0) == 0.0 .AND. imonth==2)nDaybyMonth=29
          IF(First)nLastDay=nDaybyMonth
          DO iday=nFirstDay,nLastDay
             DO ihour=1,24
                ntime=ntime+1
                varfilter(ntime)= DIURNALCYCLE(ihour)
             END DO
          END DO
          First=.TRUE.
          nFirstDay=1
       END DO
    END DO

  END FUNCTION ComposeCycleDiurnalAverage

  FUNCTION MOVINGAVERAGE(var,q) RESULT(ave)
    IMPLICIT NONE
    INTEGER :: q
    REAL(KIND=r8)    :: var(:) 
    REAL(KIND=r8)    :: sum
    REAL(KIND=r8)    :: ave(SIZE(var,dim=1))
    INTEGER :: i
    INTEGER :: r
    !                                                                                     q
    !                                                                                    _____
    !  Y1 + Y2 + ... + Yn     Y2 + Y3 +... + Yn+1     Y3 + Y4 + ... + Yn+2         1     \
    ! -------------------- , --------------------,    --------------------,   ----------  >     x(i+r) 
    !         n                        n                       n                2q + 1   /____
    !                                                                                    r=-q  

    ave=var
    DO i=1,SIZE(var,dim=1)
       sum=0
       IF(i-q >=1 .AND. i+q <= SIZE(var,dim=1))THEN
          DO r=i-q,i+q
             sum=sum+var(r)
          END DO
          ave(i)=sum/(2*q+1)
       ELSE IF(i-q < 1 .AND. i+q <= SIZE(var,dim=1))THEN
          DO r=i,i+q
             sum=sum+var(r)
          END DO
          ave(i)=sum/(1*q+1)       
       ELSE IF(i-q >= 1 .AND. i+q > SIZE(var,dim=1))THEN
          DO r=i-q,i
             sum=sum+var(r)
          END DO
          ave(i)=sum/(1*q+1)       
       END IF
    END DO


  END FUNCTION MOVINGAVERAGE

  FUNCTION MOVINGAVERAGE2(var,q) RESULT(ave)
    IMPLICIT NONE
    INTEGER :: q
    REAL(KIND=r8)    :: var(:) 
    REAL(KIND=r8)    :: sum
    REAL(KIND=r8)    :: ave(SIZE(var,dim=1))
    INTEGER :: i
    INTEGER :: r
    !                                                                                     q
    !                                                                                    _____
    !  Y1 + Y2 + ... + Yn     Y2 + Y3 +... + Yn+1     Y3 + Y4 + ... + Yn+2         1     \
    ! -------------------- , --------------------,    --------------------,   ----------  >     x(i+r) 
    !         n                        n                       n                2q + 1   /____
    !                                                                                    r=-q  

    ave=var
    DO i=1,SIZE(var,dim=1)
       sum=0
       IF(i-q >=1 .AND. i+q <= SIZE(var,dim=1))THEN
          DO r=i-q,i+q
             sum=sum+var(r)
          END DO
          ave(i)=sum/(2*q+1)
       END IF
    END DO

  END FUNCTION MOVINGAVERAGE2

  SUBROUTINE RegrLinearMultipla(VAR_Y,VAR_X,V)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: VAR_X(:,:) 
    REAL(KIND=r8), INTENT(IN   ) :: VAR_Y(  :) 
    REAL(KIND=r8), INTENT(OUT  ) :: V    (:)

    REAL(KIND=r8) :: X_Med(SIZE(VAR_X,2))
    REAL(KIND=r8) :: Y_Med
    REAL(KIND=r8) :: SXiXi (SIZE(VAR_X,1),SIZE(VAR_X,2))
    REAL(KIND=r8) :: SXjXj (SIZE(VAR_X,1),SIZE(VAR_X,2))
    REAL(KIND=r8) :: SYkYk (SIZE(VAR_X,1))
    REAL(KIND=r8) :: SXXAUX(SIZE(VAR_X,1),SIZE(VAR_X,2),SIZE(VAR_X,2))
    REAL(KIND=r8) :: SXYAUX(SIZE(VAR_X,1),SIZE(VAR_X,2))
    REAL(KIND=r8) :: SXX   (SIZE(VAR_X,2),SIZE(VAR_X,2))
    REAL(KIND=r8) :: SXY   (SIZE(VAR_X,2))    
    REAL(KIND=r8) :: B     (SIZE(VAR_X,2))   
    REAL(KIND=r8) :: A    
    REAL(KIND=r8) :: TERM ,SS
    REAL(KIND=r8) :: Y(SIZE( VAR_Y,1))   
    REAL (r8), ALLOCATABLE :: SDX(:)
    REAL (r8)              :: SDY
    REAL (r8), ALLOCATABLE :: Co (:,:)

    REAL(KIND=r8), ALLOCATABLE :: Eqn  (:,:) 
    REAL(KIND=r8), ALLOCATABLE :: Soln (:)
    REAL (r8),PARAMETER::Eps=1.E-13_r8


    INTEGER :: i,k,j,n,ErrFlag
    N=SIZE(VAR_X,2)
    ALLOCATE(SDX(1:N),Co(N,N))

    !
    ! Y = A + B1X1 +B2X2 +B3X3 + ... + Bn Xn + E
    ! _   _  _    _  _               _  _    _     
    ! A + B1*X1 + B2*X2 + .....  +   Bn*Xn = Y
    !
    ! A1 + B11*X11 + B21*X21 + .....  +   Bn1*Xn1 = Y1
    ! A2 + B12*X12 + B22*X22 + .....  +   Bn2*Xn2 = Y2
    ! A3 + B13*X13 + B23*X23 + .....  +   Bn3*Xn3 = Y3
    ! A4 + B14*X14 + B24*X24 + .....  +   Bn4*Xn4 = Y4
    !
    X_Med=AVE(VAR_X)
    Y_Med=AVE(VAR_Y)
    SXiXi=Variability(VAR_X)
    SXjXj=Variability(VAR_X)
    SYkYk=Variability(VAR_Y)
    !
    !Variabilidade de Var (Variancia)
    !
    !
    !
    !           --- N=15
    !           \   --        ____ --     --        ____ -- 
    !SXiXj  =    \ | X(i,k) - X(i)   | * | X(j,k) - X(j)   | 
    !            /  --             --     --             --
    !           /
    !           ---k=1
    !   

    DO k=1,SIZE(VAR_X,1)
       DO i=1,SIZE(VAR_X,2)
          DO j=1,SIZE(VAR_X,2)
             SXXAUX(k,i,j) = ( SXiXi(k,i) * SXjXj(k,j) )
          END DO
       END DO
    END DO
    DO k=1,SIZE(VAR_X,1)
       DO i=1,SIZE(VAR_X,2)
          SXYAUX(k,i) = (  SXiXi(k,i) * SYkYk(k) )
       END DO
    END DO

    DO i=1,SIZE(VAR_X,2)
       DO j=1,SIZE(VAR_X,2)
          SXX(i,j) =  SUM(SXXAUX(:,i,j))
       END DO
       SXY      (i) =     SUM(SXYAUX(:,i))      
    END DO

    n=SIZE(VAR_X,2)
    ALLOCATE(Eqn (n,n+1)) 
    ALLOCATE(Soln(n)    ) 

    DO i=1,n
       DO j=1,n
          Eqn(i,j)   = SXX(i,j) 
       END DO
       Eqn  (i,n+1) = SXY (i)
    END DO

    CALL SOLVE(Eqn, Soln, n, ErrFlag) 
    B =Soln
    A=0.0 
    DO j=1,SIZE(VAR_X,2)  
       !Y=A+B*X ==> A=Y-B*X
       A =   A + (Y_Med - B(j)* X_Med(j)) 
    END DO
    A =A / SIZE(VAR_X,2)    

    DO k=1,SIZE(VAR_X,1)
       TERM=0
       DO j=1,SIZE(VAR_X,2)  
          TERM=TERM + (B(j) * VAR_X(k,j))
       END DO
       y(k) = A + TERM
    END DO
    IF(ABS(SUM(VAR_Y-Y))==0.0_r8)THEN
       TERM=  1.0_r8
    ELSE  
       TERM=SUM(VAR_Y-Y)/ABS(SUM(VAR_Y-Y))
    END IF
    SS= TERM * SQRT(SUM ((VAR_Y-Y)**2)/(SIZE(VAR_X,1) - SIZE(VAR_X,2)))

    DO k=1,SIZE(VAR_X,1)
       TERM=0
       DO j=1,SIZE(VAR_X,2)  
          TERM=TERM + (B(j) * VAR_X(k,j))
       END DO
       y(k) = A + TERM + SS
       PRINT*,k,VAR_Y(k),(y(k))
    END DO

    SDX=SD (VAR_X) 
    SDY=SD (VAR_Y)
    Co=Correlation (VAR_X)
    V=0.0
    DO i=1,SIZE(VAR_X,2)  
       IF(SDY /= 0.0_r8) THEN
          V(i)= (B(i)**2) * ((SDX(i))**2/((SDY)**2))
          DO j=1,N
             IF  (i/=j) THEN
                V(i) = V(i)  + Co(i,j)*B(i)*B(j)*((SDX(i)*SDX(j))/(SDY*SDY))
             END IF
          END DO
       END IF
    END DO
    DO i=1,N
       PRINT*,V(i),SUM((V)), Co(1,i)
    END DO
    DEALLOCATE(SDX,Co)
    DEALLOCATE(Eqn   ) 
    DEALLOCATE(Soln  ) 

  END SUBROUTINE RegrLinearMultipla

  SUBROUTINE SOLVE(a, x, n, errflag)
    !
    !Declarations
    !Paste Definition of the below subroutine here 
    !
    !Subroutine to solve a set of simultaneous linear equations in 'n' variables
    !using Gaussian Elimination method
    !Author : Louisda16th a.k.a Ashwith J. Rego
    !Description:
    !This subroutine finds the unknowns in a set of linear equations
    !An example of using Gaussian Elimination is as follows:
    !Consider the equations:
    !              8X2 + 2X3 = -7
    !        3X1 + 5x2 + 2X3 = 8
    !        6X1 + 2X2 + 8X3 = 26
    ! Write the equations in matrix form as         0 8 2 -7
    !                                               3 5 2 8
    !                                               6 2 8 26
    ! Re-order the equations as  6 2 8 26   
    !                            3 5 2 8
    !                            0 8 2 -7
    ! Now Convert to uppper traingular form to get   6 2 8 26
    !                                                0 8 2 -7
    !                                                0 0 -3 -3/2
    ! Finally use back-substitution to solve the equation. For example, in case of x3 you get:
    !                                                          -3X3 = -3/2
    !                                                     =>     X3 = -1/2
    !For a detailed explanation on the algorithm check out http://en.wikipedia.org/wiki/Gaussian_elimination
    !Reference : The program has been written entirely by me. However, the algorithm is from Advanced Engineering Mathematics by Erwin Kreyzing, 8th Edition, Wiley            
    !   
    IMPLICIT NONE
    INTEGER, INTENT(IN) :: n   !Stores the number of unknowns
    INTEGER, INTENT(OUT) :: errflag
    REAL   (KIND=r8), DIMENSION(n,n+1) :: a!An n x n+1 matrix which stores the simultaneous equations
    REAL   (KIND=r8), INTENT(OUT), DIMENSION(n) :: x !Array to store the solutions
    INTEGER :: i,j,k !Counters for Loops
    INTEGER :: largest
    REAL (KIND=r8):: temp, m, sums
    LOGICAL :: FLAG
    REAL (r8),PARAMETER::Eps=1.E-13_r8

    m = 0
    !Solve Using Gaussian Elimination
    FLAG = .FALSE.
    x = 0
    DO k = 1, n-1
       DO j = 1, n
          IF (a(j, 1) /= 0 ) FLAG = .TRUE.
       END DO
       IF (FLAG .EQV. .FALSE.) THEN
          PRINT*,"No Unique Solution"
          errflag = -1
          x = 0
          EXIT
       ELSE
          largest = k
          !Find largest coefficient of first unknown
          DO j = k, n
             IF (ABS(a(j, k)) > ABS(a(largest,k))) largest = j
          END DO
          !Make the equation with largest first coefficient as the first equation
          !Largest coefficient is chosen to prevent round-off errors as far as possible
          DO j = 1, n + 1
             temp = a(k, j)
             a(k,j) = a(largest,j)
             a(largest,j)=temp
          END DO
       ENDIF
       !Convert the input matrix to Upper Traingualar form
       DO j = k+1, n
          IF(a(k,k) == 0.0_r8)THEN
             m = a(j,k)/(a(k,k)+Eps)
          ELSE
             m = a(j,k)/(a(k,k))
          END IF
          DO i = k+1, n+1
             a(j,i) = a(j,i) - m*a(k,i)
          END DO
       END DO
       !No unique solution exists if the last element in the upper triangular matrix is zero
       IF (a(n,n) == 0) THEN
          PRINT*,"No Unique Solution"
          errflag = -1
          x = 0
          EXIT
       ELSE
          !Find xn
          x(n) = a(n,n+1)/a(n,n)
          !Find the remaining unknowns by back-substitution
          DO i = n-1, 1, -1
             sums = 0
             DO j = i+1, n
                sums = sums + a(i,j)*x(j)
             END DO
             IF(a(i,i) == 0.0_r8)THEN
                x(i) = (a(i,n+1) - sums)/(a(i,i)+Eps)
             ELSE
                x(i) = (a(i,n+1) - sums)/(a(i,i))
             END IF
          END DO
       ENDIF
    END DO

  END SUBROUTINE SOLVE

  SUBROUTINE Solve2(VAR_Y,VAR_X,V)

    IMPLICIT NONE
    REAL (r8), INTENT(IN ) :: VAR_X(:,:)
    REAL (r8), INTENT(IN ) :: VAR_Y(:)
    REAL (r8), INTENT(OUT) :: V(:)

    REAL (r8), ALLOCATABLE :: Co(:,:)  
    REAL (r8), ALLOCATABLE :: A(:,:)
    REAL (r8), ALLOCATABLE :: B(:)
    REAL (r8), ALLOCATABLE :: X(:)
    REAL (r8), ALLOCATABLE :: SDX(:)
    REAL (r8), ALLOCATABLE :: SDY(:)

    LOGICAL                :: Singular
    INTEGER                :: M
    INTEGER                :: N
    INTEGER                :: I
    INTEGER                :: J,k

    PRINT *,'Number of equations?'
    !READ  *,N
    N=SIZE(VAR_X,2)
    M=SIZE(VAR_X,1)
    ALLOCATE(A(1:N,1:N),B(1:N),X(1:N),SDX(1:N),SDY(1),Co(N,N))

    DO I=1,N
       PRINT *,'Input elements of row ',I,' of A'
       !READ*,A(I,1:N)
       DO J=1,N
          IF(i==1)THEN
             IF(j==1)THEN
                A(I,J)= 1
             ELSE 
                A(I,J)= SUM(VAR_X(:,J))
             END IF
          ELSE
             IF(j==1)THEN
                A(I,J)= SUM(VAR_X(:,I))
             ELSE
                A(I,J)= SUM(VAR_X(:,I)*VAR_X(:,J))
             END IF
          END IF
       END DO
       PRINT*,'Input element ',I,' of B'
       !READ *,B(I) 
       IF(I==1)THEN
          B(I)= SUM(VAR_Y(:))       
       ELSE
          B(I)= SUM(VAR_Y(:)*VAR_X(:,I))       
       END IF
    END DO
    DO k=1,SIZE(A,2)
       WRITE(0,'(10f18.4)')(A(k,j),j=1,SIZE(A,2))
    END DO

    !
    ! Routine to solve a system A*x=b
    !
    CALL Gaussian_Elimination(A,N,B,X,Singular)
    IF(Singular) THEN
       PRINT*, 'Matrix is singular'
    ELSE
       PRINT*, 'Solution X:'
       PRINT*,X(1:N)
    ENDIF
    SDX=SD (VAR_X) 
    SDY=SD (VAR_Y)
    Co=Correlation (VAR_X)
    V=0.0
    DO i=2,N
       IF(SDY(1) /= 0.0_r8)THEN
          V(i-1)= (X(i)**2) * ((SDX(i))**2/((SDY(1))**2))
          DO j=2,N
             IF  (i/=j) THEN
                V(i-1) = V(i-1)  + Co(i,j)*X(i)*X(j)*((SDX(i)*SDX(j))/(SDY(1)*SDY(1)))
             END IF
          END DO
       END IF
    END DO
    DO i=1,N
       PRINT*,V(i),SUM((V)),Co(1,i)
    END DO
    !WRITE(0,'(8F7.3)')VAR_Y-AVE(VAR_Y)
    !PRINT*,  Media(VAR_X),Variance(VAR_X), Covariance(VAR_X),Correlation(VAR_X)
    !PRINT*, StandardDeviation (VAR_X) 
    DEALLOCATE(A,B,X,SDX,SDY,Co)

  END SUBROUTINE Solve2

  SUBROUTINE Gaussian_Elimination(A,N,B,X,Singular)
    ! Routine to solve a system Ax=b
    ! using Gaussian Elimination
    ! with partial pivoting
    ! The code is based on the Linpack routines
    ! SGEFA and SGESL
    ! and operates on columns rather than rows!
    IMPLICIT NONE
    ! Matrix A and vector B are over-written
    ! Arguments
    INTEGER  , INTENT(IN   ):: N
    REAL (r8), INTENT(INOUT):: A (:,:)
    REAL (r8), INTENT(INOUT):: B (:)
    REAL (r8), INTENT(OUT  ):: X (:)
    LOGICAL    , INTENT(OUT  ):: Singular
    ! Local variables
    INTEGER::I,J,K,Pivot_row
    REAL (r8):: Pivot,Multiplier,Sum,Element
    REAL (r8),PARAMETER::Eps=1.E-13_r8
    !
    ! Work through the matrix column by column
    !
    DO K=1,N-1
       !
       !  Find largest element in column K for pivot
       !
       Pivot_row = MAXVAL( MAXLOC( ABS( A(K:N,K) ) ) ) + K - 1
       !
       ! Test to see if A is singular
       ! if so return to main program
       !
       IF(ABS(A(Pivot_row,K)) <= Eps) THEN
          Singular=.TRUE.
          RETURN
       ELSE
          Singular = .FALSE.
       ENDIF
       !
       ! Exchange elements in column K if largest is
       ! not on the diagonal
       !
       IF(Pivot_row /= K) THEN
          Element=A(Pivot_row,K)
          A(Pivot_Row,K)=A(K,K)
          A(K,K)=Element
          Element=B(Pivot_row)
          B(Pivot_row)=B(K)
          B(K)=Element
       ENDIF
       !
       ! Compute multipliers
       ! elements of column K below diagonal
       ! are set to these multipliers for use
       ! in elimination later on
       !
       A(K+1:N,K) = A(K+1:N,K)/A(K,K)
       !
       ! Row elimination performed by columns for efficiency
       !
       DO J=K+1,N
          Pivot = A(Pivot_row,J)
          IF(Pivot_row /= K) THEN
             !       Swap if pivot row is not K
             A(Pivot_row,J)=A(K,J)
             A(K,J)=Pivot
          ENDIF
          A(K+1:N,J)=A(K+1:N,J)-Pivot* A(K+1:N,K)
       END DO
       !
       ! Apply same operations to B
       !
       B(K+1:N)=B(K+1:N)-A(K+1:N,K)*B(K)
    END DO
    !
    ! Backward substitution
    !
    DO I=N,1,-1
       Sum = 0.0
       DO J= I+1,N
          Sum=Sum+A(I,J)*X(J)
       END DO
       IF(A(I,I) == 0.0_r8)THEN
          X(I)=(B(I)-Sum)/(A(I,I)+Eps)
       ELSE
          X(I)=(B(I)-Sum)/A(I,I)
       END IF
    END DO
  END SUBROUTINE Gaussian_Elimination

  FUNCTION Media1D(VAR_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: VAR_X  (:)
    REAL(KIND=r8) :: Media1D
    INTEGER :: j

    !
    !        ----|
    !       \      
    !        \    Xi
    !        /    
    !       /
    !        ----|   
    !  Med= ---------------------- 
    !                n
    !
    Media1D=0
    Media1D = SUM(VAR_X(:))/SIZE(VAR_X,1)
  END FUNCTION Media1D

  FUNCTION Media2D(VAR_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: VAR_X  (:,:)
    REAL(KIND=r8) :: Media2D(SIZE(VAR_X,2))
    INTEGER :: j

    !
    !        ----|
    !       \      
    !        \    Xi
    !        /    
    !       /
    !        ----|   
    !  Med= ---------------------- 
    !                n
    !
    Media2D=0
    DO j=1,SIZE(VAR_X,2)
       Media2D(j) = SUM(VAR_X(:,j))/SIZE(VAR_X,1)
    END DO
  END FUNCTION Media2D

  FUNCTION Variance (Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN  ) :: Mat_X(:,:)
    REAL(KIND=r8) :: Variance(SIZE(Mat_X,2))
    REAL(KIND=r8) :: X_Med   (SIZE(Mat_X,2))
    INTEGER :: j
    !
    !
    !Variabilidade de Var (Variance)
    !
    !         ---N
    !         \   --    _ -- 2
    !          \ | Xi - X   | 
    !          /  --      --
    !         /
    !         ---i
    !Sxx  = --------------------------------
    !
    !             N-1
    !
    X_Med=AVE(Mat_X)  
    DO j=1,SIZE(Mat_X,2)
       Variance(j)=SUM((Mat_X(:,j)-X_Med(j))**2)/(SIZE(Mat_X,1)-1)
    END DO

  END FUNCTION Variance

  FUNCTION Variability1D(Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: Mat_X(:)

    REAL(KIND=r8) :: X_Med
    REAL(KIND=r8) :: Variability1D(SIZE(Mat_X,1))
    INTEGER :: j
    INTEGER :: i

    X_Med=AVE(Mat_X)  

    !
    !        ----|
    !       \     --       --  
    !  Var=  \   | Xi - Xmed | 
    !        /    --       --  
    !       /
    !        ----|   
    !
    DO i=1,SIZE(Mat_X,1)
       Variability1D(i) = Mat_X(i) - X_Med
    END DO

  END FUNCTION Variability1D


  FUNCTION Variability2D(Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: Mat_X(:,:)

    REAL(KIND=r8) :: X_Med(SIZE(Mat_X,2))
    REAL(KIND=r8) :: Variability2D(SIZE(Mat_X,1),SIZE(Mat_X,2))
    INTEGER :: j
    INTEGER :: i

    X_Med=AVE(Mat_X)  

    !
    !        ----|
    !       \     --       --  
    !  Var=  \   | Xi - Xmed | 
    !        /    --       --  
    !       /
    !        ----|   
    !
    DO j=1,SIZE(Mat_X,2)
       DO i=1,SIZE(Mat_X,1)
          Variability2D(i,j) = Mat_X(i,j) - X_Med(j)
       END DO
    END DO

  END FUNCTION Variability2D

  FUNCTION Covariance(Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: Mat_X(:,:)
    REAL(KIND=r8) :: Covariance(SIZE(Mat_X,2),SIZE(Mat_X,2))

    REAL(KIND=r8) :: X_Med(SIZE(Mat_X,2))
    REAL(KIND=r8) :: X_Variance (SIZE(Mat_X,1),SIZE(Mat_X,2))
    REAL(KIND=r8) :: X_Trasposta(SIZE(Mat_X,2),SIZE(Mat_X,1))
    INTEGER :: i,j,k
    !
    !        ----|
    !       \     --       --     --       --
    !        \   | Xi - Xmed | * | Yi - Ymed |
    !        /    --       --     --       --
    !       /
    !        ----|   
    !  Cov= --------------------------------------- 
    !                n-1  

    X_Med=AVE(Mat_X)

    !
    X_Variance=Variability(Mat_X)

    Covariance=0
    DO k=1,SIZE(Mat_X,2)
       DO j=1,SIZE(Mat_X,2)
          DO i=1,SIZE(Mat_X,1)
             Covariance(j,k) = Covariance(j,k) + X_Variance(i,j) * X_Variance (i,k)
          END DO
          Covariance(j,k) = Covariance(j,k) /(SIZE(Mat_X,1) -1.0)
       END DO
    END DO
    !DO k=1,SIZE(Mat_X,2)
    !   WRITE(0,'(3f12.4)')(Covariance(k,j),j=1,SIZE(Mat_X,2))
    !END DO
  END FUNCTION Covariance

  FUNCTION StandardDeviation1D (Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN  ) :: Mat_X(:)
    REAL(KIND=r8) :: X_Med             
    REAL(KIND=r8) :: StandardDeviation1D
    INTEGER :: j

    X_Med=AVE(Mat_X)
    !
    !
    !Variabilidade de Var (Variance)
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i                 |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    StandardDeviation1D=SQRT(SUM((Mat_X(:)-X_Med)**2)/(SIZE(Mat_X,1)-1))

  END FUNCTION StandardDeviation1D

  FUNCTION StandardDeviation2D (Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN  ) :: Mat_X(:,:)
    REAL(KIND=r8) :: X_Med             (SIZE(Mat_X,2))
    REAL(KIND=r8) :: StandardDeviation2D(SIZE(Mat_X,2))
    INTEGER :: j

    X_Med=AVE(Mat_X)
    !
    !
    !Variabilidade de Var (Variance)
    !                               
    !       --                    -- 0.5
    !      |                        |
    !      |   ---N                 |
    !      |   \   --    _ -- 2     |
    !      |    \ | Xi - X   |      |
    !      |    /  --      --       |
    !      |   /                    |
    !      |   ---i                 |
    ! SD = |------------------------|
    !      |         N - 1          |
    !      |                        |
    !       --                    --
    !
    DO j=1,SIZE(Mat_X,2)
       StandardDeviation2D(j)=SQRT(SUM((Mat_X(:,j)-X_Med(j))**2)/(SIZE(Mat_X,1)-1))
    END DO

  END FUNCTION StandardDeviation2D

  FUNCTION Correlation(Mat_X)
    IMPLICIT NONE
    REAL(KIND=r8), INTENT(IN   ) :: Mat_X(:,:)
    REAL(KIND=r8) :: Correlation(SIZE(Mat_X,2),SIZE(Mat_X,2))

    REAL(KIND=r8) :: X_Med(SIZE (Mat_X,2))
    REAL(KIND=r8) :: X_Variance (SIZE(Mat_X,1),SIZE(Mat_X,2))
    REAL(KIND=r8) :: X_Trasposta(SIZE(Mat_X,2),SIZE(Mat_X,1))
    REAL(KIND=r8) :: X_SXX      (SIZE(Mat_X,2))
    REAL(KIND=r8) :: X_SDX      (SIZE(Mat_X,2))
    REAL (r8),PARAMETER::Eps=1.E-13_r8

    INTEGER :: i,j,k
    !
    !        ----|
    !       \     --       --     --       --
    !        \   | Xi - Xmed | * | Yi - Ymed |
    !        /    --       --     --       --
    !       /
    !        ----|   
    !  Cov= ---------------------- 
    !                     ---    ---
    !                    |         |
    !                n-1 | (Sx*Sy) |
    !                    |         |
    !                     ----  ---
    !
    X_Med      = AVE(Mat_X)

    X_SXX      = Variance (Mat_X)

    X_Variance = Variability(Mat_X)

    X_SDX      = MAX(SD (Mat_X),Eps) 

    X_Trasposta= TRANSPOSE(X_Variance)

    Correlation=0  

    DO k=1,SIZE(Mat_X,2)
       DO j=1,SIZE(Mat_X,2)
          DO i=1,SIZE(Mat_X,1)
             Correlation(j,k) = Correlation(j,k) + X_Variance(i,j) * X_Trasposta (k,i)
          END DO
          Correlation(j,k) = Correlation(j,k) /((SIZE(Mat_X,1) - 1.0)*X_SDX(k)*X_SDX(j) )
       END DO
    END DO

    !DO k=1,SIZE(Mat_X,2)
    !   WRITE(0,'(10f12.4)')(Correlation(k,j),j=1,SIZE(Mat_X,2))
    !END DO

  END FUNCTION Correlation

END MODULE Class_Statistic

