PROGRAM Main
  USE Class_Wavelet
  USE Class_Statistic
  IMPLICIT NONE
  INTEGER         , PARAMETER   :: iMax         = 1            ! Number of point on x grid
  INTEGER         , PARAMETER   :: jMax         = 1            ! Number of point on xygrid
  INTEGER         , PARAMETER   :: nVar         = 1            ! Number of variables
  REAL(KIND=r8)                 :: undef=1e+20                 ! undef value

  REAL(KIND=r8)   , ALLOCATABLE :: sst           (:,:,:)     ! dados para calcular a wavelets

  REAL(KIND=r8)   , ALLOCATABLE :: sst1ave        (:)
  REAL(KIND=r8)   , ALLOCATABLE :: sst1d         (:)
  REAL(KIND=r8)   , ALLOCATABLE :: Recon_sst     (:,:,:)
  REAL(KIND=r8)   , ALLOCATABLE :: Recon_sst2    (:,:,:)
  REAL(KIND=r8)   , ALLOCATABLE :: Filter_sstENSO(:,:,:)
  REAL(KIND=r8)   , ALLOCATABLE :: SCALE         (:,:,:)  ! the wavelet scale used to construct the daughter.
  COMPLEX(KIND=r8), ALLOCATABLE :: wave          (:,:,:,:)!  wave = 2D array of the real & imaginary parts
  !  of the wavelet transform, versus time & scale.
  !  CABS(wave) gives the WAVELET amplitude,
  !  ATAN2(AIMAG(wave),DBLE(wave)) gives WAVELET phase.
  !  The WAVELET power spectrum is CABS(wave)**2.
  REAL(KIND=r8)   , ALLOCATABLE :: period         (:,:,:) ! the "Fourier" periods (in time units) corresponding to "scale".
  REAL(KIND=r8)   , ALLOCATABLE :: Cdelta         (:,:)   ! the constant "Cdelta" for the mother wavelet (Table 2).
  REAL(KIND=r8)   , ALLOCATABLE :: coi            (:,:,:) ! the e-folding factor used for the cone of influence.
  REAL(KIND=r8)   , ALLOCATABLE :: signif         (:,:,:) ! significance levels vs scale.
  ! IF SIGTEST=2, then signif(1) = the significance level
  !  signif(2...jtot) = 0.0
  REAL(KIND=r8)   , ALLOCATABLE :: global_ws      (:,:,:)
  REAL(KIND=r8)   , ALLOCATABLE :: global_signif  (:,:,:)
  REAL(KIND=r8)   , ALLOCATABLE :: P_theor        (:,:,:) ! theoretical red-noise spectrum vs scale.
  ! IF SIGTEST=2, then fft_theor(1) = the average spectrum from S1-->S2
  ! fft_theor(2...jtot) = 0.0
  REAL(KIND=r8)   , ALLOCATABLE :: fft_theor      (:,:,:) ! theoretical red-noise spectrum vs scale.
  ! IF SIGTEST=2, then fft_theor(1) = the average spectrum from S1-->S2
  ! fft_theor(2...jtot) = 0.0
  REAL(KIND=r8)   , ALLOCATABLE :: savg_dof       (:,:,:)!degrees-of-freedom that were actually used.
  REAL(KIND=r8)   , ALLOCATABLE :: savg_signif    (:,:,:)
  REAL(KIND=r8)    :: DIURNALCYCLE(24)
  INTEGER :: i
  INTEGER :: j,k
  INTEGER :: lrec
  INTEGER :: irec
  INTEGER :: iano
  INTEGER :: itime
  INTEGER :: imes
  INTEGER :: ivar,jtotal,ncount
  CHARACTER(LEN=20) :: data1
  CHARACTER(LEN=20) :: char
  REAL(KIND=r8)    :: p0,ss0,p1,ss1,sjtotal,spec,per,pjtot
  CHARACTER (LEN=20),ALLOCATABLE :: cdate(:)
  CHARACTER (LEN=20),ALLOCATABLE :: ctime(:)
  CHARACTER (LEN=2) :: clevels

  CALL Init()  
  CALL Run()
  CALL Finalize()

CONTAINS
  SUBROUTINE Init()
    IMPLICIT NONE
    REAL(KIND=r8)       :: u0,u1,u2,u3,u4,u5,u6,u7,u8,u9
    CALL Init_Wavelet()
    PRINT*,'read',jtot
    !
    !** read in the NINO3 SST data
    !   The seasonal means for the entire record have been removed to define an anomaly time series
    ALLOCATE(cdate         (          nTime          ))
    ALLOCATE(ctime         (          nTime          ))
    ALLOCATE(sst1d         (          nTime          ))
    ALLOCATE(sst1ave       (          nTime          ))
    ALLOCATE(sst           (iMax,jMax,nTime     ))

    ALLOCATE(Recon_sst     (iMax,jMax,nTime     ))
    ALLOCATE(Recon_sst2    (iMax,jMax,nTime     ))
    ALLOCATE(Filter_sstENSO(iMax,jMax,nTime     ))
    ALLOCATE(SCALE         (iMax,jMax,jtot))
    ALLOCATE(period        (iMax,jMax,jtot))
    ALLOCATE(Cdelta        (iMax,jMax))
    ALLOCATE(coi           (iMax,jMax,nTime))
    ALLOCATE(signif        (iMax,jMax,jtot))
    ALLOCATE(global_ws     (iMax,jMax,jtot))
    ALLOCATE(global_signif (iMax,jMax,jtot))
    ALLOCATE(fft_theor     (iMax,jMax,jtot))
    ALLOCATE(P_theor       (iMax,jMax,jtot))
    ALLOCATE(savg_dof      (iMax,jMax,jtot))
    ALLOCATE(savg_signif   (iMax,jMax,jtot))
    ALLOCATE(wave          (iMax,jMax,nTime,jtot))

!"2014-07-25"   "19:00:00.00"   0.08100  -0.10150  -0.17025  -0.05075   0.01800   0.06500   0.39900   0.45825   0.70650   0.86425 

    OPEN(UNIT=11,FILE=TRIM(FileNameIn)//'data.txt',STATUS='old')
    READ(11,"(1x)")
    DO k=1,nTime
       READ(11,*)cdate(k),ctime(k), u0,u1,u2,u3,u4,u5,u6,u7,u8,u9
       IF(zlevel == 0 )THEN
          sst1d(k)=u0
       ELSE IF(zlevel == 1 )THEN
          sst1d(k)=u1
       ELSE IF(zlevel == 2 )THEN
          sst1d(k)=u2
       ELSE IF(zlevel == 3 )THEN
          sst1d(k)=u3
       ELSE IF(zlevel == 4 )THEN
          sst1d(k)=u4
       ELSE IF(zlevel == 5 )THEN
          sst1d(k)=u5
       ELSE IF(zlevel == 6 )THEN
          sst1d(k)=u6
       ELSE IF(zlevel == 7 )THEN
          sst1d(k)=u7
       ELSE IF(zlevel == 8 )THEN
          sst1d(k)=u8
       ELSE IF(zlevel == 9 )THEN
          sst1d(k)=u9
       ELSE 
          STOP 'no data'
       END IF  
    END DO
    CLOSE(11)

    
    IF(OptFilter == 0)THEN
       sst1ave=0.0
       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) =   sst1d(1:nTime)-sst1ave(1:nTime)
             END DO
          END DO
       !END DO
    ELSE IF(OptFilter == 1)THEN
       sst1ave=MOVINGAVERAGE(sst1d,qint)
       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) =   sst1d(1:nTime)-sst1ave(1:nTime)
             END DO
          END DO
       !END DO
    ELSE IF(OptFilter == 2)THEN
       sst1ave=NormalizedSerieTemporal(sst1d)
       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) =   sst1ave(1:nTime)
             END DO
          END DO
       !END DO
       sst1ave=0.0  
    ELSE IF(OptFilter == 3)THEN
       sst1ave=RemoveRegrLinear(sst1d)

       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) = sst1d(1:nTime) -  sst1ave(1:nTime)
             END DO
          END DO
       !END DO

    ELSE IF(OptFilter == 4)THEN
       DIURNALCYCLE=CycleDiurnalAverage(sst1d,FirstYear,LastYear,&
            FirstMonth,LastMonth,&
            FirstDay,LastDay)

       sst1ave=RemoveCycleDiurnalAverage(sst1d,FirstYear,LastYear,&
            FirstMonth,LastMonth,&
            FirstDay,LastDay)
       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) =   sst1ave(1:nTime)
             END DO
          END DO
       !END DO
       sst1ave=ComposeCycleDiurnalAverage(sst1d,DIURNALCYCLE,FirstYear,LastYear,&
            FirstMonth,LastMonth,&
            FirstDay,LastDay)
    ELSE IF(OptFilter == 5)THEN    
       sst1ave=Adimensionalizacao_Torrence_Compo(sst1d,lag1)
       !DO ivar=1,nVar
          DO j=1,jMax
             DO i=1,iMax
                sst(i,j,1:nTime) =   sst1ave(1:nTime)
             END DO
          END DO
       !END DO       
       sst1ave=0.0  
    ELSE

       STOP 'OptFilter error'
    END IF

    PRINT'(/,"sst(1)=",F6.2,"  sst(n) = ",F6.2,/)',sst(1,1,1),sst(1,1,nTime)
! MENOR ESCALA POSSIVEL

    p0 = 2.0*dt
    ss0 = p2s(p0)
    PRINT *, 'MENOR ESCALA POSSIVEL = ',ss0
! MAIOR ESCALA POSSIVEL
      p1 = REAL(nTime)*dt
      ss1 = p2s(p1)
      PRINT *, 'MAIOR ESCALA POSSIVEL = ',ss1
! OBTENCAO DE JTOT (# DE ESCALAS) E NCOUNT (# DE ESCALAS NO PERIODO DE INTERESSE)
      PRINT '(a6,2x,a8,2x,a5,2x,a5)', 'dj', 'Pmax', 'jtot', 'N' !Pmax = p(jtot), N = ncount
      DO i = 0, 19
         dj = 0.5 - REAL(i)*0.025
         jtotal = int(1.0/dj) * int ( log(ss1/ss0) / log(2.0_r8) ) !Eq. 10
         sjtotal = s0 * 2.0**(jtotal*dj)                          !Eq. 9
         pjtot = p2s(sjtotal)
         ncount = 0
         DO j = 1, jtot
            spec = ss0 * 2.0**(j*dj) !Eq. 9
            per = s2p(spec)
            IF ((per.ge.scalemin).and.(per.le.scalemax)) ncount = ncount + 1
         END DO
         PRINT '(f15.3,2x,f15.1,2x,i5,2x,i5)', dj, pjtot, jtotal, ncount
      END DO
 
 
  END SUBROUTINE Init

  SUBROUTINE Run()
    IMPLICIT NONE
    INTEGER :: i
    INTEGER :: ii
    INTEGER :: j
    INTEGER :: jj
    INTEGER :: k
    INTEGER :: ivar
    INTEGER :: lrec
    INTEGER :: irec
    REAL(KIND=r8)               :: undef=1e+20
    INTEGER, PARAMETER          :: nvars=11
    REAL(KIND=r4)               :: aux           (iMax,jMax)
    REAL(KIND=r8)       :: u0,u1,u2,u3,u4,u5,u6,u7,u8,u9
    CHARACTER(LEN=1)::asp='"'
    !REAL(KIND=r8)              :: ylevs(ydim)=0.00_r8
    REAL(KIND=r8)               :: xdim0      =   0.00_r8
    REAL(KIND=r8)               :: ydim0      = -90.00_r8
    REAL(KIND=r8)               :: xdimdelta  = 1
    REAL(KIND=r8)               :: ydimdelta  = 1
    u0=0.0;u1=0.0;u2=0.0;u3=0.0;u4=0.0;u5=0.0;u6=0.0;u7=0.0;u8=0.0;u9=0.0;
    !DO ivar=1,nVar
       DO j=1,jMax
          DO i=1,iMax
             IF(ANY(sst(i,j,:) /= undef))THEN
                CALL Run_Wavelet(&
                     sst           (i,j,1:nTime)       , &
                     Recon_sst     (i,j,1:nTime)       , &
                     Recon_sst2    (i,j,1:nTime)       , &
                     Filter_sstENSO(i,j,1:nTime)       , &
                     wave          (i,j,1:nTime,1:jtot), &
                     SCALE         (i,j,1:jtot)        , &
                     period        (i,j,1:jtot)        , &
                     Cdelta        (i,j)               , &
                     coi           (i,j,1:nTime)       , &
                     signif        (i,j,1:jtot)        , &
                     global_ws     (i,j,1:jtot)        , &
                     global_signif (i,j,1:jtot)        , &
                     fft_theor     (i,j,1:jtot)        , &
                     P_theor       (i,j,1:jtot)        , &
                     savg_dof      (i,j,1:jtot)        , &
                     savg_signif   (i,j,1:jtot)          &
                     )

             END IF
          END DO
       END DO
  !  END DO
  
    WRITE(clevels(1:2),'(I2.2)')zlevel
    OPEN(UNIT=110,FILE='../dataout/filter_'//TRIM(clevels)//'.dat',STATUS='unknown',ACTION='WRITE')
    WRITE(110,'(A)')'asp,cdate(k),asp,ctime(k),asp, u0,u1,u2,u3,u4,u5,u6,u7,u8,u9'
    DO k=1,nTime
       u9=REAL(sst(1,1,k)+sst1ave(k),KIND=r4) - REAL(Recon_sst2(1,1,k)+sst1ave(k),KIND=r4)
       WRITE(110,'(A1,A10,A1,A1,A1,A11,A1,10e10.2)')asp,cdate(k),asp,' ',asp,ctime(k),asp, u0,u1,u2,u3,u4,u5,u6,u7,u8,u9
    END DO
    CLOSE(110,STATUS='KEEP') 
    !WRITE(*,*)savg_signif(1,1,1)
    INQUIRE(IOLENGTH=lrec) aux(:,:)
    OPEN(100,FILE=TRIM(FileNameOut)//'data_filter_'//TRIM(clevels)//'.bin',FORM='UNFORMATTED',ACCESS='DIRECT',RECL=lrec,STATUS='UNKNOWN',ACTION='WRITE')
    irec=0
    DO i=1,nTime
!       DO ivar=1,nVar

          ! input time series

          irec=irec+1
          WRITE(100,rec=irec) REAL(sst(:,:,i)+sst1ave(i),KIND=r4)

          ! reconstructed time series

          irec=irec+1
          WRITE(100,rec=irec) REAL(Recon_sst(:,:,i)+sst1ave(i),KIND=r4)

          !  filtered time series 

          irec=irec+1
          WRITE(100,rec=irec) REAL(Recon_sst2(:,:,i)+sst1ave(i),KIND=r4)

          !  filtered time series in windows

          irec=irec+1
          WRITE(100,rec=irec) REAL(sst(:,:,i)+sst1ave(i),KIND=r4) - REAL(Recon_sst2(:,:,i)+sst1ave(i),KIND=r4)



          ! scale-averaged wavelet power and significance


          irec=irec+1
          WRITE(100,rec=irec) REAL( savg_signif  (:,:,1),KIND=r4)

          ! amplitude

          DO j=jtot,1,-1
             irec=irec+1
             WRITE(100,rec=irec)REAL(ABS(wave(:,:,i,j)*CONJG(wave(:,:,i,j))),KIND=r4)
          END DO

          ! real part

          DO j = jtot, 1, -1
             irec = irec + 1
             WRITE(100,rec=irec)REAL(ABS(wave(:,:,i,j))**2,KIND=r4)
          END DO

          ! local significance

          DO j=jtot,1,-1
             irec=irec+1
             WRITE(100,rec=irec) REAL(signif(:,:,j),KIND=r4)
          END DO

          !  coi mask

          DO j=jtot,1,-1
             aux=0.0_r4
             DO jj=1,jMax
                DO ii=1,iMax
                   IF (period        (ii,jj,j) <= coi(ii,jj,i)) THEN
                      aux(ii,jj) = 1.0
                   ELSE
                      aux(ii,jj) = -1.0
                   END IF
                END DO
             END DO
             irec = irec+1
             WRITE(100,rec=irec) REAL( aux,KIND=r4)
          END DO

          ! global wavelet power

          DO j=jtot,1,-1
             irec=irec+1
             WRITE(100,rec=irec) REAL(global_ws(:,:,j),KIND=r4)
          END DO

          ! global significance

          DO j=jtot,1,-1
             irec=irec+1
             WRITE(100,rec=irec) REAL(global_signif(:,:,j),KIND=r4)
          END DO


!       END DO
    END DO

    OPEN(101,FILE=TRIM(FileNameOut)//'data_filter_'//TRIM(clevels)//'.ctl',FORM='FORMATTED',ACCESS='SEQUENTIAL',STATUS='UNKNOWN',ACTION='WRITE')
    WRITE(101,'(A8,A37               )')'dset    ','^data_filter_'//TRIM(clevels)//'.bin'
    WRITE(101,'(A8,A21               )')'title   ','file created by grads'
    WRITE(101,'(A8,A15                )')'options ',' '
    WRITE(101,'(A8,e15.6             )')'undef   ',undef
    WRITE(101,'(A8,i5,A11,F15.6,F15.6)')'XDEF    ',iMax,' LINEAR    ',xdim0 ,xdimdelta
    WRITE(101,'(A8,i5,A11,F15.6,F15.6)')'YDEF    ',jMax,' LINEAR    ',ydim0 ,ydimdelta
    WRITE(101,'(A8,i5,A11            )')'ZDEF    ',jtot,' LEVELS    '
    WRITE(101,'(10F15.6)')(period(1,1,j),j=jtot,1,-1)
    WRITE(101,'(A8,i15,A11,A20,A5     )')'TDEF    ',nTime   ,' LINEAR    ',TRIM(tctl),TRIM(dectl)
    WRITE(101,'(A8,i5                )')'vars    ',nvars*nvar 
    DO ivar=1,nvar
       IF(ivar == 1)THEN
          WRITE(101,'(A10,A5,A5,A51        )')'varori    ','  0  ','   99',' Original time series (unit)                       '
          WRITE(101,'(A10,A5,A5,A51        )')'varrec    ','  0  ','   99',' Reconstructed time series (unit)                  '
          WRITE(101,'(A10,A5,A5,A51        )')'varfil    ','  0  ','   99',' Filtered time series                  ( unit)     '
          WRITE(101,'(A10,A5,A5,A51        )')'datafil   ','  0  ','   99',' Filtered time series scalemin-scalemax( unit)     '
          WRITE(101,'(A10,A5,A5,A51        )')'sawpsig   ','  0  ','   99',' SAWP significance (unit^2)                        '
          WRITE(101,'(A10,i5,A5,A51        )')'amplitude ',jtot   ,'   99',' Amplitude (unit)                                  '
          WRITE(101,'(A10,i5,A5,A51        )')'real      ',jtot   ,'   99',' Real part (unit)                                  '
          WRITE(101,'(A10,i5,A5,A51        )')'slocal    ',jtot   ,'   99',' Local significance (unit^2)                       '
          WRITE(101,'(A10,i5,A5,A51        )')'mask      ',jtot   ,'   99',' Coi mask (-1 or +1)                               '
          WRITE(101,'(A10,i5,A5,A51        )')'global    ',jtot   ,'   99',' Global wavelet spectrum (unit^2)                  '
          WRITE(101,'(A10,i5,A5,A51        )')'sglobal   ',jtot   ,'   99',' Global significance (unit^2)                      '
       ELSE IF(ivar == 2)THEN
          WRITE(101,'(A10,A5,A5,A51        )')'varori2   ','  0  ','   99',' Original time series (unit)                       '
          WRITE(101,'(A10,A5,A5,A51        )')'varrec2   ','  0  ','   99',' Reconstructed time series (unit)                  '
          WRITE(101,'(A10,A5,A5,A51        )')'varfil2   ','  0  ','   99',' Filtered time series scalemin-scalemax( unit)     '
          WRITE(101,'(A10,A5,A5,A51        )')'datafil2   ','  0  ','   99',' Filtered time series scalemin-scalemax( unit)     '
          WRITE(101,'(A10,A5,A5,A51        )')'sawpsig2  ','  0  ','   99',' SAWP significance (unit^2)                        '
          WRITE(101,'(A10,i5,A5,A51        )')'amplitude2',jtot   ,'   99',' Amplitude (unit)                                  '
          WRITE(101,'(A10,i5,A5,A51        )')'real2     ',jtot   ,'   99',' Real part (unit)                                  '
          WRITE(101,'(A10,i5,A5,A51        )')'slocal2   ',jtot   ,'   99',' Local significance (unit^2)                       '
          WRITE(101,'(A10,i5,A5,A51        )')'mask2     ',jtot   ,'   99',' Coi mask (-1 or +1)                               '
          WRITE(101,'(A10,i5,A5,A51        )')'global2   ',jtot   ,'   99',' Global wavelet spectrum (unit^2)                  '
          WRITE(101,'(A10,i5,A5,A51        )')'sglobal2  ',jtot   ,'   99',' Global significance (unit^2)                      '
       END IF
    END DO
    WRITE(101,'(A7                         )')'endvars'


  END SUBROUTINE Run

!
! ----------------------------------------------------------------------
!
   FUNCTION p2s (period) !Morlet
      REAL(KIND=r8)  ,INTENT(IN) :: period
      REAL(KIND=r8)  , PARAMETER :: omega0=6.0
      REAL(KIND=r8)  , PARAMETER ::  pi =3.14159
      REAL(KIND=r8)  ::   factor,p2s
      factor = 4.0*pi / ( omega0 + sqrt(2.0 + omega0**2) )
      p2s = period/factor
    RETURN
  END FUNCTION p2s 

  FUNCTION s2p (scale) !Morlet
      REAL(KIND=r8)  ,INTENT(IN) :: scale
      REAL(KIND=r8)  , PARAMETER :: omega0=6.0
      REAL(KIND=r8)  , PARAMETER ::  pi =3.14159
      REAL(KIND=r8)   :: factor,s2p
      factor = 4.0*pi / ( omega0 + sqrt(2.0 + omega0**2) )
      s2p = scale*factor
      RETURN
  END FUNCTION s2p 



  SUBROUTINE Finalize()
    IMPLICIT NONE
    DEALLOCATE(sst1d        )
    DEALLOCATE(sst1ave      )
    DEALLOCATE(sst          )
    DEALLOCATE(Recon_sst    )
    DEALLOCATE(Recon_sst2   )
    DEALLOCATE(Filter_sstENSO)
    DEALLOCATE(wave         )
    DEALLOCATE(SCALE        )
    DEALLOCATE(period       )
    DEALLOCATE(Cdelta       )
    DEALLOCATE(coi          )
    DEALLOCATE(signif       )
    DEALLOCATE(global_ws    )
    DEALLOCATE(global_signif)
    DEALLOCATE(fft_theor    )
    DEALLOCATE(P_theor      )
    DEALLOCATE(savg_dof     )
    DEALLOCATE(savg_signif  )
    PRINT*,'Finalize'
  END SUBROUTINE Finalize

END PROGRAM Main

