MODULE Class_Wavelet
  IMPLICIT NONE
  ! Selecting Kinds
  INTEGER, PARAMETER :: r4 = SELECTED_REAL_KIND(6)  ! Kind for 32-bits Real Numbers
  INTEGER, PARAMETER :: i4 = SELECTED_INT_KIND(9)   ! Kind for 32-bits Integer Numbers
  INTEGER, PARAMETER :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
  INTEGER, PARAMETER :: i8 = SELECTED_INT_KIND(14)  ! Kind for 64-bits Integer Numbers
  INTEGER, PARAMETER :: r16 = SELECTED_REAL_KIND(31)! Kind for 128-bits Real Numbers

  ! these parameters depend on the particular time series
  INTEGER          :: zlevel   =0  
  INTEGER          :: nTime    =504
  INTEGER          :: subscale = 4       ! a escala  anual e tem-se 4 medidas por ano media sasonal 3 meses
  REAL(KIND=r8)    :: dt       = 0.25_r8 !amount of time between each Y value, i.e. the sampling time.
  INTEGER          :: mother   = 0       !An integer giving the mother wavelet to use.
                                         !                   0='Morlet'
                                         !                   1='Paul'
                                         !                   2='DOG' (derivative of Gaussian)
 CHARACTER(LEN=5)  :: cscalemin
 CHARACTER(LEN=5)  :: cscalemax
  REAL(KIND=r8)    :: scalemin = 2.0_r8!degrees-of-freedom that were actually used.
  REAL(KIND=r8)    :: scalemax = 7.9_r8!degrees-of-freedom that were actually used.
  REAL(KIND=r8)    :: lag1     = 0.720_r8  ! lag1 [REAL(KIND=r8)   ] = lag 1 Autocorrelation, used for signif levels.
                                             !  Default is 0.0, which corresponds to white-noise.
  REAL(KIND=r8)    :: siglvl   = 0.050_r8  ! significance level to use. Default is 0.05 (the "5%" level)


  INTEGER          :: jtot     = 0       ! 11*subscale    !the # of scales.
  REAL(KIND=r8)    :: s0       = 0       ! dt the smallest scale of the wavelet.
  REAL(KIND=r8)    :: dj       = 0       !1.0_r8/subscale!the spacing between discrete scales.
  ! Note: for accurate reconstruction and wavelet-derived variance
  !     do not pad with zeroes, set s0=dt (for Paul set s0=dt/4), and use
  !     a large "jtot" (even though the extra scales will be within
  !     the cone of influence).
  !     For plotting purposes, it is only necessary to use
  !     s0=2dt (for Morlet) and "jtot" from Eqn(10) Torrence&Compo(1998).

  REAL(KIND=r8)    :: param =6            !mother wavelet parameter. If <0 then default is used.
                                        !  For 'Morlet' this is k0 (wavenumber), default is 6.
                                        !  For 'Paul' this is m (order), default is 4.
                                        !  For 'DOG' this is m (m-th derivative), default is 2.
  INTEGER          :: npad=0             ! this is for no padding with zeroes
                                        ! the total number of points (including padding) to
                                        ! use for the wavelet transform. Typically this is some
                                        ! power of 2. It must be greater or equal to "n".
                                        ! If npad>n, then zeroes are padded onto the end
                                        ! of the time series.
 INTEGER          :: LengthTimeSeries   !  Lneght of the time series
 
 INTEGER          :: ibase2

 INTEGER          :: isigtest=0          !  isigtest [INT] = 0, 1, or 2.
                                         !            If 0, then just do a regular chi-square test,
                                         !                i.e. Eqn (18) from Torrence & Compo.
                                         !            If 1, then do a "time-average" test, i.e. Eqn (23).
                                         !                In this case, DOF(j) should be set to NA, the number
                                         !                of local wavelet spectra that were averaged together
                                         !                at each scale. For the Global Wavelet Spectrum,
                                         !                this would be dof(j)=N-scale(j),
                                         !                where N is the number of points in your time series.
                                         !            If 2, then do a "scale-average" test, i.e. Eqns (25)-(28).
                                         !                In this case, "dof(1)" and "dof(2)" should be set to the
                                         !                smallest (S1) and largest (S2) scales that were averaged
                                         !                together, respectively.
                                         !                e.g. if you scale-averaged scales between 2 and 8,
                                         !                     then dof(1)=2.0 and dof(2)=8.0
 
 REAL(KIND=r8)      :: pi
 INTEGER            :: OptFilter=1
 INTEGER            :: FirstYear,LastYear
 INTEGER            :: FirstMonth,LastMonth
 INTEGER            :: FirstDay,LastDay
 INTEGER            :: FirstHour
 INTEGER            :: LastHour 
 INTEGER            :: FirstMin 
 INTEGER            :: LastMin  
 INTEGER            :: FirstSeg 
 INTEGER            :: LastSeg  
 INTEGER            :: qint
 CHARACTER(LEN=15)  :: tctl 
 CHARACTER(LEN=15)  :: dectl
 CHARACTER(LEN=500) :: FileNameIn
 CHARACTER(LEN=500) :: FileNameOut
 CHARACTER(LEN=500) :: FileFilterNameOut
CONTAINS

SUBROUTINE Init_Wavelet()
  IMPLICIT NONE
  INTEGER                 :: n, nyear2seg,nmon2seg,nday2seg
  INTEGER                :: ierr,lastdaymon
  
    NAMELIST /WAVELETS_INPUT/FirstYear,LastYear,FirstMonth,LastMonth,FirstDay,LastDay,&
                            FirstHour,LastHour ,FirstMin ,LastMin  ,FirstSeg ,LastSeg,&  
                            nTime,zlevel,subscale,dt,mother ,scalemin,scalemax,lag1,siglvl ,param,npad,&
                            OptFilter,qint,tctl ,dectl,FileNameIn,FileNameOut,FileFilterNameOut
    lastdaymon=0

    OPEN(UNIT=15,FILE='Wavelets.Namelist',STATUS='old')

    READ (15,WAVELETS_INPUT, iostat=ierr)
    IF (ierr == 0) THEN
      WRITE(0,'(A)') ' read namelist MODEL_RES from file WAVELETS_INPUT.namelist'
    ELSE
      WRITE(0,'(A)') ' error read namelist MODEL_RES from file WAVELETS_INPUT.namelist'   
    END IF
    WRITE(cscalemin,'(I4.4)')INT(scalemin)
    WRITE(cscalemax,'(I4.4)')INT(scalemax)

    nyear2seg = (LastYear-FirstYear)*365*24*60*60
      IF(nyear2seg == 0)THEN 
         nmon2seg = LastMonth-FirstMonth
      ELSE 
         nmon2seg = nyear2seg*12 + (13-FirstMonth) + (0+LastMonth)
      END IF
      
      nday2seg=lastdaymon-LastDay-FirstDay
      
      s0  = 2*dt
      dj  = 1.0_r8/subscale
      LengthTimeSeries=nTime
      n= nTime
      pi     = 4.0_r8*ATAN(1.0_r8)
      ibase2 = NINT(LOG(DBLE(LengthTimeSeries))/LOG(2.0_r8))+1
      !      jtot= subscale!21*subscale
      jtot=1+(LOG(n * dt/s0))/dj     
      WRITE(0,*) ' subscale = ',subscale,' jtot = ',jtot
     
!     Pad with zeroes
! ---------------
!
      IF (npad == 0) THEN
         ibase2 = NINT(LOG(DBLE(LengthTimeSeries))/LOG(2.0_r8))+1
         npad   = INT(2.D0**ibase2)
         WRITE(0,*)  'Padding: ibase2 = ',ibase2,' npad = ',npad,' n = ',LengthTimeSeries
      ELSE
         npad = LengthTimeSeries       
         WRITE(0,*) 'No Padding: npad = ',npad,' n = ',n
      END IF
      WRITE(0,*) '-----'
      
      !
      !      npad = n  ! this is for no padding with zeroes
      !
      !*************************************************** Wavelet transform
      !
      !** let the WAVELET subroutine choose the defaults for these:
      !
      !mother = 0
      !param  = 6.0_r8
      !      s0=dt
      !      (for Paul set s0=dt/4)
      ! Note: for accurate reconstruction and wavelet-derived variance
      !     do not pad with zeroes, set s0=dt (for Paul set s0=dt/4), and use
      !     a large "jtot" (even though the extra scales will be within
      !     the cone of influence).
      !     For plotting purposes, it is only necessary to use
      !     s0=2dt (for Morlet) and "jtot" from Eqn(10) Torrence&Compo(1998).
      IF(mother == 1 ) THEN
         s0=dt/4.0_r8
      ELSE IF (mother == 0 ) THEN
         s0=dt 
      END IF
END SUBROUTINE Init_Wavelet



SUBROUTINE Run_Wavelet(TimeSerie,Recon_TimeSerie_in,Recon_TimeSerie2_in,Filter_TimeSerie_in,&
                    wave,SCALE,period,Cdelta,coi,signif,&
                    global_ws,global_signif,fft_theor,P_theor,savg_dof,savg_signif)
!
!****************************************************************************
! WAVETEST: Example Fortran program for WAVELET, using NINO3 SST dataset
!
! See "http://paos.colorado.edu/research/wavelets/"
!
!  Copyright (C) 1998, Christopher Torrence and Gilbert P. Compo
! This software may be used, copied, or redistributed as long as it is not
! sold and this copyright notice is reproduced on each copy made.  This
! routine is provided as is without any express or implied warranties
! whatsoever.
!
! Modified: November 2010 by PAULO KUBOTA include IMPLICIT NONE and
!           to convert all routines to REAL(KIND=r8)   .
!****************************************************************************
!
      IMPLICIT NONE

      REAL(KIND=r8)   , INTENT(IN ) :: TimeSerie          (:)
      REAL(KIND=r4)   , INTENT(OUT) :: Recon_TimeSerie_in (:)
      REAL(KIND=r4)   , INTENT(OUT) :: Recon_TimeSerie2_in(:)
      REAL(KIND=r4)   , INTENT(OUT) :: Filter_TimeSerie_in(:)
      COMPLEX(KIND=r8), INTENT(OUT  ) :: wave             (:,:)!  wave = 2D array of the real & imaginary parts
                                                              !  of the wavelet transform, versus time & scale.
                                                              !  CABS(wave) gives the WAVELET amplitude,
                                                              !  ATAN2(AIMAG(wave),DBLE(wave)) gives WAVELET phase.
                                                              !  The WAVELET power spectrum is CABS(wave)**2.      
      REAL(KIND=r8)   , INTENT(OUT  ) :: SCALE            (:) !  the wavelet scale used to construct the daughter.
      REAL(KIND=r8)   , INTENT(OUT  ) :: period           (:) !  the "Fourier" periods (in time units) corresponding to "scale".
      REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta               ! the constant "Cdelta" for the mother wavelet (Table 2).

      REAL(KIND=r8)   , INTENT(OUT  ) :: coi              (:) ! the e-folding factor used for the cone of influence.
      REAL(KIND=r8)   , INTENT(OUT  ) :: signif           (:) ! significance levels vs scale.
                                                              ! IF SIGTEST=2, then signif(1) = the significance level
                                                              !  signif(2...jtot) = 0.0
      REAL(KIND=r8)   , INTENT(OUT  ) :: global_ws        (:)
      REAL(KIND=r8)   , INTENT(OUT  ) :: global_signif    (:)
      REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor        (:)
      REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor          (:)
      REAL(KIND=r8)   , INTENT(OUT  ) :: savg_dof         (:) !degrees-of-freedom that were actually used.
      REAL(KIND=r8)   , INTENT(OUT  ) :: savg_signif      (:)

      INTEGER          :: i
      INTEGER          :: j
      INTEGER          :: javg1        ! index of the lower scales.
      INTEGER          :: javg2        ! index of the tower scales.
      REAL(KIND=r8)    :: dof(jtot)    ! degrees-of-freedom for signif test.
                                       ! IF SIGTEST=0, then the input dof is ignored.
                                       ! IF SIGTEST=1, then dof(j) = NA, the number of times averaged together.
                                       ! IF SIGTEST=2, then dof(1)=S1, dof(2)=S2, the range of scales averaged.
      REAL(KIND=r8)    :: ymean
      REAL(KIND=r8)    :: variance
      REAL(KIND=r8)    :: recon_mean
      REAL(KIND=r8)    :: recon_vari
      REAL(KIND=r8)    :: psi0                ! the constant 'psi(0)' for the mother wavelet (Table 2)
      REAL(KIND=r8)    :: fourier_factor

      REAL(KIND=r8)  :: Recon_TimeSerie (LengthTimeSeries)
      REAL(KIND=r8)  :: Recon_TimeSerie2(LengthTimeSeries)
      REAL(KIND=r8)  :: Filter_TimeSerie(LengthTimeSeries)
      Recon_TimeSerie =0.0_r8
      Recon_TimeSerie2=0.0_r8
      Filter_TimeSerie=0.0_r8
      wave     =0.0_r8     ; SCALE    =0.0_r8     ; period   =0.0_r8     ; Cdelta   =0.0_r8     
      coi      =0.0_r8     ; signif   =0.0_r8     ; global_ws     =0.0_r8; global_signif =0.0_r8
      fft_theor     =0.0_r8; P_theor       =0.0_r8; savg_dof      =0.0_r8;savg_signif  =0.0_r8 
      javg1=0;javg2=0	 ;dof =0.0_r8 ;ymean=0.0_r8;variance=0.0_r8;recon_mean=0.0_r8;
      recon_vari=0.0_r8;psi0	=0.0_r8;      fourier_factor=0.0_r8;
      !
      !** get the wavelet transform
      !
      PRINT*,'BEGIN WAVELET'  

      CALL WAVELET(LengthTimeSeries, &!INTEGER         , INTENT(IN   ) :: n
                   TimeSerie       , &!REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
                   dt              , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
                   mother          , &!INTEGER         , INTENT(INOUT) :: mother !local
                   param           , &!REAL(KIND=r8)   , INTENT(IN   ) :: param
                   s0              , &!REAL(KIND=r8)   , INTENT(IN   ) :: s0
                   dj              , &!REAL(KIND=r8)   , INTENT(IN   ) :: dj
                   jtot            , &!INTEGER         , INTENT(IN   ) :: jtot
                   npad            , &!INTEGER         , INTENT(IN   ) :: npad
                   wave            , &!COMPLEX(KIND=r8), INTENT(OUT  ) :: wave  (n,jtot)
                   scale           , &!REAL(KIND=r8)   , INTENT(OUT  ) :: scale (  jtot)
                   period          , &!REAL(KIND=r8)   , INTENT(OUT  ) :: period(  jtot)
                   coi               )!REAL(KIND=r8)   , INTENT(OUT  ) :: coi   (n)

      PRINT*,'END WAVELET'  
      !
      !    Significance testing
      !
      !** local significance test
      !  
      !
      !   If 0, then just do a regular chi-square test,
      !   i.e. Eqn (18) from Torrence & Compo.
      !
      PRINT*,'BEGIN  WAVE_SIGNIF Significance testing'  
      isigtest = 0
      CALL WAVE_SIGNIF (isigtest        , &!INTEGER         , INTENT(IN   ) :: isigtest
                        LengthTimeSeries, &!INTEGER         , INTENT(IN   ) ::n
                        TimeSerie       , &!REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
                        dt              , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
                        mother          , &!INTEGER         , INTENT(IN   ) ::mother
                        param           , &!REAL(KIND=r8)   , INTENT(IN   ) :: param
                        dj              , &!REAL(KIND=r8)   , INTENT(IN   ) :: dj
                        jtot            , &!INTEGER         , INTENT(IN   ) ::jtot
                        scale           , &!REAL(KIND=r8)   , INTENT(IN   ) :: scale(jtot)
                        period          , &!REAL(KIND=r8)   , INTENT(IN   ) :: period(jtot)
                        lag1            , &!REAL(KIND=r8)   , INTENT(INOUT) :: lag1
                        siglvl          , &!REAL(KIND=r8)   , INTENT(INOUT) :: siglvl
                        dof             , &!REAL(KIND=r8)   , INTENT(INOUT) :: dof(jtot)
                        fft_theor       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor(jtot)
                        P_theor         , &!REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor(jtot)
                        signif          , &!REAL(KIND=r8)   , INTENT(OUT  ) :: signif(jtot)
                        ymean           , &!REAL(KIND=r8)   , INTENT(OUT  ) :: ymean
                        variance        , &!REAL(KIND=r8)   , INTENT(OUT  ) :: variance
                        Cdelta          , &!REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta
                        psi0              )!REAL(KIND=r8)   , INTENT(OUT  ) :: psi0

      PRINT*,'END  WAVE_SIGNIF Significance testing'  

      ! Global wavelet spectrum & significance test
      ! -------------------------------------------

      print *, 'Global wavelet spectrum...'

      DO j=1,jtot
        DO i=1,LengthTimeSeries
          global_ws(j) = global_ws(j) + ABS(wave(i,j))**2
        END DO
        global_ws(j) = global_ws(j)/LengthTimeSeries
        dof(j)       = LengthTimeSeries - SCALE(j)
      END DO
      !
      !** global wavelet spectrum & significance test
      !
      isigtest = 1      
      !
      !           If 1, then do a "time-average" test, i.e. Eqn (23).
      !              In this case, DOF(j) should be set to NA, the number
      !              of local wavelet spectra that were averaged together
      !              at each scale. For the Global Wavelet Spectrum,
      !              this would be dof(j)=N-scale(j),
      !              where N is the number of points in your time series.
      !
      PRINT*,'BEGIN  WAVE_SIGNIF  global wavelet spectrumSignificance testing'  

      CALL WAVE_SIGNIF (isigtest           , &!INTEGER         , INTENT(IN   ) :: isigtest
                        LengthTimeSeries   , &!INTEGER         , INTENT(IN   ) ::n
                        TimeSerie          , &!REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
                        dt                 , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
                        mother             , &!INTEGER         , INTENT(IN   ) ::mother
                        param              , &!REAL(KIND=r8)   , INTENT(IN   ) :: param
                        dj                 , &!REAL(KIND=r8)   , INTENT(IN   ) :: dj
                        jtot               , &!INTEGER         , INTENT(IN   ) ::jtot
                        scale              , &!REAL(KIND=r8)   , INTENT(IN   ) :: scale(jtot)
                        period             , &!REAL(KIND=r8)   , INTENT(IN   ) :: period(jtot)
                        lag1               , &!REAL(KIND=r8)   , INTENT(INOUT) :: lag1
                        siglvl             , &!REAL(KIND=r8)   , INTENT(INOUT) :: siglvl
                        dof                , &!REAL(KIND=r8)   , INTENT(INOUT) :: dof(jtot)
                        fft_theor          , &!REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor(jtot)
                        P_theor            , &!REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor(jtot)
                        global_signif      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: signif(jtot)
                        ymean              , &!REAL(KIND=r8)   , INTENT(OUT  ) :: ymean
                        variance           , &!REAL(KIND=r8)   , INTENT(OUT  ) :: variance
                        Cdelta             , &!REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta
                        psi0                 )!REAL(KIND=r8)   , INTENT(OUT  ) :: psi0

      PRINT*,'END  WAVE_SIGNIF  global wavelet spectrumSignificance testing'  

      !** scale-average time series & significance test
      isigtest = 2
      !          If 2, then do a "scale-average" test, i.e. Eqns (25)-(28).
      !              In this case, "dof(1)" and "dof(2)" should be set to the
      !              smallest (S1) and largest (S2) scales that were averaged
      !              together, respectively.
      !              e.g. if you scale-averaged scales between 2 and 8,
      !                   then dof(1)=2.0 and dof(2)=8.0
      IF (mother.EQ.0) THEN
         !*******************************************   Morlet wavelet
          fourier_factor = (4.0_r8*pi)/(param + SQRT(2.0_r8+param**2))
      ELSE IF (mother.EQ.1) THEN
         !*******************************************   Paul wavelet
         IF (param.LT.0) param = 4.0_r8
         fourier_factor = (4.0_r8*pi)/(2.0_r8*INT(param) + 1.0_r8)
      ELSE IF (mother.EQ.2) THEN
         !*******************************************   DOG wavelet
         fourier_factor = 2.0_r8*pi*SQRT(2.0_r8/(2.0_r8*INT(param)+1.0_r8))
      ELSE
         STOP
      END IF
      !
      !
      !    scale average between 2 and 7.9 years
      !
      !savg_dof(1) = scalemin
      !savg_dof(2) = scalemax
      
      if (scalemin.ge.0.0d0) then
         savg_dof(1) = scalemin/fourier_factor
      else
         savg_dof(1) = 0.0d0
      endif
      if (scalemax.ge.0.0d0) then
         savg_dof(2) = scalemax/fourier_factor
      else
         savg_dof(2) = scale(jtot) + 10.0d0
      endif
      !
      !    find the "j"-values that correspond to savg_dof(1) and savg_dof(2)
      !
      PRINT*,'find the "j"-values that correspond to savg_dof(1) and savg_dof(2)'
      PRINT*,'SCALE=',MINVAL(SCALE),MAXVAL(SCALE),'fourier_factor=',fourier_factor
      PRINT*,'((SCALE(j) >= savg_dof(1))=',MINVAL(SCALE),'>=',savg_dof(1)
      PRINT*,'((SCALE(j) <= savg_dof(2))=',MAXVAL(SCALE),'<=',savg_dof(2)

      javg1 = 0
      javg2 = 0
      DO j=1,jtot
        IF ((SCALE(j) >= savg_dof(1)) .AND. (javg1 == 0)) javg1 = j
        IF  (SCALE(j) <= savg_dof(2)) javg2 = j
      END DO

      !
      !   call wave_signif first, to get the value of "Cdelta"
      !
      PRINT*,'BEGIN  WAVE_SIGNIF Significance testing'  

      CALL WAVE_SIGNIF (isigtest        , &  !INTEGER         , INTENT(IN   ) :: isigtest
                        LengthTimeSeries, &  !INTEGER         , INTENT(IN   ) :: n
                        TimeSerie       , &  !REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
                        dt              , &  !REAL(KIND=r8)   , INTENT(IN   ) :: dt
                        mother          , &  !INTEGER         , INTENT(IN   ) :: mother
                        param           , &  !REAL(KIND=r8)   , INTENT(IN   ) :: param
                        dj              , &  !REAL(KIND=r8)   , INTENT(IN   ) :: dj
                        jtot            , &  !INTEGER         , INTENT(IN   ) :: jtot
                        scale           , &  !REAL(KIND=r8)   , INTENT(IN   ) :: scale(jtot)
                        period          , &  !REAL(KIND=r8)   , INTENT(IN   ) :: period(jtot)
                        lag1            , &  !REAL(KIND=r8)   , INTENT(INOUT) :: lag1
                        siglvl          , &  !REAL(KIND=r8)   , INTENT(INOUT) :: siglvl
                        savg_dof        , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: dof(jtot)
                        fft_theor       , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor(jtot)
                        P_theor         , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor(jtot)
                        savg_signif     , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: signif(jtot)
                        ymean           , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: ymean
                        variance        , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: variance
                        Cdelta          , &  !REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta
                        psi0              )  !REAL(KIND=r8)   , INTENT(OUT  ) :: psi0
      PRINT*,'END  WAVE_SIGNIF Significance testing'  

      !
      !   construct the scale-averaged time series [Eqn(24)]
      !
      !                     ___ j2
      !                     \             2
      !  ---2     dj * dt    \   |Wn(Sj)|
      !  Wn   = ----------- * |----------
      !            Cd        /     Sj
      !                     /____
      !                       j=j1
      !
      DO i=1,LengthTimeSeries
        Filter_TimeSerie(i) = 0.0_r8
        DO  j=javg1,javg2
          Filter_TimeSerie(i) = Filter_TimeSerie(i) + (ABS(wave(i,j))**2)/SCALE(j)
        END DO
        Filter_TimeSerie(i) = dj*dt*Filter_TimeSerie(i)/Cdelta
      END DO


!************************************************************* print results
      !PRINT*,' n=',LengthTimeSeries
      !PRINT*,' dt=',dt
      !PRINT*,' mother=',mother
      !PRINT*,' param=',param
      !PRINT*,' s0=',s0
      !PRINT*,' dj=',dj
      !PRINT*,' jtot=',jtot
      !PRINT*,' npad=',npad
      !PRINT'(/,"Let w = wave(n/2,j)",/)'
      !PRINT'(A4,7A10)',"j","Scale","Period","ABS(w)^2","phase(w)",  &
      !  "5%signif","Global","GWS5%sig"
      !PRINT'(I4,7F10.3)',(j,SCALE(j),period(j), ABS(wave(LengthTimeSeries/2,j))**2, &
      !                      ATAN2(DIMAG(wave(LengthTimeSeries/2,j)),DBLE(wave(LengthTimeSeries/2,j)))*180._r8/pi, &
      !   signif(j),global_ws(j),global_signif(j),j=1,jtot)
      !PRINT'(/,A,F10.3)', &
      !    ' Scale-average degrees of freedom = ',savg_dof(1)
      !PRINT'(A,F10.3,/)', &
      !    ' Scale-avg 5% significance level  = ',savg_signif(1)
      
!************************************************************ Reconstruction
!                                                            ---------------
!** construct the wavelet derived variance (Parseval's theorem)  [Eqn(14)]
!   Cdelta & psi0 are returned from WAVE_SIGNIF
!                            _______N-1  _______ J
!                            \           \            2
!         2        dj * dt    \           \  | Wn(Sj) |
!   sigma     = -------------  |           |------------
!                  Cd * N     /           /      Sj
!                            /______     /________
!                               n=0          j=0
!
      recon_vari = 0.0_r8
      DO i=1,LengthTimeSeries
        DO j=1,jtot
          recon_vari = recon_vari + (ABS(wave(i,j))**2)/SCALE(j)
        END DO
      END DO
      recon_vari = dj*dt*recon_vari/(Cdelta*LengthTimeSeries)

      !PRINT'(A,F14.5)',' Reconstructed variance=',recon_vari
      !PRINT'(A,F14.5)',' Original variance   =',variance
      !PRINT'(A,F14.5,A,/)',' Ratio = ',recon_vari/variance, &
      !     ' (this is low due to padding with zeroes)'

!** reconstruct the time series [Eqn(11)]
!
!
!                              _______ J
!                            \            
!          dj * sqrt(dt)      \  REAL | Wn(Sj) |
!   Xn = -----------------     |------------------
!           Cd * Psi0         /     SQRT( Sj )
!                            /________
!                                j=0
!


!   check mean and RMS difference of reconstructed time series

      recon_mean = 0.0_r8
      recon_vari = 0.0_r8
      DO i=1,LengthTimeSeries
        Recon_TimeSerie(i)=0.0_r8
        DO j=1,jtot
          Recon_TimeSerie(i) = Recon_TimeSerie(i)+(DBLE(wave(i,j)))/SQRT(SCALE(j))
        END DO
        Recon_TimeSerie(i) = dj*SQRT(dt)*Recon_TimeSerie(i)/(Cdelta*psi0)
        recon_vari   = recon_vari+(TimeSerie(i)-ymean-Recon_TimeSerie(i))**2
        recon_mean   = recon_mean + Recon_TimeSerie(i)
      END DO
      recon_mean = recon_mean/LengthTimeSeries
      recon_vari = SQRT(recon_vari/LengthTimeSeries)

      ! PRINT'(A,F14.6)',' Reconstructed mean=',recon_mean
      ! PRINT'(A,F14.6)',' Original mean   =',ymean
      ! PRINT'(A,F14.6,/)',' Root-mean-square difference of time series=', &
      !       recon_vari
      ! Filtering
      ! ---------
      !
      ! reconstruct the time series between 'javg1' and 'javg2' [Eq. 29]
      PRINT *, 'Filtering...'
      !
      !
      !
      !                              _______ J=javg2
      !                            \            
      !          dj * sqrt(dt)      \  REAL | Wn(Sj) |
      !   Xn = -----------------     |------------------
      !           Cd * Psi0         /     SQRT( Sj )
      !                            /________
      !                                j=javg1
      !


      !   check mean and RMS difference of reconstructed time series

      !

      DO i=1,LengthTimeSeries
         Recon_TimeSerie2(i)=0.D0
         DO j=javg1,javg2
            Recon_TimeSerie2(i) = Recon_TimeSerie2(i)+(DBLE(wave(i,j)))/SQRT(scale(j))
         END DO
         Recon_TimeSerie2(i) = dj*SQRT(dt)*Recon_TimeSerie2(i)/(Cdelta*psi0)
      END DO
      PRINT *, '-----'

      Recon_TimeSerie_in = Recon_TimeSerie
      Recon_TimeSerie2_in= Recon_TimeSerie2
      Filter_TimeSerie_in= Filter_TimeSerie
END SUBROUTINE Run_Wavelet


  !****************************************************************************
  !
  ! WAVEPACK:  routines to compute the wavelet transform of a time series,
  !            and significance levels.
  !
  ! Written by:   Christopher Torrence and Gilbert P. Compo
  !
  ! Available from:  http://paos.colorado.edu/research/wavelets/
  !
  ! Requires the following packages:   CFFTPACK, CHISQR
  !
  ! Notes:
  !
  !  (1) All routines are written in single precision (REAL(KIND=r8)   ),
  !      except for CHISQR, which requires REAL(KIND=r8)    input.
  !      Single precision should be sufficient for most applications.
  !
  !  (2) The CFFTPACK and CHISQR routines were not written by us,
  !      and no guarentees are made as to their reliability or efficiency.
  !
  !  (3) No provision is made for output to files or to graphics packages.
  !      The user is expected to call these routines from within
  !      their own programs. See sample program "wavetest.f".
  !
  !  (4) Little error checking is done. Check your input carefully.
  !      The programs are not completely ANSI compatible, so use caution.
  !
  !  (5) Time series are currently limited to 65535 points.
  !
  !
  ! Reference: Torrence, C. and G. P. Compo, 1998: A Practical Guide to
  !            Wavelet Analysis. <I>Bull. Amer. Meteor. Soc.</I>, 79, 61-78.
  !
  ! Notice: Please acknowledge the use of this software in any publications:
  !    ``Wavelet software was provided by C. Torrence and G. Compo,
  !      and is available at URL: http://paos.colorado.edu/research/wavelets/''.
  !
  !  Copyright (C) 1998, Christopher Torrence
  ! This software may be used, copied, or redistributed as long as it is not
  ! sold and this copyright notice is reproduced on each copy made.  This
  ! routine is provided as is without any express or implied warranties
  ! whatsoever.
  !
  !
  ! Modified: November 1999 by Arjan van Dijk to include IMPLICIT NONE and
  !           to convert all routines to REAL(KIND=r8)   .
  !****************************************************************************




  !****************************************************************************
  ! WAVELET: computes the wavelet transform of a time series,
  !          with appropriate parameters.
  !
  !
  ! INPUTS:
  !
  !  n [INT] = the number of points in "y".
  !
  !  y [REAL(KIND=r8)   ] = the time series of length "n".
  !
  !  dt [REAL(KIND=r8)   ] = amount of time between each Y value, i.e. the sampling time.
  !
  !  mother [INT] = An integer giving the mother wavelet to use.
  !                   0='Morlet'
  !                   1='Paul'
  !                   2='DOG' (derivative of Gaussian)
  !               If (mother<0 or >2) then default is 'Morlet'.
  !
  !  param [REAL(KIND=r8)   ] = mother wavelet parameter. If <0 then default is used.
  !            For 'Morlet' this is k0 (wavenumber), default is 6.
  !            For 'Paul' this is m (order), default is 4.
  !            For 'DOG' this is m (m-th derivative), default is 2.
  !
  !
  !  s0 [REAL(KIND=r8)   ] = the smallest scale of the wavelet.  Typically = 2*dt.
  !         Note: for accurate reconstruction and variance computation
  !             set s0=dt for Morlet; s0=dt/4 for Paul
  !
  !  dj [REAL(KIND=r8)   ] = the spacing between discrete scales. Typically = 0.25.
  !         A smaller # will give better scale resolution, but be slower.
  !
  !  jtot [INT] = the # of scales.
  !             Scales range from s0 up to s0*2^[(jtot-1)*dj],
  !             Typically jtot=1+(LOG2(n dt/s0))/dj
  !
  !  npad [INT] = the total number of points (including padding) to
  !             use for the wavelet transform. Typically this is some
  !             power of 2. It must be greater or equal to "n".
  !             If npad>n, then zeroes are padded onto the end
  !             of the time series.
  !
  !
  ! OUTPUTS:
  !
  !  wave [DCMPLX(n,jtot)] = 2D array of the real & imaginary parts
  !                 of the wavelet transform, versus time & scale.
  !                 CABS(wave) gives the WAVELET amplitude,
  !                 ATAN2(AIMAG(wave),DBLE(wave)) gives WAVELET phase.
  !                 The WAVELET power spectrum is CABS(wave)**2.
  !
  !  scale [REAL(KIND=r8)   (jtot)] = the wavelet scales that were used.
  !
  !  period [REAL(KIND=r8)   (jtot)] = the "Fourier" periods (in time units) corresponding
  !            to "scale".
  !
  !  coi [REAL(KIND=r8)   (n)] = the e-folding factor used for the cone of influence.
  !
  !
  ! REQUIRES:   WAVE_FUNCTION, CFFTPACK
  !
  !  Copyright (C) 1998, Christopher Torrence and Gilbert P. Compo
  ! This software may be used, copied, or redistributed as long as it is not
  ! sold and this copyright notice is reproduced on each copy made.  This
  ! routine is provided as is without any express or implied warranties
  ! whatsoever.

  SUBROUTINE WAVELET ( &
       n         , &!sINTEGER          , INTENT(IN   ) :: n
       y         , &!REAL(KIND=r8)     , INTENT(IN   ) :: y(n)
       dt        , &!REAL(KIND=r8)     , INTENT(IN   ) :: dt
       mother    , &!INTEGER           , INTENT(INOUT) :: mother !local
       param     , &!REAL(KIND=r8)     , INTENT(IN   ) :: param
       s0        , &!REAL(KIND=r8)     , INTENT(IN   ) :: s0
       dj        , &!REAL(KIND=r8)     , INTENT(IN   ) :: dj
       jtot      , &!INTEGER           , INTENT(IN   ) :: jtot
       npad      , &!INTEGER           , INTENT(IN   ) :: npad
       wave      , &!COMPLEX(KIND=r8)  , INTENT(OUT  ) :: wave  (n,jtot)
       scale     , &!REAL(KIND=r8)     , INTENT(OUT  ) :: scale (jtot)
       period    , &!REAL(KIND=r8)     , INTENT(OUT  ) :: period(jtot)
       coi         )!REAL(KIND=r8)     , INTENT(OUT  ) :: coi   (n)
    IMPLICIT NONE

    INTEGER         , INTENT(IN   ) :: n
    INTEGER         , INTENT(INOUT) :: mother !local
    INTEGER         , INTENT(IN   ) :: jtot
    INTEGER         , INTENT(IN   ) :: npad
    REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
    REAL(KIND=r8)   , INTENT(IN   ) :: dt
    REAL(KIND=r8)   , INTENT(INOUT) :: param
    REAL(KIND=r8)   , INTENT(IN   ) :: s0
    REAL(KIND=r8)   , INTENT(IN   ) :: dj
    REAL(KIND=r8)   , INTENT(OUT  ) :: SCALE (jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: period(jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: coi   (n)
    COMPLEX(KIND=r8)  , INTENT(OUT  ) :: wave  (n,jtot)

    INTEGER          :: i
    INTEGER          :: j
    INTEGER          :: k
    REAL(KIND=r8)    :: ymean
    REAL(KIND=r8)    :: freq1
    REAL(KIND=r8)    :: pi
    REAL(KIND=r8)    :: period1
    REAL(KIND=r8)    :: coi1

    !** initialize work arrays

    INTEGER, PARAMETER :: nk=5000000
    REAL(KIND=r8)      :: wsave(4*nk+15)
    REAL(KIND=r8)      :: kwave(nk)
    COMPLEX(KIND=r8)     :: yfft(nk)
    REAL(KIND=r8)      :: xfft(nk)
    REAL(KIND=r8)      :: daughter2(nk)

    COMPLEX(KIND=r8)     :: daughter(nk)
    INTEGER, PARAMETER :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
    REAL(KIND=r8)  :: CREAL   (nk)
    REAL(KIND=r8)  :: CIMAG   (nk)

    pi = 4.0_r8*ATAN(1.0_r8)

    IF (npad.LT.n) THEN
       PRINT*,'**WAVELET: "npad" must be greater than or equal to "n"'
       RETURN
    END IF

    IF ((mother.LT.0).OR.(mother.GT.2)) mother = 0

    !** find the time-series mean & remove it
    ymean = 0.0_r8

    DO i=1,n
       ymean = ymean + y(i)
    END DO

    yfft=CMPLX(0.0_r8,0.0_r8,KIND=r8)
    ymean = ymean/n

    DO i=1,n
       yfft(i) = y(i) - ymean
    END DO

    !** if desired, pad with extra zeroes

    DO  i=n+1,npad
       yfft(i) = 0.0_r8
       xfft(i) = 0.0_r8
    END DO

    !** find the FFT of the time series [Eqn(3)]
    !                     N-1
    !                  -------
    !           1      \             2*pi*i*k*n
    ! X(k) = -------    \ X(n) exp(--------------)
    !           N       /               N
    !                  /
    !                  -------
    !                     n=0
    CALL CFFTI(npad    , & ! INTEGER         , INTENT(IN   ) :: N
         wsave         )   ! REAL(KIND=r8)   , INTENT(IN   ) ::  WSAVE(*)

    i=1
    DO k=1,npad
       xfft(i)=REAL(yfft(k))
       i=i+1
       xfft(i)=AIMAG(yfft(k))
       i=i+1
    END DO

    CALL CFFTF(nk      , &!INTEGER             , INTENT(IN   ) :: nk
         npad          , &!INTEGER             , INTENT(IN   ) :: N
         xfft          , &!COMPLEX(KIND=r8)    , INTENT(INOUT) :: C    (nk)
         wsave           )!REAL(KIND=r8)       , INTENT(INOUT) :: WSAVE(nk)

    i=1
    DO k=1,npad
       yfft(k) = CMPLX(xfft(i),xfft(i+1))
       i=i+2
    END DO

    DO k=1,npad
       yfft(k) = yfft(k)/npad
    END DO

    !** construct the wavenumber array [Eqn(5)]
    !  DBLE -> Conversion to REAL(KIND=r8)    (real part)
    !
    !            ---------
    !           |   2*pi*k               N  
    !           |  --------   => k <= ------- 
    !           |     N*dt               2
    !Omega(k) = |
    !           |
    !           | - 2*pi*k               N  
    !           |  --------   => k >  ------- 
    !           |     N*dt               2
    !           -----------------
    !
    !
    !
    freq1 = 2.0_r8*pi/(DBLE(npad)*dt)
    kwave(1) = 0.0_r8
    DO i=2,npad/2+1
       kwave(i) = (DBLE(i)-1.0_r8)*freq1
    END DO

    DO  i=npad/2+2,npad
       kwave(i) = -kwave(npad-i+2)
    END DO

    !**----- Main wavelet loop
    !      PRINT '(A8,2A12,/,5X,27("-"))','j','scale','period'
    DO j=1,jtot
       SCALE(j) = s0*(2._r8**(DBLE(j-1)*dj))

       CALL WAVE_FUNCTION(npad    , &!INTEGER         , INTENT(IN   ) :: nk
            dt                    , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
            mother                , &!INTEGER         , INTENT(IN   ) :: mother
            param                 , &!REAL(KIND=r8)   , INTENT(INOUT) :: param
            SCALE(j)              , &!REAL(KIND=r8)   , INTENT(IN   ) :: scale1
            kwave                 , &!REAL(KIND=r8)   , INTENT(IN   ) :: kwave(nk)
            period1               , &!REAL(KIND=r8)   , INTENT(OUT  ) :: period1
            coi1                  , &!REAL(KIND=r8)   , INTENT(OUT  ) :: coi1
            daughter                )!COMPLEX(KIND=r8), INTENT(OUT  ) :: daughter(nk)

       period(j) = period1
       !
       !**    multiply the daughter by the time-series FFT
       !
       DO k=1,npad
          daughter(k) = daughter(k)*yfft(k)
       END DO
       !        
       !**    inverse FFT [Eqn(4)]
       !      
       !               N-1
       !            -------
       !            \   __    ___                      
       ! Ws(s) = -   \ X(k) * Fi  * (s*omega(k))* exp(i*omega(k)*n*dt)
       !             /                
       !            /
       !            -------
       !               k=0

       i=1
       DO k=1,npad
          daughter2(i)=REAL(daughter(k))
          i=i+1
          daughter2(i)=AIMAG(daughter(k))
          i=i+1
       END DO

       CALL CFFTB(npad        , & !INTEGER        , INTENT(IN   ) :: N
            daughter2         , & !REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
            wsave               ) !REAL(KIND=r8)   , INTENT(INOUT) :: WSAVE(*)

       i=1
       DO k=1,npad
          daughter(k) = CMPLX(daughter2(i),daughter2(i+1))
          i=i+2
       END DO

       !
       !**    store the wavelet transform, discard zero-padding at end
       !
       DO i=1,n
          wave(i,j) = daughter(i)
       END DO
       !        PRINT '(I8,2F12.3)',j,scale(j),period(j)
    END DO
    !**----- end loop

    !** construct the cone of influence
    DO i=1,(n+1)/2
       coi(i) = coi1*dt*(DBLE(i)-1._r8)
       coi(n-i+1) = coi(i)
    END DO

    RETURN
  END SUBROUTINE WAVELET


  !****************************************************************************
  ! WAVE_FUNCTION: computes the daughter wavelets for a particular
  !              wavelet function, with appropriate parameters.
  !
  !
  ! INPUTS:
  !
  !  nk [INT] = the number of points in "kwave"
  !
  !  dt [REAL(KIND=r8)   ] = amount of time between each Y value, i.e. the sampling time.
  !
  !  mother [INT] = An integer giving the mother wavelet to use.
  !                   0='Morlet'
  !                   1='Paul'
  !                   2='DOG' (derivative of Gaussian)
  !
  !  param [REAL(KIND=r8)   ] = mother wavelet parameter. If <0 then default is used.
  !            For 'Morlet' this is k0 (wavenumber), default is 6.
  !            For 'Paul' this is m (order), default is 4.
  !            For 'DOG' this is m (m-th derivative), default is 2.
  !
  !  scale1 [REAL(KIND=r8)   ] = the wavelet scale used to construct the daughter.
  !
  !  kwave [REAL(KIND=r8)   (n)] = vector of wavenumbers, used to construct daughter.
  !
  !
  ! OUTPUTS:
  !
  !  period1 [REAL(KIND=r8)   ] = the "Fourier" period (in time units) that corresponds
  !            to "scale1".
  !
  !  coi1 [REAL(KIND=r8)   ] = the e-folding factor used for the cone of influence.
  !
  !  daughter [DCMPLX(nk)] = real & imaginary parts of the wavelet function
  !                    at "scale1" and "kwave".
  !
  !
  ! REQUIRES:   FACTORIAL, CHISQR
  !
  !
  ! Reference: Tables 1 & 2 in
  !            Torrence, C. and G. P. Compo, 1998: A Practical Guide to
  !            Wavelet Analysis. <I>Bull. Amer. Meteor. Soc.</I>, 79, 61-78.
  !
  !  Copyright (C) 1998, Christopher Torrence and Gilbert P. Compo
  ! This software may be used, copied, or redistributed as long as it is not
  ! sold and this copyright notice is reproduced on each copy made.  This
  ! routine is provided as is without any express or implied warranties
  ! whatsoever.

  SUBROUTINE WAVE_FUNCTION (nk       , &!INTEGER         , INTENT(IN   ) :: nk
       dt                            , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
       mother                        , &!INTEGER         , INTENT(IN   ) :: mother
       param                         , &!REAL(KIND=r8)   , INTENT(INOUT) :: param
       scale1                        , &!REAL(KIND=r8)   , INTENT(IN   ) :: scale1
       kwave                         , &!REAL(KIND=r8)   , INTENT(IN   ) :: kwave(nk)
       period1                       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: period1
       coi1                          , &!REAL(KIND=r8)   , INTENT(OUT  ) :: coi1
       daughter                        )!COMPLEX(KIND=r8), INTENT(OUT  ) :: daughter(nk)
    IMPLICIT NONE
    INTEGER, PARAMETER :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
    INTEGER         , INTENT(IN   ) :: nk
    INTEGER         , INTENT(IN   ) :: mother
    REAL(KIND=r8)   , INTENT(IN   ) :: dt
    REAL(KIND=r8)   , INTENT(IN   ) :: kwave(nk)
    REAL(KIND=r8)   , INTENT(INOUT) :: param
    REAL(KIND=r8)   , INTENT(IN   ) :: scale1
    REAL(KIND=r8)   , INTENT(OUT  ) :: period1
    REAL(KIND=r8)   , INTENT(OUT  ) :: coi1
    COMPLEX(KIND=r8), INTENT(OUT  ) :: daughter(nk)

    COMPLEX(KIND=r8)       :: norm
    REAL(KIND=r8)          :: expnt,sk,pi,fourier_factor
    INTEGER k,m
   ! REAL(KIND=r8)    gamma

    pi = 4.0_r8*ATAN(1.0_r8)

    IF (mother.EQ.0) THEN
       !*******************************************   Morlet wavelet
       IF (param.LT.0) param = 6.0_r8
       norm = SQRT(2.0_r8*pi*scale1/dt)*(pi**(-0.25_r8))
       DO k=1,nk/2+1
          expnt = -0.5_r8*(scale1*kwave(k) - param)**2
          daughter(k) = DCMPLX(norm*EXP(expnt))
       END DO
       DO k=nk/2+2,nk
          daughter(k) = DCMPLX(0.0_r8)
       END DO
       fourier_factor = (4.0_r8*pi)/(param + SQRT(2.0_r8+param**2))
       period1 = scale1*fourier_factor
       coi1 = fourier_factor/SQRT(2.0_r8)
    ELSE IF (mother.EQ.1) THEN
       !*******************************************   Paul wavelet
       IF (param.LT.0) param = 4.0_r8
       m = INT(param)
       norm = SQRT(2.0_r8*pi*scale1/dt)* &
            (2.0_r8**m/SQRT(DBLE(m*FACTORIAL(2*m-1))))
       DO k=1,nk/2+1
          expnt = -scale1*kwave(k)
          daughter(k) = DCMPLX(norm*(scale1*kwave(k))**m*EXP(expnt))
       END DO
       DO k=nk/2+2,nk
          daughter(k) = DCMPLX(0.0_r8)
       END DO
       fourier_factor = (4.0_r8*pi)/(2.0_r8*m + 1.0_r8)
       period1 = scale1*fourier_factor
       coi1 = fourier_factor*SQRT(2.0_r8)
    ELSE IF (mother.EQ.2) THEN
       !*******************************************   DOG wavelet
       IF (param.LT.0) param = 2.0_r8
       m = INT(param)
       norm = SQRT(2.0_r8*pi*scale1/dt)*SQRT(1.0_r8/GAMMA(m+0.5_r8))
       norm = -norm*(DCMPLX(0.0_r8,1.0_r8)**m)
       DO k=1,nk
          sk = scale1*kwave(k)
          daughter(k) = norm*(sk**m)*EXP(-0.5_r8*sk**2)
       END DO
       fourier_factor = 2.0_r8*pi*SQRT(2.0_r8/(2.0_r8*m+1.0_r8))
       period1 = scale1*fourier_factor
       coi1 = fourier_factor/SQRT(2.0_r8)
    ELSE
       STOP
    END IF
    RETURN
  END SUBROUTINE WAVE_FUNCTION


  !****************************************************************************
  ! FACTORIAL: compute the factorial (n!) of an integer n
  !  Copyright (C) 1998, Christopher Torrence
  FUNCTION FACTORIAL(n)
    IMPLICIT NONE
    INTEGER factorial,n,i

    factorial = 1
    DO  i=1,n
       factorial = factorial*i
    END DO
  END FUNCTION FACTORIAL



  !****************************************************************************
  ! WAVE_SIGNIF: computes the significance levels for a wavelet transform.
  !
  !
  ! INPUTS:
  !
  !  isigtest [INT] = 0, 1, or 2.
  !
  !          If 0, then just do a regular chi-square test,
  !              i.e. Eqn (18) from Torrence & Compo.
  !          If 1, then do a "time-average" test, i.e. Eqn (23).
  !              In this case, DOF(j) should be set to NA, the number
  !              of local wavelet spectra that were averaged together
  !              at each scale. For the Global Wavelet Spectrum,
  !              this would be dof(j)=N-scale(j),
  !              where N is the number of points in your time series.
  !          If 2, then do a "scale-average" test, i.e. Eqns (25)-(28).
  !              In this case, "dof(1)" and "dof(2)" should be set to the
  !              smallest (S1) and largest (S2) scales that were averaged
  !              together, respectively.
  !              e.g. if you scale-averaged scales between 2 and 8,
  !                   then dof(1)=2.0 and dof(2)=8.0
  !
  !
  !  n [INT] = the number of points in "y".
  !
  !  y [REAL(KIND=r8)   ] = the time series of length "n".
  !
  !  dt [REAL(KIND=r8)   ] = amount of time between each Y value, i.e. the sampling time.
  !
  !  mother [INT] = An integer giving the mother wavelet to use.
  !                   0='Morlet'
  !                   1='Paul'
  !                   2='DOG' (derivative of Gaussian)
  !
  !  param [REAL(KIND=r8)   ] = mother wavelet parameter.
  !
  !  s0 [REAL(KIND=r8)   ] = the smallest scale of the wavelet.
  !
  !  dj [REAL(KIND=r8)   ] = the spacing between discrete scales.
  !
  !  jtot [INT] = the # of scales.
  !
  !  scale [REAL(KIND=r8)   (jtot)] = the wavelet scales that were used.
  !
  !  period [REAL(KIND=r8)   (jtot)] = the "Fourier" periods corresponding to "scale".
  !
  !  lag1 [REAL(KIND=r8)   ] = lag 1 Autocorrelation, used for signif levels.
  !              Default is 0.0, which corresponds to white-noise.
  !
  !  siglvl [REAL(KIND=r8)   ] = significance level to use. Default is 0.05 (the "5%" level)
  !
  !  dof [REAL(KIND=r8)   (jtot)] = degrees-of-freedom for signif test.
  !     IF SIGTEST=0, then the input dof is ignored.
  !     IF SIGTEST=1, then dof(j) = NA, the number of times averaged together.
  !     IF SIGTEST=2, then dof(1)=S1, dof(2)=S2, the range of scales averaged.
  !
  !
  ! OUTPUTS:
  !
  !  dof [REAL(KIND=r8)   (jtot)] = degrees-of-freedom that were actually used.
  !     IF SIGTEST=0, then dof(j) = 2 (or 1 for the 'DOG')
  !     IF SIGTEST=1, then dof(j) = degrees-of-freedom versus scale.
  !     IF SIGTEST=2, then dof(1)=degrees-of-freedom, dof(2...jtot)=0.0
  !
  !  fft_theor [REAL(KIND=r8)   (jtot)] = theoretical red-noise spectrum vs scale.
  !     IF SIGTEST=2, then fft_theor(1) = the average spectrum from S1-->S2
  !                   fft_theor(2...jtot) = 0.0
  !
  !  signif [REAL(KIND=r8)   (jtot)] = significance levels vs scale.
  !     IF SIGTEST=2, then signif(1) = the significance level
  !                   signif(2...jtot) = 0.0
  !
  !  ymean [REAL(KIND=r8)   ] = the mean of the time series.
  !
  !  variance [REAL(KIND=r8)   ] = the variance of the time series.
  !
  !  Cdelta [REAL(KIND=r8)   ] = the constant "Cdelta" for the mother wavelet (Table 2).
  !
  !  psi0[REAL(KIND=r8)   ] = the constant 'psi(0)' for the mother wavelet (Table 2)
  !
  ! REQUIRES:   CHISQR
  !
  !  Copyright (C) 1998, Christopher Torrence and Gilbert P. Compo
  ! This software may be used, copied, or redistributed as long as it is not
  ! sold and this copyright notice is reproduced on each copy made.  This
  ! routine is provided as is without any express or implied warranties
  ! whatsoever.

  SUBROUTINE WAVE_SIGNIF (isigtest  , &!INTEGER         , INTENT(IN   ) :: isigtest
       n                            , &!INTEGER         , INTENT(IN   ) :: n
       y                            , &!REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
       dt                           , &!REAL(KIND=r8)   , INTENT(IN   ) :: dt
       mother                       , &!INTEGER         , INTENT(IN   ) :: mother
       param                        , &!REAL(KIND=r8)   , INTENT(IN   ) :: param
       dj                           , &!REAL(KIND=r8)   , INTENT(IN   ) :: dj
       jtot                         , &!INTEGER         , INTENT(IN   ) :: jtot
       scale                        , &!REAL(KIND=r8)   , INTENT(IN   ) :: scale(jtot)
       period                       , &!REAL(KIND=r8)   , INTENT(IN   ) :: period(jtot)
       lag1                         , &!REAL(KIND=r8)   , INTENT(INOUT) :: lag1
       siglvl                       , &!REAL(KIND=r8)   , INTENT(INOUT) :: siglvl
       dof                          , &!REAL(KIND=r8)   , INTENT(INOUT) :: dof(jtot)
       fft_theor                    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor(jtot)
       P_theor                      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor(jtot)
       signif                       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: signif(jtot)
       ymean                        , &!REAL(KIND=r8)   , INTENT(OUT  ) :: ymean
       variance                     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: variance
       Cdelta                       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta
       psi0                           )!REAL(KIND=r8)   , INTENT(OUT  ) :: psi0
    IMPLICIT NONE

    INTEGER, INTENT(IN   ) :: isigtest
    INTEGER, INTENT(IN   ) ::n
    INTEGER, INTENT(IN   ) ::mother
    INTEGER, INTENT(IN   ) ::jtot

    REAL(KIND=r8)   , INTENT(IN   ) :: y(n)
    REAL(KIND=r8)   , INTENT(IN   ) :: dt
    REAL(KIND=r8)   , INTENT(IN   ) :: param
    REAL(KIND=r8)   , INTENT(IN   ) :: dj
    REAL(KIND=r8)   , INTENT(IN   ) :: SCALE(jtot)
    REAL(KIND=r8)   , INTENT(IN   ) :: period(jtot)

    REAL(KIND=r8)   , INTENT(INOUT) :: lag1
    REAL(KIND=r8)   , INTENT(INOUT) :: siglvl
    REAL(KIND=r8)   , INTENT(INOUT) :: dof(jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: fft_theor(jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor  (jtot)       !REAL(KIND=r8)   , INTENT(OUT  ) :: P_theor(jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: signif(jtot)
    REAL(KIND=r8)   , INTENT(OUT  ) :: ymean
    REAL(KIND=r8)   , INTENT(OUT  ) :: variance
    REAL(KIND=r8)   , INTENT(OUT  ) :: Cdelta
    REAL(KIND=r8)   , INTENT(OUT  ) :: psi0

    INTEGER         :: i,j,m,status,javg1,javg2,navg
    REAL(KIND=r8)   :: pi,freq1,dofmin,gammafac,dj0,Savg,Smid
    REAL(KIND=r8)   :: fft_theor1
    REAL(KIND=r8)   :: chisqr,p,q,bound

    pi = 4._r8*ATAN(1._r8)

    IF (siglvl.LE.0.) siglvl = 0.05_r8
    IF (lag1.LE.0._r8) lag1 = 0._r8

    Cdelta = -1._r8
    gammafac = -1._r8
    dj0 = -1._r8
    psi0 = -1._r8

    IF (mother.EQ.0) THEN
       !*******************************************   Morlet wavelet
       dofmin = 2._r8
       IF (param.EQ.6._r8) THEN
          Cdelta = 0.776_r8
          gammafac = 2.32_r8
          dj0 = 0.60_r8
          psi0 = pi**(-0.25_r8)
       END IF
    ELSE IF (mother.EQ.1) THEN
       !*******************************************   Paul wavelet
       m = INT(param)
       dofmin = 2._r8
       IF (m.EQ.4) THEN
          Cdelta = 1.132_r8
          gammafac = 1.17_r8
          dj0 = 1.5_r8
          psi0 = 1.079_r8
       END IF
    ELSE IF (mother.EQ.2) THEN
       !*******************************************   DOG wavelet
       m = INT(param)
       dofmin = 1._r8
       IF (m.EQ.2) THEN
          Cdelta = 3.541_r8
          gammafac = 1.43_r8
          dj0 = 1.4_r8
          psi0 = 0.867_r8
       ELSE IF (m.EQ.6) THEN
          Cdelta = 1.966_r8
          gammafac = 1.37_r8
          dj0 = 0.97_r8
          psi0 = 0.884_r8
       END IF
    ELSE
       STOP
    END IF

    !** find the time-series variance
    ymean = 0._r8
    DO i=1,n
       ymean = ymean + y(i)
    END DO
    ymean = ymean/n
    variance = 0._r8
    DO i=1,n
       variance = variance + (y(i) - ymean)**2
    END DO
    variance = variance/(DBLE(n)) ! - 1._r8)

    !** construct theoretical red(white)-noise power spectrum [Eqn(16)]
    !              _    _ 2
    !             |      |
    !         1 - | alfa |
    !             |_    _|
    ! Pk =------------------------------------- 
    !              _    _2              _        _
    !             |      |             |  2*pi*k  |
    !         1 + | alfa | - 2*alfa*cos|--------- | 
    !             |_    _|             |_   N    _|
    !
    DO j=1,jtot
       freq1 = dt/period(j)
       fft_theor(j) = variance*(1._r8-lag1**2)/ &
            (1._r8 - 2._r8*lag1*COS(freq1*2._r8*pi) + lag1**2)
       P_theor(j)  = fft_theor(j) 
    END DO

    q = DBLE(siglvl)
    p = 1_r8 - q

    IF (isigtest.EQ.0) THEN
       !*******************************************   no smoothing, dof=dofmin
       !   see Eqn(18)
       !                               _   _ 2
       ! | Wn(s) | 2         1        |     |
       !   ------      ==> ----- Pk * | Chi |
       !         2           2        |_   _|
       !   signif
       ! 
      DO j=1,jtot
          !
          !  dof [REAL(KIND=r8)   (jtot)] = degrees-of-freedom that were actually used.
          !     IF SIGTEST=0, then dof(j) = 2 (or 1 for the 'DOG')
          !
          dof(j) = dofmin
          CALL CDFCHI(2    , &!! INTEGER         , INTENT(IN   ) :: which
               p           , &!! REAL(KIND=r8)   , INTENT(INOUT) :: p
               q           , &!! REAL(KIND=r8)   , INTENT(INOUT) :: q
               chisqr      , &!! REAL(KIND=r8)   , INTENT(INOUT) :: x
               dofmin      , &!! REAL(KIND=r8)   , INTENT(INOUT) :: df
               status      , &!! INTEGER         , INTENT(OUT  ) :: status
               bound         )!! REAL(KIND=r8)   , INTENT(OUT  ) :: bound 

          !
          !                               _   _ 2
          ! | Wn(s) | 2         1        |     |
          !   ------      ==> ----- Pk * | Chi |
          !         2           2        |_   _|
          !   signif
          ! 
          !     dofmin <--> Degrees of freedom of the
          !             chi-square distribution.
          !             Input range: (0, +infinity).
          !             Search range: [ 1E-300, 1E300]
          !                  REAL(KIND=r8)    DF
          !fft_theor = Variace^2 * Pk
          signif(j) = fft_theor(j)*chisqr/dofmin
       END DO
    ELSE IF (isigtest.EQ.1) THEN
          !***********************************   time-averaged, dof depend on scale
          !
          !  dof [REAL(KIND=r8)   (jtot)] = degrees-of-freedom that were actually used.
          !     IF SIGTEST=1, then dof(j) = degrees-of-freedom versus scale.
          !

       IF (gammafac.LE.0._r8) THEN
          PRINT*,'**WAVE_SIGNIF: "gammafac" undefined for this wavelet'
          RETURN
       END IF
       DO j=1,jtot
          IF (dof(j).LT.1.) dof(j) = 1._r8
          !   see Eqn(23)
          !               _____________________
          !              /    _           _2   \
          !        \    /    |  na*deltat  |
          ! v = 2 * \  / 1 + |-------------|
          !          \/      |_   gamas   _|
          !
          dof(j) = dofmin*SQRT(1._r8+(dof(j)*dt/gammafac/SCALE(j))**2)
          IF (dof(j).LT.dofmin) dof(j) = dofmin
          CALL CDFCHI(2           , &!! INTEGER              , INTENT(IN   ) :: which
                      p           , &!! REAL(KIND=r8)        , INTENT(INOUT) :: p
                      q           , &!! REAL(KIND=r8)        , INTENT(INOUT) :: q
                      chisqr      , &!! REAL(KIND=r8)        , INTENT(INOUT) :: x
                      dof(j)      , &!! REAL(KIND=r8)        , INTENT(INOUT) :: df
                      status      , &!! INTEGER              , INTENT(OUT  ) :: status
                      bound         )!! REAL(KIND=r8)        , INTENT(OUT  ) :: bound 
       !                               _   _ 2
       ! | Wn(s) | 2         1        |     |
       !   ------      ==> ----- Pk * | Chi |
       !         2           2        |_   _|
       !   signif
       ! 
       !     dof <--> Degrees of freedom of the
       !             chi-square distribution.
       !             Input range: (0, +infinity).
       !             Search range: [ 1E-300, 1E300]
       !                  REAL(KIND=r8)    DF

          signif(j) = fft_theor(j)*chisqr/dof(j)
       END DO
    ELSE IF (isigtest.EQ.2) THEN
       !***********************************   scale-averaged, dof depend on scale
       !
       !  dof [REAL(KIND=r8)   (jtot)] = degrees-of-freedom that were actually used.
       !     IF SIGTEST=2, then dof(1) = degrees-of-freedom, dof(2...jtot)=0.0
       !

       IF (Cdelta.LE.0.) THEN
          PRINT*,'**WAVE_SIGNIF: "Cdelta" and "dj0" '// &
               'undefined for this wavelet'
          RETURN
       END IF
       javg1 = 0
       javg2 = 0
       DO j=1,jtot
          IF ((SCALE(j).GE.dof(1)).AND.(javg1.EQ.0)) javg1 = j
          IF (SCALE(j).LE.dof(2)) javg2 = j
       END DO
       IF ((javg1.EQ.0).OR.(javg2.EQ.0).OR.(javg1.GT.javg2)) THEN
          PRINT*,'**WAVE_SIGNIF: Scales in "dof(1)" & "dof(2)" '// &
               'are out of range.'
          RETURN
       END IF
       navg = javg2 - javg1 + 1
       !   see Eqn(25)
       !         __         __ -1
       !        |  _______j2   |
       !        |  \      1    |
       ! Savg = |   \ -------- |
       !        |   /     Sj   |
       !        |  /______     |
       !        |__    j=j1  __|  
       !
       Savg = 0._r8
       DO j=javg1,javg2
          Savg = Savg + 1._r8/SCALE(j)
       END DO
       Savg = 1._r8/Savg
       !   see Eqn(27)
       !
       !             __         __ 
       !            |  _______j2   |
       ! _          |  \      Pj   |
       ! P = Savg * |   \ -------- |
       !            |   /     Sj   |
       !            |  /______     |
       !            |__    j=j1  __|  
       !


       fft_theor1 = 0._r8
       DO  j=javg1,javg2
          fft_theor1 = fft_theor1 + fft_theor(j)/SCALE(j)
       END DO

       fft_theor(1) = Savg*fft_theor1

       !   see Eqn(28)
       !                                       _________________________
       !                                      /                         \
       !                            \        /                     2
       !               2*navg* Savg  \      /          | navg*dj  |
       !   v     = -----------------  \    /     1   + |----------|
       !                  Smid         \  /            |    dj0   |
       !                                \/
       !                                  
       !

       Smid = EXP(0.5_r8*(LOG(SCALE(javg1)) + LOG(SCALE(javg2))))
       dof(1) = (dofmin*navg*Savg/Smid)*SQRT(1 + (navg*dj/dj0)**2)
       !
       !   see Eqn(26)
       !       
       !             _____2                 _     _ 2
       ! Cd * Savg * Wn(s)          ___    |_ Chiv_|
       ! -----------------      ==>  P   * -----------
       !               2                       v
       !  dj*dt* signif
       ! 

       CALL CDFCHI(2           , &!! INTEGER         , INTENT(IN   ) :: which
                   p           , &!! REAL(KIND=r8)   , INTENT(INOUT) :: p
                   q           , &!! REAL(KIND=r8)   , INTENT(INOUT) :: q
                   chisqr      , &!! REAL(KIND=r8)   , INTENT(INOUT) :: x
                   dof(1)      , &!! REAL(KIND=r8)   , INTENT(INOUT) :: df
                   status      , &!! INTEGER         , INTENT(OUT  ) :: status
                   bound         )!! REAL(KIND=r8)   , INTENT(OUT  ) :: bound 

       signif(1)=(dj*dt/Cdelta/Savg)*fft_theor(1)*chisqr/dof(1)

       DO j=2,jtot
          dof(j)       = 0._r8
          fft_theor(j) = 0._r8
          signif(j)    = 0._r8
       END DO
    ELSE
       STOP
    END IF

  END SUBROUTINE WAVE_SIGNIF




  !**************************************************************************
  !      DCDFLIB: Available at http://www.netlib.org/random/
  !
  !   ***NOTE (C.T.) *** this is a subset of DCDFLIB, which only includes
  !            the routines used by the CHI SQUARE cumulative distribution.
  !
  ! Modified: November 1999 by Arjan van Dijk to include IMPLICIT NONE and
  !           to convert all routines to REAL(KIND=r8)   .
  !**************************************************************************




  !**************************************************************************
  !                                           DCDFLIB
  !                  Library of Fortran Routines for Cumulative Distribution
  !                       Functions, Inverses, and Other Parameters
  !                                    (February, 1994)
  !                            Compiled and Written by:
  !                                     Barry W. Brown
  !                                      James Lovato
  !                                      Kathy Russell
  !
  !
  !                              LEGALITIES
  !
  ! Code that appeared  in an    ACM  publication  is subject  to    their
  ! algorithms policy:
  !
  !        Submittal of  an  algorithm  for publication  in   one of   the  ACM
  !        Transactions implies that unrestricted use  of the algorithm within  a
  !        computer is permissible.     General permission  to copy and  distribute
  !        the algorithm without fee is granted provided that the copies        are not
  !        made  or     distributed for  direct   commercial  advantage.    The ACM
  !        copyright notice and the title of the publication and its date appear,
  !        and  notice is given that copying  is by permission of the Association
  !        for Computing Machinery.  To copy otherwise, or to republish, requires
  !        a fee and/or specific permission.
  !
  !        Krogh, F.  Algorithms        Policy.  ACM  Tran.   Math.  Softw.   13(1987),
  !        183-186.
  !
  ! We place the DCDFLIB code that we have written in the public domain.
  !
  !                                     NO WARRANTY
  !
  !        WE PROVIDE ABSOLUTELY        NO WARRANTY  OF ANY  KIND  EITHER  EXPRESSED OR
  !        IMPLIED,  INCLUDING BUT   NOT LIMITED TO,  THE  IMPLIED  WARRANTIES OF
  !        MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.        THE ENTIRE RISK
  !        AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS  WITH YOU.  SHOULD
  !        THIS PROGRAM PROVE  DEFECTIVE, YOU ASSUME  THE COST  OF  ALL NECESSARY
  !        SERVICING, REPAIR OR CORRECTION.
  !
  !        IN NO        EVENT  SHALL THE UNIVERSITY  OF TEXAS OR  ANY  OF ITS COMPONENT
  !        INSTITUTIONS INCLUDING M. D. ANDERSON HOSPITAL BE LIABLE  TO YOU FOR
  !        DAMAGES, INCLUDING ANY  LOST PROFITS, LOST MONIES,   OR OTHER SPECIAL,
  !        INCIDENTAL   OR  CONSEQUENTIAL DAMAGES   ARISING     OUT  OF  THE USE OR
  !        INABILITY TO USE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA OR
  !        ITS ANALYSIS BEING  RENDERED INACCURATE OR  LOSSES SUSTAINED  BY THIRD
  !        PARTIES) THE PROGRAM.
  !
  !        (Above NO WARRANTY modified from the GNU NO WARRANTY statement.)
  !


  SUBROUTINE cdfchi(which  , &! INTEGER                , INTENT(IN   ) :: which
                    p      , &! REAL(KIND=r8)   , INTENT(INOUT) :: p
                    q      , &! REAL(KIND=r8)   , INTENT(INOUT) :: q
                    x      , &! REAL(KIND=r8)   , INTENT(INOUT) :: x
                    df     , &! REAL(KIND=r8)   , INTENT(INOUT) :: df
                    status , &! INTEGER                , INTENT(OUT  ) :: status
                    bound    )! REAL(KIND=r8)   , INTENT(OUT  ) :: bound 
    !**********************************************************************
    !
    !      SUBROUTINE CDFCHI( WHICH, P, Q, X, DF, STATUS, BOUND )
    !             Cumulative Distribution Function
    !             CHI-Square distribution
    !
    !
    !                         Function
    !
    !
    !     Calculates any one parameter of the chi-square
    !     distribution given values for the others.
    !
    !
    !                         Arguments
    !
    !
    !     WHICH --> Integer indicating which of the next three argument
    !             values is to be calculated from the others.
    !             Legal range: 1..3
    !             iwhich = 1 : Calculate P and Q from X and DF
    !             iwhich = 2 : Calculate X from P,Q and DF
    !             iwhich = 3 : Calculate DF from P,Q and X
    !                  INTEGER WHICH
    !
    !     P <--> The integral from 0 to X of the chi-square
    !            distribution.
    !            Input range: [0, 1].
    !                  REAL(KIND=r8)    P
    !
    !     Q <--> 1-P.
    !            Input range: (0, 1].
    !            P + Q = 1.0.
    !                  REAL(KIND=r8)    Q
    !
    !     X <--> Upper limit of integration of the non-central
    !            chi-square distribution.
    !            Input range: [0, +infinity).
    !            Search range: [0,1E300]
    !                  REAL(KIND=r8)    X
    !
    !     DF <--> Degrees of freedom of the
    !             chi-square distribution.
    !             Input range: (0, +infinity).
    !             Search range: [ 1E-300, 1E300]
    !                  REAL(KIND=r8)    DF
    !
    !     STATUS <-- 0 if calculation completed correctly
    !             -I if input parameter number I is out of range
    !              1 if answer appears to be lower than lowest
    !                search bound
    !              2 if answer appears to be higher than greatest
    !                search bound
    !              3 if P + Q .ne. 1
    !             10 indicates error returned from cumgam.  See
    !                references in cdfgam
    !                  INTEGER STATUS
    !
    !     BOUND <-- Undefined if STATUS is 0
    !
    !             Bound exceeded by parameter number I if STATUS
    !             is negative.
    !
    !             Lower search bound if STATUS is 1.
    !
    !             Upper search bound if STATUS is 2.
    !
    !
    !                         Method
    !
    !
    !     Formula    26.4.19   of Abramowitz  and       Stegun, Handbook  of
    !     Mathematical Functions   (1966) is used   to reduce the chisqure
    !     distribution to the incomplete distribution.
    !
    !     Computation of other parameters involve a seach for a value that
    !     produces  the desired  value  of P.   The search relies  on  the
    !     monotinicity of P with the other parameter.
    !
    !**********************************************************************
    IMPLICIT NONE
    INTEGER         , INTENT(OUT  ) :: status
    INTEGER         , INTENT(IN   ) :: which
    REAL(KIND=r8)   , INTENT(INOUT) :: p
    REAL(KIND=r8)   , INTENT(INOUT) :: q
    REAL(KIND=r8)   , INTENT(INOUT) :: x
    REAL(KIND=r8)   , INTENT(OUT  ) :: bound
    REAL(KIND=r8)   , INTENT(INOUT) :: df

    !     .. Parameters ..

    REAL(KIND=r8)   , PARAMETER :: tol=1.0D-8
    REAL(KIND=r8)   , PARAMETER :: atol=1.0D-50
    REAL(KIND=r8)   , PARAMETER :: zero=1.0D-300
    REAL(KIND=r8)   , PARAMETER :: inf=1.0D300
    !     ..
    !     .. Scalar Arguments ..
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    fx,cum,ccum,pq,porq
    LOGICAL qhi,qleft,qporq
    !     ..
    !     .. External Functions ..
    !REAL(KIND=r8)    spmpar
    !EXTERNAL spmpar
    !     ..
    !     .. External Subroutines ..
    !EXTERNAL dinvr,dstinv,cumchi
    !     ..
    !     .. Executable Statements ..
    !
    !     Check arguments
    !
    IF (.NOT. ((which.LT.1).OR. (which.GT.3))) GO TO 30
    IF (.NOT. (which.LT.1)) GO TO 10
    bound = 1.0_r8
    GO TO 20

10  bound = 3.0_r8
20  status = -1
    RETURN

30  IF (which.EQ.1) GO TO 70
    !
    !     P
    !
    IF (.NOT. ((p.LT.0.0_r8).OR. (p.GT.1.0_r8))) GO TO 60
    IF (.NOT. (p.LT.0.0_r8)) GO TO 40
    bound = 0.0_r8
    GO TO 50

40  bound = 1.0_r8
50  status = -2
    RETURN

60  CONTINUE
70  IF (which.EQ.1) GO TO 110
    !
    !     Q
    !
    IF (.NOT. ((q.LE.0.0_r8).OR. (q.GT.1.0_r8))) GO TO 100
    IF (.NOT. (q.LE.0.0_r8)) GO TO 80
    bound = 0.0_r8
    GO TO 90

80  bound = 1.0_r8
90  status = -3
    RETURN

100 CONTINUE
110 IF (which.EQ.2) GO TO 130
    !
    !     X
    !
    IF (.NOT. (x.LT.0.0_r8)) GO TO 120
    bound = 0.0_r8
    status = -4
    RETURN

120 CONTINUE
130 IF (which.EQ.3) GO TO 150
    !
    !     DF
    !
    IF (.NOT. (df.LE.0.0_r8)) GO TO 140
    bound = 0.0_r8
    status = -5
    RETURN

140 CONTINUE
150 IF (which.EQ.1) GO TO 190
    !
    !     P + Q
    !
    pq = p + q
    IF (.NOT. (ABS(((pq)-0.5_r8)-0.5_r8).GT. &
         (3.0_r8*spmpar(1)))) GO TO 180
    IF (.NOT. (pq.LT.0.0_r8)) GO TO 160
    bound = 0.0_r8
    GO TO 170

160 bound = 1.0_r8
170 status = 3
    RETURN

180 CONTINUE
190 IF (which.EQ.1) GO TO 220
    !
    !     Select the minimum of P or Q
    !
    qporq = p .LE. q
    IF (.NOT. (qporq)) GO TO 200
    porq = p
    GO TO 210

200 porq = q
210 CONTINUE
    !
    !     Calculate ANSWERS
    !
220 IF ((1).EQ. (which)) THEN
       !
       !     Calculating P and Q
       !
       status = 0
       CALL cumchi(x    , &!REAL(KIND=r8)   , INTENT(IN   ) :: x
                   df   , &!REAL(KIND=r8)   , INTENT(IN   ) :: df
                   p    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: cum
                   q      )!REAL(KIND=r8)   , INTENT(INOUT) :: ccum
       IF (porq.GT.1.5_r8) THEN
          status = 10
          RETURN

       END IF

    ELSE IF ((2).EQ. (which)) THEN
       !
       !     Calculating X
       !
       x = 5.0_r8
       CALL dstinv(0.0_r8   , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            inf             , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            0.5_r8          , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            0.5_r8          , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            5.0_r8          , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            atol            , &!REAL(KIND=r8)   , INTENT(IN   ) ::
            tol               )!REAL(KIND=r8)   , INTENT(IN   ) ::
       status = 0
       CALL dinvr(status  , &  !INTEGER         , INTENT(OUT   ) :: status
            x             , &  !REAL(KIND=r8)   , INTENT(OUT   ) :: x
            fx            , &  !REAL(KIND=r8)   , INTENT(OUT   ) :: fx
            qleft         , &  !LOGICAL         , INTENT(OUT   ) :: qhi
            qhi             )  !LOGICAL         , INTENT(OUT   ) :: qleft
230    IF (.NOT. (status.EQ.1)) GO TO 270
       CALL cumchi(x      , &!REAL(KIND=r8)   , INTENT(IN   ) :: x
                   df     , &!REAL(KIND=r8)   , INTENT(IN   ) :: df
                   cum    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: cum
                   ccum     )!REAL(KIND=r8)   , INTENT(INOUT) :: ccum
       IF (.NOT. (qporq)) GO TO 240
       fx = cum - p
       GO TO 250

240    fx = ccum - q
250    IF (.NOT. ((fx+porq).GT.1.5_r8)) GO TO 260
       status = 10
       RETURN

260    CALL dinvr(status  , &!INTEGER        , INTENT(OUT   ) :: status
            x             , &!REAL(KIND=r8)  , INTENT(OUT   ) :: x
            fx            , &!REAL(KIND=r8)  , INTENT(OUT   ) :: fx
            qleft         , &!LOGICAL        , INTENT(OUT   ) :: qhi
            qhi             )!LOGICAL        , INTENT(OUT   ) :: qleft
       GO TO 230

270    IF (.NOT. (status.EQ.-1)) GO TO 300
       IF (.NOT. (qleft)) GO TO 280
       status = 1
       bound = 0.0_r8
       GO TO 290

280    status = 2
       bound = inf
290    CONTINUE
300    CONTINUE

    ELSE IF ((3).EQ. (which)) THEN
       !
       !     Calculating DF
       !
       df = 5.0_r8
       CALL dstinv(zero  , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            inf          , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            0.5_r8       , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            0.5_r8       , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            5.0_r8       , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            atol         , & !REAL(KIND=r8)   , INTENT(IN   ) ::
            tol            ) !REAL(KIND=r8)   , INTENT(IN   ) ::
       status = 0
       CALL dinvr(status , &!INTEGER        , INTENT(OUT   ) :: status
            df           , &!REAL(KIND=r8)  , INTENT(OUT   ) :: x
            fx           , &!REAL(KIND=r8)  , INTENT(OUT   ) :: fx
            qleft        , &!LOGICAL        , INTENT(OUT   ) :: qhi
            qhi            )!LOGICAL        , INTENT(OUT   ) :: qleft
310    IF (.NOT. (status.EQ.1)) GO TO 350
       CALL cumchi(x    , &!REAL(KIND=r8)   , INTENT(IN   ) :: x
                   df   , &!REAL(KIND=r8)   , INTENT(IN   ) :: df
                   cum  , &!REAL(KIND=r8)   , INTENT(OUT  ) :: cum
                   ccum   )!REAL(KIND=r8)   , INTENT(INOUT) :: ccum
       IF (.NOT. (qporq)) GO TO 320
       fx = cum - p
       GO TO 330

320    fx = ccum - q
330    IF (.NOT. ((fx+porq).GT.1.5_r8)) GO TO 340
       status = 10
       RETURN

340    CALL dinvr(status, &!INTEGER        , INTENT(OUT   ) ::  status
            df    , &!REAL(KIND=r8)   , INTENT(OUT   ) ::  x
            fx    , &!REAL(KIND=r8)   , INTENT(OUT   ) ::  fx
            qleft , &!LOGICAL        , INTENT(OUT   ) :: qhi
            qhi     )!LOGICAL        , INTENT(OUT   ) :: qleft
       GO TO 310

350    IF (.NOT. (status.EQ.-1)) GO TO 380
       IF (.NOT. (qleft)) GO TO 360
       status = 1
       bound = zero
       GO TO 370

360    status = 2
       bound = inf
370    CONTINUE
380 END IF

    RETURN

  END  SUBROUTINE cdfchi




  SUBROUTINE cumchi(x   , &!REAL(KIND=r8)   , INTENT(IN   ) :: x
                   df   , &!REAL(KIND=r8)   , INTENT(IN   ) :: df
                   cum  , &!REAL(KIND=r8)   , INTENT(OUT  ) :: cum
                   ccum   )!REAL(KIND=r8)   , INTENT(INOUT) :: ccum
    !**********************************************************************
    !
    !     SUBROUTINE FUNCTION CUMCHI(X,DF,CUM,CCUM)
    !             CUMulative of the CHi-square distribution
    !
    !
    !                         Function
    !
    !
    !     Calculates the cumulative chi-square distribution.
    !
    !
    !                         Arguments
    !
    !
    !     X       --> Upper limit of integration of the
    !               chi-square distribution.
    !                             X is REAL(KIND=r8)   
    !
    !     DF      --> Degrees of freedom of the
    !               chi-square distribution.
    !                             DF is REAL(KIND=r8)   
    !
    !     CUM <-- Cumulative chi-square distribution.
    !                             CUM is DOUBLE PRECISIO
    !
    !     CCUM <-- Compliment of Cumulative chi-square distribution.
    !                             CCUM is DOUBLE PRECISI
    !
    !
    !                         Method
    !
    !
    !     Calls incomplete gamma function (CUMGAM)
    !
    !**********************************************************************
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)   , INTENT(IN   ) :: df
    REAL(KIND=r8)   , INTENT(IN   ) :: x
    REAL(KIND=r8)   , INTENT(OUT  ) :: cum
    REAL(KIND=r8)   , INTENT(INOUT) :: ccum
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    a,xx
    !     ..
    !     .. External Subroutines ..
    !EXTERNAL cumgam
    !     ..
    !     .. Executable Statements ..
    a = df*0.5_r8
    xx = x*0.5_r8
    CALL cumgam(xx    , & !REAL(KIND=r8)   , INTENT(IN   ) :: a 
         a     , & !REAL(KIND=r8)   , INTENT(IN   ) :: x
         cum   , & !REAL(KIND=r8)   , INTENT(OUT  ) :: cum
         ccum    ) !REAL(KIND=r8)   , INTENT(INOUT) :: ccum
    RETURN

  END SUBROUTINE cumchi



  SUBROUTINE cumgam(x    , & !REAL(KIND=r8)   , INTENT(IN   ) :: a 
       a    , & !REAL(KIND=r8)   , INTENT(IN   ) :: x
       cum  , & !REAL(KIND=r8)   , INTENT(OUT  ) :: cum
       ccum   ) !REAL(KIND=r8)   , INTENT(INOUT) :: ccum
    !**********************************************************************
    !
    !     SUBROUTINE CUMGAM(X,A,CUM,CCUM)
    !           REAL(KIND=r8)    cUMulative incomplete GAMma distribution
    !
    !
    !                         Function
    !
    !
    !     Computes   the  cumulative of    the     incomplete   gamma
    !     distribution, i.e., the integral from 0 to X of
    !          (1/GAM(A))*EXP(-T)*T**(A-1) DT
    !     where GAM(A) is the complete gamma function of A, i.e.,
    !          GAM(A) = integral from 0 to infinity of
    !                  EXP(-T)*T**(A-1) DT
    !
    !
    !                         Arguments
    !
    !
    !     X --> The upper limit of integration of the incomplete gamma.
    !                            X is REAL(KIND=r8)   
    !
    !     A --> The shape parameter of the incomplete gamma.
    !                            A is REAL(KIND=r8)   
    !
    !     CUM <-- Cumulative incomplete gamma distribution.
    !                          CUM is REAL(KIND=r8)   
    !
    !     CCUM <-- Compliment of Cumulative incomplete gamma distribution.
    !                            CCUM is DOUBLE PRECISIO
    !
    !
    !                         Method
    !
    !
    !     Calls the routine GRATIO.
    !
    !**********************************************************************
    IMPLICIT NONE
    !
    !     ..
    !     .. Scalar Arguments ..
    REAL(KIND=r8)   , INTENT(IN   ) :: a 
    REAL(KIND=r8)   , INTENT(IN   ) :: x
    REAL(KIND=r8)   , INTENT(OUT  ) :: cum
    REAL(KIND=r8)   , INTENT(INOUT) :: ccum
    !     :: ..
    !     .. External Routines ..
    !EXTERNAL gratio
    !     ..
    !     .. Executable Statements ..
    IF (.NOT. (x.LE.0.0_r8)) GO TO 10
    cum = 0.0_r8
    ccum = 1.0_r8
    RETURN

10  CALL gratio(a    , &!REAL(KIND=r8)   , INTENT(IN   ) :: a
         x    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: ans
         cum  , &!REAL(KIND=r8)   , INTENT(OUT  ) :: qans
         ccum , &!REAL(KIND=r8)   , INTENT(IN   ) :: x
         0      )!INTEGER          , INTENT(IN        ) :: ind

    !     Call gratio routine

    RETURN

  END SUBROUTINE cumgam



  SUBROUTINE dinvr(status  , &!INTEGER          , INTENT(OUT   ) ::  status
       x       , &!REAL(KIND=r8)   , INTENT(OUT   ) ::  x
       fx      , &!REAL(KIND=r8)   , INTENT(OUT   ) ::  fx
       qleft   , &!LOGICAL          , INTENT(OUT   ) :: qhi
       qhi       )!LOGICAL          , INTENT(OUT   ) :: qleft
    !**********************************************************************
    !
    !     SUBROUTINE DINVR(STATUS, X, FX, QLEFT, QHI)
    !          REAL(KIND=r8)   
    !          bounds the zero of the function and invokes zror
    !                  Reverse Communication
    !
    !
    !                         Function
    !
    !
    !     Bounds the    function  and  invokes  ZROR   to perform the   zero
    !     finding.  STINVR   must  have   been  called  before this     routine
    !     in order to set its parameters.
    !
    !
    !                         Arguments
    !
    !
    !     STATUS <--> At the beginning of a zero finding problem, STATUS
    !               should be set to 0 and INVR invoked.       (The value
    !               of parameters other than X will be ignored on this cal
    !
    !               When INVR needs the function evaluated, it will set
    !               STATUS to 1 and return.  The value of the function
    !               should be set in FX and INVR again called without
    !               changing any of its other parameters.
    !
    !               When INVR has finished without error, it will return
    !               with STATUS 0.  In that case X is approximately a root
    !               of F(X).
    !
    !               If INVR cannot bound the function, it returns status
    !               -1 and sets QLEFT and QHI.
    !                    INTEGER STATUS
    !
    !     X <-- The value of X at which F(X) is to be evaluated.
    !                    REAL(KIND=r8)    X
    !
    !     FX --> The value of F(X) calculated when INVR returns with
    !            STATUS = 1.
    !                    REAL(KIND=r8)    FX
    !
    !     QLEFT <-- Defined only if QMFINV returns .FALSE.  In that
    !          case it is .TRUE. If the stepping search terminated
    !          unsucessfully at SMALL.  If it is .FALSE. the search
    !          terminated unsucessfully at BIG.
    !                  QLEFT is LOGICAL
    !
    !     QHI <-- Defined only if QMFINV returns .FALSE.  In that
    !          case it is .TRUE. if F(X) .GT. Y at the termination
    !          of the search and .FALSE. if F(X) .LT. Y at the
    !          termination of the search.
    !                  QHI is LOGICAL

    !
    !**********************************************************************
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    INTEGER         , INTENT(OUT   ) ::  status
    REAL(KIND=r8)   , INTENT(OUT   ) ::  x
    REAL(KIND=r8)   , INTENT(INOUT ) ::  fx
    LOGICAL         , INTENT(OUT   ) :: qhi
    LOGICAL         , INTENT(OUT   ) :: qleft

    REAL(KIND=r8)    :: zabsst,zabsto,zbig,zrelst,zrelto,zsmall, &
         zstpmu

    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    :: absstp,abstol,big,fbig,fsmall,relstp,reltol, &
         small,step,stpmul,xhi,xlb,xlo,xsave,xub,yy,zx,zy, &
         zz
    INTEGER :: i99999
    LOGICAL :: qbdd,qcond,qdum1,qdum2,qincr,qlim,qup
    !     ..
    !     .. External Subroutines ..
    !EXTERNAL dstzr,dzror
    !     ..
    !     .. Intrinsic Functions ..
    !INTRINSIC abs,max,min
    !     ..
    !     .. Statement Functions ..
    LOGICAL :: qxmon
    !     ..
    !     .. Save statement ..
    SAVE
    !     ..
    !     .. Statement Function definitions ..
    qxmon(zx,zy,zz) = zx .LE. zy .AND. zy .LE. zz
    !     ..
    !     .. Executable Statements ..

    IF (status.GT.0) GO TO 310

    qcond = .NOT. qxmon(small,x,big)
    IF (qcond) STOP ' SMALL, X, BIG not monotone in INVR'
    xsave = x
    !
    !     See that SMALL and BIG bound the zero and set QINCR
    !
    x = small
    !     GET-FUNCTION-VALUE
    ASSIGN 10 TO i99999
    GO TO 300

10  fsmall = fx
    x = big
    !     GET-FUNCTION-VALUE
    ASSIGN 20 TO i99999
    GO TO 300

20  fbig = fx
    qincr = fbig .GT. fsmall
    IF (.NOT. (qincr)) GO TO 50
    IF (fsmall.LE.0.0_r8) GO TO 30
    status = -1
    qleft = .TRUE.
    qhi = .TRUE.
    RETURN

30  IF (fbig.GE.0.0_r8) GO TO 40
    status = -1
    qleft = .FALSE.
    qhi = .FALSE.
    RETURN

40  GO TO 80

50  IF (fsmall.GE.0.0_r8) GO TO 60
    status = -1
    qleft = .TRUE.
    qhi = .FALSE.
    RETURN

60  IF (fbig.LE.0.0_r8) GO TO 70
    status = -1
    qleft = .FALSE.
    qhi = .TRUE.
    RETURN

70  CONTINUE
80  x = xsave
    step = MAX(absstp,relstp*ABS(x))
    !      YY = F(X) - Y
    !     GET-FUNCTION-VALUE
    ASSIGN 90 TO i99999
    GO TO 300

90  yy = fx
    IF (.NOT. (yy.EQ.0.0_r8)) GO TO 100
    status = 0
    RETURN

100 qup = (qincr .AND. (yy.LT.0.0_r8)) .OR. &
         (.NOT.qincr .AND. (yy.GT.0.0_r8))
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !
    !     HANDLE CASE IN WHICH WE MUST STEP HIGHER
    !
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    IF (.NOT. (qup)) GO TO 170
    xlb = xsave
    xub = MIN(xlb+step,big)
    GO TO 120

110 IF (qcond) GO TO 150
    !      YY = F(XUB) - Y
120 x = xub
    !     GET-FUNCTION-VALUE
    ASSIGN 130 TO i99999
    GO TO 300

130 yy = fx
    qbdd = (qincr .AND. (yy.GE.0.0_r8)) .OR. &
         (.NOT.qincr .AND. (yy.LE.0.0_r8))
    qlim = xub .GE. big
    qcond = qbdd .OR. qlim
    IF (qcond) GO TO 140
    step = stpmul*step
    xlb = xub
    xub = MIN(xlb+step,big)
140 GO TO 110

150 IF (.NOT. (qlim.AND..NOT.qbdd)) GO TO 160
    status = -1
    qleft = .FALSE.
    qhi = .NOT. qincr
    x = big
    RETURN

160 GO TO 240
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !
    !     HANDLE CASE IN WHICH WE MUST STEP LOWER
    !
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
170 xub = xsave
    xlb = MAX(xub-step,small)
    GO TO 190

180 IF (qcond) GO TO 220
    !      YY = F(XLB) - Y
190 x = xlb
    !     GET-FUNCTION-VALUE
    ASSIGN 200 TO i99999
    GO TO 300

200 yy = fx
    qbdd = (qincr .AND. (yy.LE.0.0_r8)) .OR. &
         (.NOT.qincr .AND. (yy.GE.0.0_r8))
    qlim = xlb .LE. small
    qcond = qbdd .OR. qlim
    IF (qcond) GO TO 210
    step = stpmul*step
    xub = xlb
    xlb = MAX(xub-step,small)
210 GO TO 180

220 IF (.NOT. (qlim.AND..NOT.qbdd)) GO TO 230
    status = -1
    qleft = .TRUE.
    qhi = qincr
    x = small
    RETURN

230 CONTINUE
240 CALL dstzr(xlb,xub,abstol,reltol)
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !
    !     IF WE REACH HERE, XLB AND XUB BOUND THE ZERO OF F.
    !
    !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    status = 0
    GO TO 260

250 IF (.NOT. (status.EQ.1)) GO TO 290
260 CALL dzror(status , &!  INTEGER              , INTENT(INOUT) ::  status
         x      , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: X
         fx     , &!  REAL(KIND=r8)    , INTENT(IN   ) :: fx
         xlo    , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: xlo
         xhi    , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: xhi
         qdum1  , &!  LOGICAL              , INTENT(OUT  ) :: qleft
         qdum2    )!  LOGICAL              , INTENT(OUT  ) :: qhi
    IF (.NOT. (status.EQ.1)) GO TO 280
    !     GET-FUNCTION-VALUE
    ASSIGN 270 TO i99999
    GO TO 300

270 CONTINUE
280 GO TO 250

290 x = xlo
    status = 0
    RETURN

    ENTRY dstinv(zsmall,zbig,zabsst,zrelst,zstpmu,zabsto,zrelto)
    !**********************************************************************
    !
    !      SUBROUTINE DSTINV( SMALL, BIG, ABSSTP, RELSTP, STPMUL,
    !     +                  ABSTOL, RELTOL )
    !      REAL(KIND=r8)    - SeT INverse finder - Reverse Communication
    !
    !
    !                         Function
    !
    !
    !     Concise Description - Given a monotone function F finds X
    !     such that F(X) = Y.  Uses Reverse communication -- see invr.
    !     This routine sets quantities needed by INVR.
    !
    !          More Precise Description of INVR -
    !
    !     F must be a monotone function, the results of QMFINV are
    !     otherwise undefined.  QINCR must be .TRUE. if F is non-
    !     decreasing and .FALSE. if F is non-increasing.
    !
    !     QMFINV will return .TRUE. if and only if F(SMALL) and
    !     F(BIG) bracket Y, i. e.,
    !          QINCR is .TRUE. and F(SMALL).LE.Y.LE.F(BIG) or
    !          QINCR is .FALSE. and F(BIG).LE.Y.LE.F(SMALL)
    !
    !     if QMFINV returns .TRUE., then the X returned satisfies
    !     the following condition.  let
    !             TOL(X) = MAX(ABSTOL,RELTOL*ABS(X))
    !     then if QINCR is .TRUE.,
    !          F(X-TOL(X)) .LE. Y .LE. F(X+TOL(X))
    !     and if QINCR is .FALSE.
    !          F(X-TOL(X)) .GE. Y .GE. F(X+TOL(X))
    !
    !
    !                         Arguments
    !
    !
    !     SMALL --> The left endpoint of the interval to be
    !          searched for a solution.
    !                  SMALL is REAL(KIND=r8)   
    !
    !     BIG --> The right endpoint of the interval to be
    !          searched for a solution.
    !                  BIG is REAL(KIND=r8)   
    !
    !     ABSSTP, RELSTP --> The initial step size in the search
    !          is MAX(ABSSTP,RELSTP*ABS(X)). See algorithm.
    !                  ABSSTP is REAL(KIND=r8)   
    !                  RELSTP is REAL(KIND=r8)   
    !
    !     STPMUL --> When a step doesn't bound the zero, the step
    !              size is multiplied by STPMUL and another step
    !              taken.  A popular value is 2.0
    !                  REAL(KIND=r8)    STPMUL
    !
    !     ABSTOL, RELTOL --> Two numbers that determine the accuracy
    !          of the solution.  See function for a precise definition.
    !                  ABSTOL is REAL(KIND=r8)   
    !                  RELTOL is REAL(KIND=r8)   
    !
    !
    !                         Method
    !
    !
    !     Compares F(X) with Y for the input value of X then uses QINCR
    !     to determine whether to step left or right to bound the
    !     desired x.  the initial step size is
    !          MAX(ABSSTP,RELSTP*ABS(S)) for the input value of X.
    !     Iteratively steps right or left until it bounds X.
    !     At each step which doesn't bound X, the step size is doubled.
    !     The routine is careful never to step beyond SMALL or BIG.  If
    !     it hasn't bounded X at SMALL or BIG, QMFINV returns .FALSE.
    !     after setting QLEFT and QHI.
    !
    !     If X is successfully bounded then Algorithm R of the paper
    !     'Two Efficient Algorithms with Guaranteed Convergence for
    !     Finding a Zero of a Function' by J. C. P. Bus and
    !     T. J. Dekker in ACM Transactions on Mathematical
    !     Software, Volume 1, No. 4 page 330 (DEC. '75) is employed
    !     to find the zero of the function F(X)-Y. This is routine
    !     QRZERO.
    !
    !**********************************************************************
    small = zsmall
    big = zbig
    absstp = zabsst
    relstp = zrelst
    stpmul = zstpmu
    abstol = zabsto
    reltol = zrelto
    RETURN

    !      STOP '*** EXECUTION FLOWING INTO FLECS PROCEDURES ***'
    !     TO GET-FUNCTION-VALUE
300 status = 1
    RETURN

310 CONTINUE
    GO TO i99999

  END  SUBROUTINE dinvr



  SUBROUTINE dzror(status  , &!  INTEGER             , INTENT(INOUT) ::  status
       x       , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: X
       fx      , &!  REAL(KIND=r8)    , INTENT(IN   ) :: fx
       xlo     , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: xlo
       xhi     , &!  REAL(KIND=r8)    , INTENT(OUT  ) :: xhi
       qleft   , &!  LOGICAL             , INTENT(OUT  ) :: qleft
       qhi       )!  LOGICAL             , INTENT(OUT  ) :: qhi
    !**********************************************************************
    !
    !     SUBROUTINE DZROR(STATUS, X, FX, XLO, XHI, QLEFT, QHI)
    !     REAL(KIND=r8)    ZeRo of a function -- Reverse Communication
    !
    !
    !                         Function
    !
    !
    !     Performs the zero finding.  STZROR must have been called before
    !     this routine in order to set its parameters.
    !
    !
    !                         Arguments
    !
    !
    !     STATUS <--> At the beginning of a zero finding problem, STATUS
    !               should be set to 0 and ZROR invoked.       (The value
    !               of other parameters will be ignored on this call.)
    !
    !               When ZROR needs the function evaluated, it will set
    !               STATUS to 1 and return.  The value of the function
    !               should be set in FX and ZROR again called without
    !               changing any of its other parameters.
    !
    !               When ZROR has finished without error, it will return
    !               with STATUS 0.  In that case (XLO,XHI) bound the answe
    !
    !               If ZROR finds an error (which implies that F(XLO)-Y an
    !               F(XHI)-Y have the same sign, it returns STATUS -1.  In
    !               this case, XLO and XHI are undefined.
    !                    INTEGER STATUS
    !
    !     X <-- The value of X at which F(X) is to be evaluated.
    !                    REAL(KIND=r8)    X
    !
    !     FX --> The value of F(X) calculated when ZROR returns with
    !            STATUS = 1.
    !                    REAL(KIND=r8)    FX
    !
    !     XLO <-- When ZROR returns with STATUS = 0, XLO bounds the
    !             inverval in X containing the solution below.
    !                    REAL(KIND=r8)    XLO
    !
    !     XHI <-- When ZROR returns with STATUS = 0, XHI bounds the
    !             inverval in X containing the solution above.
    !                    REAL(KIND=r8)    XHI
    !
    !     QLEFT <-- .TRUE. if the stepping search terminated unsucessfully
    !              at XLO.  If it is .FALSE. the search terminated
    !              unsucessfully at XHI.
    !                  QLEFT is LOGICAL
    !
    !     QHI <-- .TRUE. if F(X) .GT. Y at the termination of the
    !              search and .FALSE. if F(X) .LT. Y at the
    !              termination of the search.
    !                  QHI is LOGICAL
    !
    !**********************************************************************
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    INTEGER          , INTENT(INOUT) ::  status
    REAL(KIND=r8)    , INTENT(OUT  ) :: X
    REAL(KIND=r8)    , INTENT(IN   ) :: fx
    REAL(KIND=r8)    , INTENT(OUT  ) :: xlo
    REAL(KIND=r8)    , INTENT(OUT  ) :: xhi
    LOGICAL          , INTENT(OUT  ) :: qleft
    LOGICAL          , INTENT(OUT  ) :: qhi

    REAL(KIND=r8)    zabstl,zreltl,zxhi,zxlo
    !     ..
    !     .. Save statement ..
    SAVE
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    a,abstol,b,c,d,fa,fb,fc,fd,fda,fdb,m,mb,p,q, &
         reltol,tol,w,xxhi,xxlo,zx
    INTEGER ext,i99999
    LOGICAL first,qrzero
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC abs,max,sign
    !     ..
    !     .. Statement Functions ..
    REAL(KIND=r8)    ftol
    !     ..
    !     .. Statement Function definitions ..
    ftol(zx) = 0.5_r8*MAX(abstol,reltol*ABS(zx))
    !     ..
    !     .. Executable Statements ..

    IF (status.GT.0) GO TO 280
    xlo = xxlo
    xhi = xxhi
    b = xlo
    x = xlo
    !     GET-FUNCTION-VALUE
    ASSIGN 10 TO i99999
    GO TO 270

10  fb = fx
    xlo = xhi
    a = xlo
    x = xlo
    !     GET-FUNCTION-VALUE
    ASSIGN 20 TO i99999
    GO TO 270
    !
    !     Check that F(ZXLO) < 0 < F(ZXHI)  or
    !              F(ZXLO) > 0 > F(ZXHI)
    !
20  IF (.NOT. (fb.LT.0.0_r8)) GO TO 40
    IF (.NOT. (fx.LT.0.0_r8)) GO TO 30
    status = -1
    qleft = fx .LT. fb
    qhi = .FALSE.
    RETURN

30  CONTINUE
40  IF (.NOT. (fb.GT.0.0_r8)) GO TO 60
    IF (.NOT. (fx.GT.0.0_r8)) GO TO 50
    status = -1
    qleft = fx .GT. fb
    qhi = .TRUE.
    RETURN

50  CONTINUE
60  fa = fx
    !
    first = .TRUE.
70  c = a
    fc = fa
    ext = 0
80  IF (.NOT. (ABS(fc).LT.ABS(fb))) GO TO 100
    IF (.NOT. (c.NE.a)) GO TO 90
    d = a
    fd = fa
90  a = b
    fa = fb
    xlo = c
    b = xlo
    fb = fc
    c = a
    fc = fa
100 tol = ftol(xlo)
    m = (c+b)*.5_r8
    mb = m - b
    IF (.NOT. (ABS(mb).GT.tol)) GO TO 240
    IF (.NOT. (ext.GT.3)) GO TO 110
    w = mb
    GO TO 190

110 tol = SIGN(tol,mb)
    p = (b-a)*fb
    IF (.NOT. (first)) GO TO 120
    q = fa - fb
    first = .FALSE.
    GO TO 130

120 fdb = (fd-fb)/ (d-b)
    fda = (fd-fa)/ (d-a)
    p = fda*p
    q = fdb*fa - fda*fb
130 IF (.NOT. (p.LT.0.0_r8)) GO TO 140
    p = -p
    q = -q
140 IF (ext.EQ.3) p = p*2.0_r8
    IF (.NOT. ((p*1.0_r8).EQ.0.0_r8.OR.p.LE. (q*tol))) GO TO 150
    w = tol
    GO TO 180

150 IF (.NOT. (p.LT. (mb*q))) GO TO 160
    w = p/q
    GO TO 170

160 w = mb
170 CONTINUE
180 CONTINUE
190 d = a
    fd = fa
    a = b
    fa = fb
    b = b + w
    xlo = b
    x = xlo
    !     GET-FUNCTION-VALUE
    ASSIGN 200 TO i99999
    GO TO 270

200 fb = fx
    IF (.NOT. ((fc*fb).GE.0.0_r8)) GO TO 210
    GO TO 70

210 IF (.NOT. (w.EQ.mb)) GO TO 220
    ext = 0
    GO TO 230

220 ext = ext + 1
230 GO TO 80

240 xhi = c
    qrzero = (fc.GE.0.0_r8 .AND. fb.LE.0.0_r8) .OR. &
         (fc.LT.0.0_r8 .AND. fb.GE.0.0_r8)
    IF (.NOT. (qrzero)) GO TO 250
    status = 0
    GO TO 260

250 status = -1
260 RETURN

    ENTRY dstzr(zxlo,zxhi,zabstl,zreltl)

    !**********************************************************************
    !
    !     SUBROUTINE DSTZR( XLO, XHI, ABSTOL, RELTOL )
    !     REAL(KIND=r8)    SeT ZeRo finder - Reverse communication version
    !
    !
    !                         Function
    !
    !
    !
    !     Sets quantities needed by ZROR.  The function of ZROR
    !     and the quantities set is given here.
    !
    !     Concise Description - Given a function F
    !     find XLO such that F(XLO) = 0.
    !
    !          More Precise Description -
    !
    !     Input condition. F is a REAL(KIND=r8)    function of a single
    !     REAL(KIND=r8)    argument and XLO and XHI are such that
    !          F(XLO)*F(XHI)  .LE.        0.0
    !
    !     If the input condition is met, QRZERO returns .TRUE.
    !     and output values of XLO and XHI satisfy the following
    !          F(XLO)*F(XHI)  .LE. 0.
    !          ABS(F(XLO)  .LE. ABS(F(XHI)
    !          ABS(XLO-XHI)  .LE. TOL(X)
    !     where
    !          TOL(X) = MAX(ABSTOL,RELTOL*ABS(X))
    !
    !     If this algorithm does not find XLO and XHI satisfying
    !     these conditions then QRZERO returns .FALSE.  This
    !     implies that the input condition was not met.
    !
    !
    !                         Arguments
    !
    !
    !     XLO --> The left endpoint of the interval to be
    !           searched for a solution.
    !                  XLO is REAL(KIND=r8)   
    !
    !     XHI --> The right endpoint of the interval to be
    !           for a solution.
    !                  XHI is REAL(KIND=r8)   
    !
    !     ABSTOL, RELTOL --> Two numbers that determine the accuracy
    !                    of the solution.  See function for a
    !                    precise definition.
    !                  ABSTOL is REAL(KIND=r8)   
    !                  RELTOL is REAL(KIND=r8)   
    !
    !
    !                         Method
    !
    !
    !     Algorithm R of the paper 'Two Efficient Algorithms with
    !     Guaranteed Convergence for Finding a Zero of a Function'
    !     by J. C. P. Bus and T. J. Dekker in ACM Transactions on
    !     Mathematical Software, Volume 1, no. 4 page 330
    !     (Dec. '75) is employed to find the zero of F(X)-Y.
    !
    !**********************************************************************
    xxlo = zxlo
    xxhi = zxhi
    abstol = zabstl
    reltol = zreltl
    RETURN

    !      STOP '*** EXECUTION FLOWING INTO FLECS PROCEDURES ***'
    !     TO GET-FUNCTION-VALUE
270 status = 1
    RETURN

280 CONTINUE
    GO TO i99999

  END  SUBROUTINE dzror








  REAL(KIND=r8)    FUNCTION erf(x)
    !-----------------------------------------------------------------------
    !             EVALUATION OF THE REAL ERROR FUNCTION
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)    x
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    ax,bot,t,top,x2
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC abs,exp,sign
    !     ..
    !     .. Data statements ..
    !-------------------------
    !-------------------------
    !-------------------------
    !-------------------------
    REAL(KIND=r8)   , PARAMETER :: c = 0.564189583547756_r8
    REAL(KIND=r8)   , PARAMETER :: a(5)=(/&
         0.771058495001320D-04,-0.133733772997339D-02,&
         0.323076579225834D-01, 0.479137145607681D-01,&
         0.128379167095513D+00/)
    REAL(KIND=r8)   , PARAMETER :: b(3)=(/&
         0.301048631703895D-02,0.538971687740286D-01, &
         0.375795757275549D+00/)
    REAL(KIND=r8), PARAMETER :: p(8)=(/&
         -1.36864857382717e-07_r8,5.64195517478974e-01_r8,&
          7.21175825088309e+00_r8,4.31622272220567e+01_r8,&
          1.52989285046940e+02_r8,3.39320816734344e+02_r8,&
          4.51918953711873e+02_r8,3.00459261020162e+02_r8 /) 
    REAL(KIND=r8)   , PARAMETER :: q(8)=(/&
         1.00000000000000D+00,1.27827273196294D+01,&
         7.70001529352295D+01,2.77585444743988D+02,&
         6.38980264465631D+02,9.31354094850610D+02,&
         7.90950925327898D+02,3.00459260956983D+02/)
    REAL(KIND=r8)   , PARAMETER :: r(5)=(/&
         2.10144126479064D+00,2.62370141675169D+01,&
         2.13688200555087D+01,4.65807828718470D+00,&
         2.82094791773523D-01/)
    REAL(KIND=r8)   , PARAMETER :: s(4)=(/&
         9.41537750555460D+01,1.87114811799590D+02,&
         9.90191814623914D+01,1.80124575948747D+01/)
    !     ..
    !     .. Executable Statements ..
    !-------------------------
    ax = ABS(x)
    IF (ax.GT.0.5_r8) GO TO 10
    t = x*x
    top = ((((a(1)*t+a(2))*t+a(3))*t+a(4))*t+a(5)) + 1.0_r8
    bot = ((b(1)*t+b(2))*t+b(3))*t + 1.0_r8
    erf = x* (top/bot)
    RETURN
    !
10  IF (ax.GT.4.0_r8) GO TO 20
    top = ((((((p(1)*ax+p(2))*ax+p(3))*ax+p(4))*ax+p(5))*ax+p(6))*ax+ &
         p(7))*ax + p(8)
    bot = ((((((q(1)*ax+q(2))*ax+q(3))*ax+q(4))*ax+q(5))*ax+q(6))*ax+ &
         q(7))*ax + q(8)
    erf = 0.5_r8 + (0.5_r8-EXP(-x*x)*top/bot)
    IF (x.LT.0.0_r8) erf = -erf
    RETURN
    !
20  IF (ax.GE.5.8_r8) GO TO 30
    x2 = x*x
    t = 1.0_r8/x2
    top = (((r(1)*t+r(2))*t+r(3))*t+r(4))*t + r(5)
    bot = (((s(1)*t+s(2))*t+s(3))*t+s(4))*t + 1.0_r8
    erf = (c-top/ (x2*bot))/ax
    erf = 0.5_r8 + (0.5_r8-EXP(-x2)*erf)
    IF (x.LT.0.0_r8) erf = -erf
    RETURN
    !
30  erf = SIGN(1.0_r8,x)
    RETURN

  END  FUNCTION erf



  REAL(KIND=r8)    FUNCTION erfc1(ind,x)
    !-----------------------------------------------------------------------
    !         EVALUATION OF THE COMPLEMENTARY ERROR FUNCTION
    !
    !          ERFC1(IND,X) = ERFC(X)          IF IND = 0
    !          ERFC1(IND,X) = EXP(X*X)*ERFC(X)   OTHERWISE
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)    x
    INTEGER ind
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    ax,bot,e,t,top,w
    !     ..
    !     .. External Functions ..
    ! REAL(KIND=r8)    exparg
    ! EXTERNAL exparg
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC abs,dble,exp
    !     ..
    !     .. Data statements ..
    !-------------------------
    !-------------------------
    !-------------------------
    !-------------------------
    REAL(KIND=r8)    , PARAMETER :: c=0.564189583547756_r8
    REAL(KIND=r8)    , PARAMETER :: a(5)=(/&
         0.771058495001320D-04,-0.133733772997339D-02, &
         0.323076579225834D-01, 0.479137145607681D-01, &
         0.128379167095513D+00/)
    REAL(KIND=r8)    , PARAMETER ::  b(3)=(/&
         0.301048631703895D-02,0.538971687740286D-01, &
         0.375795757275549D+00/)
    REAL(KIND=r8)    , PARAMETER :: p(8) =(/&
         -1.36864857382717D-07,5.64195517478974D-01, &
         7.21175825088309D+00,4.31622272220567D+01, &
         1.52989285046940D+02,3.39320816734344D+02, &
         4.51918953711873D+02,3.00459261020162D+02/) 
    REAL(KIND=r8)    , PARAMETER :: q(8)=(/&
         1.00000000000000D+00,1.27827273196294D+01, &
         7.70001529352295D+01,2.77585444743988D+02, &
         6.38980264465631D+02,9.31354094850610D+02, &
         7.90950925327898D+02,3.00459260956983D+02/)
    REAL(KIND=r8)    , PARAMETER :: r(5) =(/&
         2.10144126479064D+00,2.62370141675169D+01, &
         2.13688200555087D+01,4.65807828718470D+00, &
         2.82094791773523D-01/)
    REAL(KIND=r8)    , PARAMETER :: s(4) =(/&
         9.41537750555460D+01,1.87114811799590D+02, &
         9.90191814623914D+01,1.80124575948747D+01/)
    !     ..
    !     .. Executable Statements ..
    !-------------------------
    !
    !                   ABS(X) .LE. 0.5
    !
    ax = ABS(x)
    IF (ax.GT.0.5_r8) GO TO 10
    t = x*x
    top = ((((a(1)*t+a(2))*t+a(3))*t+a(4))*t+a(5)) + 1.0_r8
    bot = ((b(1)*t+b(2))*t+b(3))*t + 1.0_r8
    erfc1 = 0.5_r8 + (0.5_r8-x* (top/bot))
    IF (ind.NE.0) erfc1 = EXP(t)*erfc1
    RETURN
    !
    !                0.5 .LT. ABS(X) .LE. 4
    !
10  IF (ax.GT.4.0_r8) GO TO 20
    top = ((((((p(1)*ax+p(2))*ax+p(3))*ax+p(4))*ax+p(5))*ax+p(6))*ax+ &
         p(7))*ax + p(8)
    bot = ((((((q(1)*ax+q(2))*ax+q(3))*ax+q(4))*ax+q(5))*ax+q(6))*ax+ &
         q(7))*ax + q(8)
    erfc1 = top/bot
    GO TO 40
    !
    !                    ABS(X) .GT. 4
    !
20  IF (x.LE.-5.6_r8) GO TO 60
    IF (ind.NE.0) GO TO 30
    IF (x.GT.100.0_r8) GO TO 70
    IF (x*x.GT.-exparg(1)) GO TO 70
    !
30  t = (1.0_r8/x)**2
    top = (((r(1)*t+r(2))*t+r(3))*t+r(4))*t + r(5)
    bot = (((s(1)*t+s(2))*t+s(3))*t+s(4))*t + 1.0_r8
    erfc1 = (c-t*top/bot)/ax
    !
    !                    FINAL ASSEMBLY
    !
40  IF (ind.EQ.0) GO TO 50
    IF (x.LT.0.0_r8) erfc1 = 2.0_r8*EXP(x*x) - erfc1
    RETURN

50  w = DBLE(x)*DBLE(x)
    t = w
    e = w - DBLE(t)
    erfc1 = ((0.5_r8+ (0.5_r8-e))*EXP(-t))*erfc1
    IF (x.LT.0.0_r8) erfc1 = 2.0_r8 - erfc1
    RETURN
    !
    !             LIMIT VALUE FOR LARGE NEGATIVE X
    !
60  erfc1 = 2.0_r8
    IF (ind.NE.0) erfc1 = 2.0_r8*EXP(x*x)
    RETURN
    !
    !             LIMIT VALUE FOR LARGE POSITIVE X
    !                  WHEN IND = 0
    !
70  erfc1 = 0.0_r8
    RETURN

  END FUNCTION erfc1











  REAL(KIND=r8)    FUNCTION exparg(l)
    !--------------------------------------------------------------------
    !     IF L = 0 THEN  EXPARG(L) = THE LARGEST POSITIVE W FOR WHICH
    !     EXP(W) CAN BE COMPUTED.
    !
    !     IF L IS NONZERO THEN  EXPARG(L) = THE LARGEST NEGATIVE W FOR
    !     WHICH THE COMPUTED VALUE OF EXP(W) IS NONZERO.
    !
    !     NOTE... ONLY AN APPROXIMATE VALUE FOR EXPARG(L) IS NEEDED.
    !--------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    INTEGER, INTENT(IN   ) :: l
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    :: lnb
    INTEGER b,m
    !     ..
    !     .. External Functions ..
    !INTEGER ipmpar
    !EXTERNAL ipmpar
    !     ..
    !     .. Intrinsic Functions ..
    !INTRINSIC dble,dlog
    !     ..
    !     .. Executable Statements ..
    !
    b = ipmpar(4)
    IF (b.NE.2) GO TO 10
    lnb = .69314718055995_r8
    GO TO 40

10  IF (b.NE.8) GO TO 20
    lnb = 2.0794415416798_r8
    GO TO 40

20  IF (b.NE.16) GO TO 30
    lnb = 2.7725887222398_r8
    GO TO 40

30  lnb = dlog(DBLE(b))
    !
40  IF (l.EQ.0) GO TO 50
    m = ipmpar(9) - 1
    exparg = 0.99999_r8* (m*lnb)
    RETURN

50  m = ipmpar(10)
    exparg = 0.99999_r8* (m*lnb)
    RETURN

  END FUNCTION exparg








  REAL(KIND=r8)    FUNCTION gam1(a)
    !     ------------------------------------------------------------------
    !     COMPUTATION OF 1/GAMMA(A+1) - 1  FOR -0.5 .LE. A .LE. 1.5
    !     ------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)    a
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    bot,d,t,top,w
    !     ..
    !     .. Data statements ..
    !     -------------------
    !     -------------------
    !     -------------------
    !     -------------------
    REAL(KIND=r8)   , PARAMETER :: p(7) =(/&
         0.577215664901533D+00,-0.409078193005776D+00, &
         -0.230975380857675D+00, 0.597275330452234D-01, &
         0.766968181649490D-02,-0.514889771323592D-02, &
         0.589597428611429D-03/)
    REAL(KIND=r8)   , PARAMETER :: q(5) =(/&
         0.100000000000000D+01,0.427569613095214D+00, &
         0.158451672430138D+00,0.261132021441447D-01, &
         0.423244297896961D-02/)
    REAL(KIND=r8)   , PARAMETER :: r(9)=(/&
         -0.422784335098468D+00,-0.771330383816272D+00,&
         -0.244757765222226D+00, 0.118378989872749D+00,&
          0.930357293360349D-03,-0.118290993445146D-01,&
          0.223047661158249D-02, 0.266505979058923D-03,&
         -0.132674909766242D-03/)
    REAL(KIND=r8)   , PARAMETER :: s1=0.273076135303957D+00
    REAL(KIND=r8)   , PARAMETER :: s2=0.559398236957378D-01
    !     ..
    !     .. Executable Statements ..
    !     -------------------
    t = a
    d = a - 0.5_r8
    IF (d.GT.0.0_r8) t = d - 0.5_r8
    IF (t) 40,10,20
    !
10  gam1 = 0.0_r8
    RETURN
    !
20  top = (((((p(7)*t+p(6))*t+p(5))*t+p(4))*t+p(3))*t+p(2))*t + p(1)
    bot = (((q(5)*t+q(4))*t+q(3))*t+q(2))*t + 1.0_r8
    w = top/bot
    IF (d.GT.0.0_r8) GO TO 30
    gam1 = a*w
    RETURN

30  gam1 = (t/a)* ((w-0.5_r8)-0.5_r8)
    RETURN
    !
40  top = (((((((r(9)*t+r(8))*t+r(7))*t+r(6))*t+r(5))*t+r(4))*t+r(3))* &
         t+r(2))*t + r(1)
    bot = (s2*t+s1)*t + 1.0_r8
    w = top/bot
    IF (d.GT.0.0_r8) GO TO 50
    gam1 = a* ((w+0.5_r8)+0.5_r8)
    RETURN

50  gam1 = t*w/a
    RETURN

  END FUNCTION gam1




  REAL(KIND=r8)    FUNCTION gamma(a)
    !-----------------------------------------------------------------------
    !
    !         EVALUATION OF THE GAMMA FUNCTION FOR REAL ARGUMENTS
    !
    !                      -----------
    !
    !     GAMMA(A) IS ASSIGNED THE VALUE 0 WHEN THE GAMMA FUNCTION CANNOT
    !     BE COMPUTED.
    !
    !-----------------------------------------------------------------------
    !     WRITTEN BY ALFRED H. MORRIS, JR.
    !          NAVAL SURFACE WEAPONS CENTER
    !          DAHLGREN, VIRGINIA
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)    a
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    bot,g,lnx,s,t,top,w,x,z
    INTEGER i,j,m,n
    !     ..
    !     .. External Functions ..
    !REAL(KIND=r8)    exparg,spmpar
    !EXTERNAL exparg,spmpar
    !     ..
    !     .. Intrinsic Functions ..
    !INTRINSIC abs,dble,dlog,exp,int,mod,sin
    !     ..
    !     .. Data statements ..
    !--------------------------
    !     D = 0.5*(LN(2*PI) - 1)
    !--------------------------
    !--------------------------
    !--------------------------
    REAL(KIND=r8)   , PARAMETER :: pi = 3.1415926535898_r8
    REAL(KIND=r8)   , PARAMETER :: d  = 0.41893853320467274178_r8
    REAL(KIND=r8)   , PARAMETER :: p(7)=(/&
         0.539637273585445D-03,0.261939260042690D-02, &
         0.204493667594920D-01,0.730981088720487D-01, &
         0.279648642639792D+00,0.553413866010467D+00, &
         1.0_r8/)
    REAL(KIND=r8)   , PARAMETER :: q(7) = (/&
         -0.832979206704073D-03, 0.470059485860584D-02, &
         0.225211131035340D-01,-0.170458969313360D+00, &
         -0.567902761974940D-01, 0.113062953091122D+01, &
         1.0_r8/)
    REAL(KIND=r8)   , PARAMETER :: r1=  0.820756370353826D-03
    REAL(KIND=r8)   , PARAMETER :: r2= -0.595156336428591D-03
    REAL(KIND=r8)   , PARAMETER :: r3=  0.793650663183693D-03
    REAL(KIND=r8)   , PARAMETER :: r4= -0.277777777770481D-02
    REAL(KIND=r8)   , PARAMETER :: r5=  0.833333333333333D-01
    !     ..
    !     .. Executable Statements ..
    !--------------------------
    gamma = 0.0_r8
    x = a
    IF (ABS(a).GE.15.0_r8) GO TO 110
    !-----------------------------------------------------------------------
    !            EVALUATION OF GAMMA(A) FOR ABS(A) .LT. 15
    !-----------------------------------------------------------------------
    t = 1.0_r8
    m = INT(a) - 1
    !
    !     LET T BE THE PRODUCT OF A-J WHEN A .GE. 2
    !
    IF (m) 40,30,10
10  DO j = 1,m
       x = x - 1.0_r8
       t = x*t
    END DO
30  x = x - 1.0_r8
    GO TO 80
    !
    !     LET T BE THE PRODUCT OF A+J WHEN A .LT. 1
    !
40  t = a
    IF (a.GT.0.0_r8) GO TO 70
    m = -m - 1
    IF (m.EQ.0) GO TO 60
    DO  j = 1,m
       x = x + 1.0_r8
       t = x*t
    END DO
60  x = (x+0.5_r8) + 0.5_r8
    t = x*t
    IF (t.EQ.0.0_r8) RETURN
    !
70  CONTINUE
    !
    !     THE FOLLOWING CODE CHECKS IF 1/T CAN OVERFLOW. THIS
    !     CODE MAY BE OMITTED IF DESIRED.
    !
    IF (ABS(t).GE.1.D-30) GO TO 80
    IF (ABS(t)*spmpar(3).LE.1.0001D0) RETURN
    gamma = 1.0_r8/t
    RETURN
    !
    !     COMPUTE GAMMA(1 + X) FOR  0 .LE. X .LT. 1
    !
80  top = p(1)
    bot = q(1)
    DO   i = 2,7
       top = p(i) + x*top
       bot = q(i) + x*bot
    END DO
    gamma = top/bot
    !
    !     TERMINATION
    !
    IF (a.LT.1.0_r8) GO TO 100
    gamma = gamma*t
    RETURN

100 gamma = gamma/t
    RETURN
    !-----------------------------------------------------------------------
    !            EVALUATION OF GAMMA(A) FOR ABS(A) .GE. 15
    !-----------------------------------------------------------------------
110 IF (ABS(a).GE.1.D3) RETURN
    IF (a.GT.0.0_r8) GO TO 120
    x = -a
    n = x
    t = x - n
    IF (t.GT.0.9D0) t = 1.0_r8 - t
    s = SIN(pi*t)/pi
    IF (MOD(n,2).EQ.0) s = -s
    IF (s.EQ.0.0_r8) RETURN
    !
    !     COMPUTE THE MODIFIED ASYMPTOTIC SUM
    !
120 t = 1.0_r8/ (x*x)
    g = ((((r1*t+r2)*t+r3)*t+r4)*t+r5)/x
    !
    !     ONE MAY REPLACE THE NEXT STATEMENT WITH  LNX = ALOG(X)
    !     BUT LESS ACCURACY WILL NORMALLY BE OBTAINED.
    !
    lnx = dlog(x)
    !
    !     FINAL ASSEMBLY
    !
    z = x
    g = (d+g) + (z-0.5D0)* (lnx-1.D0)
    w = g
    t = g - DBLE(w)
    IF (w.GT.0.99999D0*exparg(0)) RETURN
    gamma = EXP(w)* (1.0_r8+t)
    IF (a.LT.0.0_r8) gamma = (1.0_r8/ (gamma*s))/x
    RETURN

  END FUNCTION gamma








  SUBROUTINE gratio( a   , &!REAL(KIND=r8)   , INTENT(IN   ) :: a
       x   , &!REAL(KIND=r8)   , INTENT(OUT  ) :: ans
       ans , &!REAL(KIND=r8)   , INTENT(OUT  ) :: qans
       qans, &!REAL(KIND=r8)   , INTENT(IN   ) :: x
       ind   )!INTEGER         , INTENT(IN   ) :: ind
    ! ----------------------------------------------------------------------
    !        EVALUATION OF THE INCOMPLETE GAMMA RATIO FUNCTIONS
    !                    P(A,X) AND Q(A,X)
    !
    !                   ----------
    !
    !     IT IS ASSUMED THAT A AND X ARE NONNEGATIVE, WHERE A AND X
    !     ARE NOT BOTH 0.
    !
    !     ANS AND QANS ARE VARIABLES. GRATIO ASSIGNS ANS THE VALUE
    !     P(A,X) AND QANS THE VALUE Q(A,X). IND MAY BE ANY INTEGER.
    !     IF IND = 0 THEN THE USER IS REQUESTING AS MUCH ACCURACY AS
    !     POSSIBLE (UP TO 14 SIGNIFICANT DIGITS). OTHERWISE, IF
    !     IND = 1 THEN ACCURACY IS REQUESTED TO WITHIN 1 UNIT OF THE
    !     6-TH SIGNIFICANT DIGIT, AND IF IND .NE. 0,1 THEN ACCURACY
    !     IS REQUESTED TO WITHIN 1 UNIT OF THE 3RD SIGNIFICANT DIGIT.
    !
    !     ERROR RETURN ...
    !        ANS IS ASSIGNED THE VALUE 2 WHEN A OR X IS NEGATIVE,
    !     WHEN A*X = 0, OR WHEN P(A,X) AND Q(A,X) ARE INDETERMINANT.
    !     P(A,X) AND Q(A,X) ARE COMPUTATIONALLY INDETERMINANT WHEN
    !     X IS EXCEEDINGLY CLOSE TO A AND A IS EXTREMELY LARGE.
    ! ----------------------------------------------------------------------
    !     WRITTEN BY ALFRED H. MORRIS, JR.
    !        NAVAL SURFACE WEAPONS CENTER
    !        DAHLGREN, VIRGINIA
    !     --------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)   , INTENT(IN   ) :: a
    REAL(KIND=r8)   , INTENT(OUT  ) :: ans
    REAL(KIND=r8)   , INTENT(OUT  ) :: qans
    REAL(KIND=r8)   , INTENT(IN   ) :: x
    INTEGER         , INTENT(IN   ) :: ind
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    a2n,a2nm1,acc,am0,amn,an,an0,apn,b2n, &
         b2nm1,c,c0,c1,c2,c3,c4,c5,c6,cma, &
         e,e0,g,h,j,l,r,rta,rtx,s, &
         sum,t,t1,tol,twoa,u,w,x0,y,z
    INTEGER i,iop,m,max,n
    !     ..
    !     .. Local Arrays ..
    REAL(KIND=r8)    :: wk(20)
    !     ..
    !     .. External Functions ..
    !REAL(KIND=r8)    erf,erfc1,gam1,gamma,rexp,rlog,spmpar
    !EXTERNAL erf,erfc1,gam1,gamma,rexp,rlog,spmpar
    !     ..
    !     .. Intrinsic Functions ..
    !INTRINSIC abs,dble,dlog,dmax1,exp,int,sqrt
    !     ..
    !     .. Data statements ..
    !     --------------------
    !     --------------------
    !     ALOG10 = LN(10)
    !     RT2PIN = 1/SQRT(2*PI)
    !     RTPI   = SQRT(PI)
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    !     --------------------
    REAL(KIND=r8)   , PARAMETER :: acc0(3)=(/ 5.D-15 , 5.D-7  , 5.D-4 /)
    REAL(KIND=r8)   , PARAMETER :: big (3)=(/20.0_r8  ,14.0_r8  ,10.0_r8/)
    REAL(KIND=r8)   , PARAMETER :: e00 (3)=(/  .25D-3,  .25D-1,  .14D0/)
    REAL(KIND=r8)   , PARAMETER :: x00 (3)=(/31.0_r8  ,17.0_r8  , 9.7D0/) 
    REAL(KIND=r8)   , PARAMETER :: alog10 = 2.30258509299405D0
    REAL(KIND=r8)   , PARAMETER :: rt2pin = .398942280401433D0
    REAL(KIND=r8)   , PARAMETER :: rtpi   =1.77245385090552D0
    REAL(KIND=r8)   , PARAMETER :: third  = .333333333333333D0

    REAL(KIND=r8)   , PARAMETER :: d0(13) =(/&
         0.833333333333333D-01,-0.148148148148148D-01,  &
         0.115740740740741D-02, 0.352733686067019D-03,  &
         -0.178755144032922D-03, 0.391926317852244D-04,  &
         -0.218544851067999D-05,-0.185406221071516D-05,  &
         0.829671134095309D-06,-0.176659527368261D-06,  &
         0.670785354340150D-08, 0.102618097842403D-07,  &
         -0.438203601845335D-08/)

    REAL(KIND=r8)   , PARAMETER ::  d10=-0.185185185185185D-02

    REAL(KIND=r8)   , PARAMETER ::  d1(12)=(/&
         -0.347222222222222D-02, 0.264550264550265D-02,-0.990226337448560D-03,  &
         0.205761316872428D-03,-0.401877572016461D-06,-0.180985503344900D-04,  &
         0.764916091608111D-05,-0.161209008945634D-05, 0.464712780280743D-08,  &
         0.137863344691572D-06,-0.575254560351770D-07, 0.119516285997781D-07/)

    REAL(KIND=r8)   , PARAMETER ::  d20 =.413359788359788D-02

    REAL(KIND=r8)   , PARAMETER ::  d2(10)=(/&
         -0.268132716049383D-02, 0.771604938271605D-03, 0.200938786008230D-05, &
         -0.107366532263652D-03, 0.529234488291201D-04,-0.127606351886187D-04, &
         0.342357873409614D-07, 0.137219573090629D-05,-0.629899213838006D-06, &
         0.142806142060642D-06/)

    REAL(KIND=r8)   , PARAMETER ::  d30 =0.649434156378601D-03

    REAL(KIND=r8)   , PARAMETER ::  d3(8)=(/&
         0.229472093621399D-03,-0.469189494395256D-03, 0.267720632062839D-03, &
         -0.756180167188398D-04,-0.239650511386730D-06, 0.110826541153473D-04, &
         -0.567495282699160D-05, 0.142309007324359D-05/)

    REAL(KIND=r8)   , PARAMETER ::  d40 = -0.861888290916712D-03

    REAL(KIND=r8)   , PARAMETER ::  d4(6)=(/&
         0.784039221720067D-03,-0.299072480303190D-03,-0.146384525788434D-05,  &
         0.664149821546512D-04,-0.396836504717943D-04, 0.113757269706784D-04/)

    REAL(KIND=r8)   , PARAMETER ::  d50 =-0.336798553366358D-03

    REAL(KIND=r8)   , PARAMETER :: d5(4) =(/&
         -0.697281375836586D-04,0.277275324495939D-03,-0.199325705161888D-03,  &
         0.679778047793721D-04/)

    REAL(KIND=r8)   , PARAMETER :: d60=0.531307936463992D-03

    REAL(KIND=r8)   , PARAMETER ::  d6(2)  =(/&
         -0.592166437353694D-03,0.270878209671804D-03/)

    REAL(KIND=r8)   , PARAMETER ::  d70=0.344367606892378D-03
    !     ..
    !     .. Executable Statements ..
    !     --------------------
    !     ****** E IS A MACHINE DEPENDENT CONSTANT. E IS THE SMALLEST
    !            FLOATING POINT NUMBER FOR WHICH 1.0 + E .GT. 1.0 .
    !
    e = spmpar(1)
    !
    !     --------------------
    IF (a.LT.0.0_r8 .OR. x.LT.0.0_r8) GO TO 430
    IF (a.EQ.0.0_r8 .AND. x.EQ.0.0_r8) GO TO 430
    IF (a*x.EQ.0.0_r8) GO TO 420
    !
    iop = ind + 1
    IF (iop.NE.1 .AND. iop.NE.2) iop = 3
    acc = dmax1(acc0(iop),e)
    e0 = e00(iop)
    x0 = x00(iop)
    !
    !            SELECT THE APPROPRIATE ALGORITHM
    !
    IF (a.GE.1.0_r8) GO TO 10
    IF (a.EQ.0.5D0) GO TO 390
    IF (x.LT.1.1D0) GO TO 160
    t1 = a*dlog(x) - x
    u = a*EXP(t1)
    IF (u.EQ.0.0_r8) GO TO 380
    r = u* (1.0_r8+gam1(a))
    GO TO 250
    !
10  IF (a.GE.big(iop)) GO TO 30
    IF (a.GT.x .OR. x.GE.x0) GO TO 20
    twoa = a + a
    m = INT(twoa)
    IF (twoa.NE.DBLE(m)) GO TO 20
    i = m/2
    IF (a.EQ.DBLE(i)) GO TO 210
    GO TO 220

20  t1 = a*dlog(x) - x
    r = EXP(t1)/gamma(a)
    GO TO 40
    !
30  l = x/a
    IF (l.EQ.0.0_r8) GO TO 370
    s = 0.5D0 + (0.5D0-l)
    z = rlog(l)
    IF (z.GE.700.0_r8/a) GO TO 410
    y = a*z
    rta = SQRT(a)
    IF (ABS(s).LE.e0/rta) GO TO 330
    IF (ABS(s).LE.0.4D0) GO TO 270
    !
    t = (1.0_r8/a)**2
    t1 = (((0.75D0*t-1.0_r8)*t+3.5D0)*t-105.0_r8)/ (a*1260.0_r8)
    t1 = t1 - y
    r = rt2pin*rta*EXP(t1)
    !
40  IF (r.EQ.0.0_r8) GO TO 420
    IF (x.LE.dmax1(a,alog10)) GO TO 50
    IF (x.LT.x0) GO TO 250
    GO TO 100
    !
    !               TAYLOR SERIES FOR P/R
    !
50  apn = a + 1.0_r8
    t = x/apn
    wk(1) = t
    DO n = 2,20
       apn = apn + 1.0_r8
       t = t* (x/apn)
       IF (t.LE.1.D-3) GO TO 70
       wk(n) = t
    END DO
    n = 20
    !
70  sum = t
    tol = 0.5D0*acc
80  apn = apn + 1.0_r8
    t = t* (x/apn)
    sum = sum + t
    IF (t.GT.tol) GO TO 80
    !
    max = n - 1
    DO m = 1,max
       n = n - 1
       sum = sum + wk(n)
    END DO
    ans = (r/a)* (1.0_r8+sum)
    qans = 0.5D0 + (0.5D0-ans)
    RETURN
    !
    !               ASYMPTOTIC EXPANSION
    !
100 amn = a - 1.0_r8
    t = amn/x
    wk(1) = t
    DO n = 2,20
       amn = amn - 1.0_r8
       t = t* (amn/x)
       IF (ABS(t).LE.1.D-3) GO TO 120
       wk(n) = t
    END DO
    n = 20
    !
120 sum = t
130 IF (ABS(t).LE.acc) GO TO 140
    amn = amn - 1.0_r8
    t = t* (amn/x)
    sum = sum + t
    GO TO 130
    !
140 max = n - 1
    DO  m = 1,max
       n = n - 1
       sum = sum + wk(n)
    END DO
    qans = (r/x)* (1.0_r8+sum)
    ans = 0.5D0 + (0.5D0-qans)
    RETURN
    !
    !             TAYLOR SERIES FOR P(A,X)/X**A
    !
160 an = 3.0_r8
    c = x
    sum = x/ (a+3.0_r8)
    tol = 3.0_r8*acc/ (a+1.0_r8)
170 an = an + 1.0_r8
    c = -c* (x/an)
    t = c/ (a+an)
    sum = sum + t
    IF (ABS(t).GT.tol) GO TO 170
    j = a*x* ((sum/6.0_r8-0.5D0/ (a+2.0_r8))*x+1.0_r8/ (a+1.0_r8))
    !
    z = a*dlog(x)
    h = gam1(a)
    g = 1.0_r8 + h
    IF (x.LT.0.25D0) GO TO 180
    IF (a.LT.x/2.59D0) GO TO 200
    GO TO 190

180 IF (z.GT.-.13394D0) GO TO 200
    !
190 w = EXP(z)
    ans = w*g* (0.5D0+ (0.5D0-j))
    qans = 0.5D0 + (0.5D0-ans)
    RETURN
    !
200 l = rexp(z)
    w = 0.5D0 + (0.5D0+l)
    qans = (w*j-l)*g - h
    IF (qans.LT.0.0_r8) GO TO 380
    ans = 0.5D0 + (0.5D0-qans)
    RETURN
    !
    !             FINITE SUMS FOR Q WHEN A .GE. 1
    !               AND 2*A IS AN INTEGER
    !
210 sum = EXP(-x)
    t = sum
    n = 1
    c = 0.0_r8
    GO TO 230
    !
220 rtx = SQRT(x)
    sum = erfc1(0,rtx)
    t = EXP(-x)/ (rtpi*rtx)
    n = 0
    c = -0.5D0
    !
230 IF (n.EQ.i) GO TO 240
    n = n + 1
    c = c + 1.0_r8
    t = (x*t)/c
    sum = sum + t
    GO TO 230

240 qans = sum
    ans = 0.5D0 + (0.5D0-qans)
    RETURN
    !
    !              CONTINUED FRACTION EXPANSION
    !
250 tol = dmax1(5.0_r8*e,acc)
    a2nm1 = 1.0_r8
    a2n = 1.0_r8
    b2nm1 = x
    b2n = x + (1.0_r8-a)
    c = 1.0_r8
260 a2nm1 = x*a2n + c*a2nm1
    b2nm1 = x*b2n + c*b2nm1
    am0 = a2nm1/b2nm1
    c = c + 1.0_r8
    cma = c - a
    a2n = a2nm1 + cma*a2n
    b2n = b2nm1 + cma*b2n
    an0 = a2n/b2n
    IF (ABS(an0-am0).GE.tol*an0) GO TO 260
    !
    qans = r*an0
    ans = 0.5D0 + (0.5D0-qans)
    RETURN
    !
    !              GENERAL TEMME EXPANSION
    !
270 IF (ABS(s).LE.2.0_r8*e .AND. a*e*e.GT.3.28D-3) GO TO 430
    c = EXP(-y)
    w = 0.5D0*erfc1(1,SQRT(y))
    u = 1.0_r8/a
    z = SQRT(z+z)
    IF (l.LT.1.0_r8) z = -z
    IF (iop-2) 280,290,300
    !
280 IF (ABS(s).LE.1.D-3) GO TO 340
    c0 = ((((((((((((d0(13)*z+d0(12))*z+d0(11))*z+d0(10))*z+d0(9))*z+ &
         d0(8))*z+d0(7))*z+d0(6))*z+d0(5))*z+d0(4))*z+d0(3))*z+d0(2))* &
         z+d0(1))*z - third
    c1 = (((((((((((d1(12)*z+d1(11))*z+d1(10))*z+d1(9))*z+d1(8))*z+ &
         d1(7))*z+d1(6))*z+d1(5))*z+d1(4))*z+d1(3))*z+d1(2))*z+d1(1))* &
         z + d10
    c2 = (((((((((d2(10)*z+d2(9))*z+d2(8))*z+d2(7))*z+d2(6))*z+ &
         d2(5))*z+d2(4))*z+d2(3))*z+d2(2))*z+d2(1))*z + d20
    c3 = (((((((d3(8)*z+d3(7))*z+d3(6))*z+d3(5))*z+d3(4))*z+d3(3))*z+ &
         d3(2))*z+d3(1))*z + d30
    c4 = (((((d4(6)*z+d4(5))*z+d4(4))*z+d4(3))*z+d4(2))*z+d4(1))*z + &
         d40
    c5 = (((d5(4)*z+d5(3))*z+d5(2))*z+d5(1))*z + d50
    c6 = (d6(2)*z+d6(1))*z + d60
    t = ((((((d70*u+c6)*u+c5)*u+c4)*u+c3)*u+c2)*u+c1)*u + c0
    GO TO 310
    !
290 c0 = (((((d0(6)*z+d0(5))*z+d0(4))*z+d0(3))*z+d0(2))*z+d0(1))*z - &
         third
    c1 = (((d1(4)*z+d1(3))*z+d1(2))*z+d1(1))*z + d10
    c2 = d2(1)*z + d20
    t = (c2*u+c1)*u + c0
    GO TO 310
    !
300 t = ((d0(3)*z+d0(2))*z+d0(1))*z - third
    !
310 IF (l.LT.1.0_r8) GO TO 320
    qans = c* (w+rt2pin*t/rta)
    ans = 0.5D0 + (0.5D0-qans)
    RETURN

320 ans = c* (w-rt2pin*t/rta)
    qans = 0.5D0 + (0.5D0-ans)
    RETURN
    !
    !             TEMME EXPANSION FOR L = 1
    !
330 IF (a*e*e.GT.3.28D-3) GO TO 430
    c = 0.5D0 + (0.5D0-y)
    w = (0.5D0-SQRT(y)* (0.5D0+ (0.5D0-y/3.0_r8))/rtpi)/c
    u = 1.0_r8/a
    z = SQRT(z+z)
    IF (l.LT.1.0_r8) z = -z
    IF (iop-2) 340,350,360
    !
340 c0 = ((((((d0(7)*z+d0(6))*z+d0(5))*z+d0(4))*z+d0(3))*z+d0(2))*z+ &
         d0(1))*z - third
    c1 = (((((d1(6)*z+d1(5))*z+d1(4))*z+d1(3))*z+d1(2))*z+d1(1))*z + &
         d10
    c2 = ((((d2(5)*z+d2(4))*z+d2(3))*z+d2(2))*z+d2(1))*z + d20
    c3 = (((d3(4)*z+d3(3))*z+d3(2))*z+d3(1))*z + d30
    c4 = (d4(2)*z+d4(1))*z + d40
    c5 = (d5(2)*z+d5(1))*z + d50
    c6 = d6(1)*z + d60
    t = ((((((d70*u+c6)*u+c5)*u+c4)*u+c3)*u+c2)*u+c1)*u + c0
    GO TO 310
    !
350 c0 = (d0(2)*z+d0(1))*z - third
    c1 = d1(1)*z + d10
    t = (d20*u+c1)*u + c0
    GO TO 310
    !
360 t = d0(1)*z - third
    GO TO 310
    !
    !                   SPECIAL CASES
    !
370 ans = 0.0_r8
    qans = 1.0_r8
    RETURN
    !
380 ans = 1.0_r8
    qans = 0.0_r8
    RETURN
    !
390 IF (x.GE.0.25D0) GO TO 400
    ans = erf(SQRT(x))
    qans = 0.5D0 + (0.5D0-ans)
    RETURN

400 qans = erfc1(0,SQRT(x))
    ans = 0.5D0 + (0.5D0-qans)
    RETURN
    !
410 IF (ABS(s).LE.2.0_r8*e) GO TO 430
420 IF (x.LE.a) GO TO 370
    GO TO 380
    !
    !                   ERROR RETURN
    !
430 ans = 2.0_r8
    RETURN

  END  SUBROUTINE gratio





  INTEGER FUNCTION ipmpar(i)
    !-----------------------------------------------------------------------
    !
    !     IPMPAR PROVIDES THE INTEGER MACHINE CONSTANTS FOR THE COMPUTER
    !     THAT IS USED. IT IS ASSUMED THAT THE ARGUMENT I IS AN INTEGER
    !     HAVING ONE OF THE VALUES 1-10. IPMPAR(I) HAS THE VALUE ...
    !
    !  INTEGERS.
    !
    !     ASSUME INTEGERS ARE REPRESENTED IN THE N-DIGIT, BASE-A FORM
    !
    !             SIGN ( X(N-1)*A**(N-1) + ... + X(1)*A + X(0) )
    !
    !             WHERE 0 .LE. X(I) .LT. A FOR I=0,...,N-1.
    !
    !     IPMPAR(1) = A, THE BASE.
    !
    !     IPMPAR(2) = N, THE NUMBER OF BASE-A DIGITS.
    !
    !     IPMPAR(3) = A**N - 1, THE LARGEST MAGNITUDE.
    !
    !  FLOATING-POINT NUMBERS.
    !
    !     IT IS ASSUMED THAT THE SINGLE AND REAL(KIND=r8)    FLOATING
    !     POINT ARITHMETICS HAVE THE SAME BASE, SAY B, AND THAT THE
    !     NONZERO NUMBERS ARE REPRESENTED IN THE FORM
    !
    !             SIGN (B**E) * (X(1)/B + ... + X(M)/B**M)
    !
    !             WHERE X(I) = 0,1,...,B-1 FOR I=1,...,M,
    !             X(1) .GE. 1, AND EMIN .LE. E .LE. EMAX.
    !
    !     IPMPAR(4) = B, THE BASE.
    !
    !  SINGLE-PRECISION
    !
    !     IPMPAR(5) = M, THE NUMBER OF BASE-B DIGITS.
    !
    !     IPMPAR(6) = EMIN, THE SMALLEST EXPONENT E.
    !
    !     IPMPAR(7) = EMAX, THE LARGEST EXPONENT E.
    !
    !  DOUBLE-PRECISION
    !
    !     IPMPAR(8) = M, THE NUMBER OF BASE-B DIGITS.
    !
    !     IPMPAR(9) = EMIN, THE SMALLEST EXPONENT E.
    !
    !     IPMPAR(10) = EMAX, THE LARGEST EXPONENT E.
    !
    !-----------------------------------------------------------------------
    !
    !     TO DEFINE THIS FUNCTION FOR THE COMPUTER BEING USED, ACTIVATE
    !     THE DATA STATMENTS FOR THE COMPUTER BY REMOVING THE C FROM
    !     COLUMN 1. (ALL THE OTHER DATA STATEMENTS SHOULD HAVE C IN
    !     COLUMN 1.)
    !
    !-----------------------------------------------------------------------
    !
    !     IPMPAR IS AN ADAPTATION OF THE FUNCTION I1MACH, WRITTEN BY
    !     P.A. FOX, A.D. HALL, AND N.L. SCHRYER (BELL LABORATORIES).
    !     IPMPAR WAS FORMED BY A.H. MORRIS (NSWC). THE CONSTANTS ARE
    !     FROM BELL LABORATORIES, NSWC, AND OTHER SOURCES.
    !
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    INTEGER, INTENT(IN   ) ::  i
    !     ..
    !     .. Data statements ..
    !
    !     MACHINE CONSTANTS FOR AMDAHL MACHINES.
    !
    !     DATA IMACH( 1) /   2 /
    !     DATA IMACH( 2) /  31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /  16 /
    !     DATA IMACH( 5) /   6 /
    !     DATA IMACH( 6) / -64 /
    !     DATA IMACH( 7) /  63 /
    !     DATA IMACH( 8) /  14 /
    !     DATA IMACH( 9) / -64 /
    !     DATA IMACH(10) /  63 /
    !
    !     MACHINE CONSTANTS FOR THE AT&T 3B SERIES, AT&T
    !     PC 7300, AND AT&T 6300.
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    24 /
    !     DATA IMACH( 6) /  -125 /
    !     DATA IMACH( 7) /   128 /
    !     DATA IMACH( 8) /    53 /
    !     DATA IMACH( 9) / -1021 /
    !     DATA IMACH(10) /  1024 /
    !
    !     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   33 /
    !     DATA IMACH( 3) / 8589934591 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   24 /
    !     DATA IMACH( 6) / -256 /
    !     DATA IMACH( 7) /  255 /
    !     DATA IMACH( 8) /   60 /
    !     DATA IMACH( 9) / -256 /
    !     DATA IMACH(10) /  255 /
    !
    !     MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   39 /
    !     DATA IMACH( 3) / 549755813887 /
    !     DATA IMACH( 4) /    8 /
    !     DATA IMACH( 5) /   13 /
    !     DATA IMACH( 6) /  -50 /
    !     DATA IMACH( 7) /   76 /
    !     DATA IMACH( 8) /   26 /
    !     DATA IMACH( 9) /  -50 /
    !     DATA IMACH(10) /   76 /
    !
    !     MACHINE CONSTANTS FOR THE BURROUGHS 6700/7700 SYSTEMS.
    !
    !     DATA IMACH( 1) /      2 /
    !     DATA IMACH( 2) /     39 /
    !     DATA IMACH( 3) / 549755813887 /
    !     DATA IMACH( 4) /      8 /
    !     DATA IMACH( 5) /     13 /
    !     DATA IMACH( 6) /    -50 /
    !     DATA IMACH( 7) /     76 /
    !     DATA IMACH( 8) /     26 /
    !     DATA IMACH( 9) / -32754 /
    !     DATA IMACH(10) /  32780 /
    !
    !     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES
    !     60 BIT ARITHMETIC, AND THE CDC CYBER 995 64 BIT
    !     ARITHMETIC (NOS OPERATING SYSTEM).
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   48 /
    !     DATA IMACH( 3) / 281474976710655 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   48 /
    !     DATA IMACH( 6) / -974 /
    !     DATA IMACH( 7) / 1070 /
    !     DATA IMACH( 8) /   95 /
    !     DATA IMACH( 9) / -926 /
    !     DATA IMACH(10) / 1070 /
    !
    !     MACHINE CONSTANTS FOR THE CDC CYBER 995 64 BIT
    !     ARITHMETIC (NOS/VE OPERATING SYSTEM).
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    63 /
    !     DATA IMACH( 3) / 9223372036854775807 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    48 /
    !     DATA IMACH( 6) / -4096 /
    !     DATA IMACH( 7) /  4095 /
    !     DATA IMACH( 8) /    96 /
    !     DATA IMACH( 9) / -4096 /
    !     DATA IMACH(10) /  4095 /
    !
    !     MACHINE CONSTANTS FOR THE CRAY 1, XMP, 2, AND 3.
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    63 /
    !     DATA IMACH( 3) / 9223372036854775807 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    47 /
    !     DATA IMACH( 6) / -8189 /
    !     DATA IMACH( 7) /  8190 /
    !     DATA IMACH( 8) /    94 /
    !     DATA IMACH( 9) / -8099 /
    !     DATA IMACH(10) /  8190 /
    !
    !     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   15 /
    !     DATA IMACH( 3) / 32767 /
    !     DATA IMACH( 4) /   16 /
    !     DATA IMACH( 5) /    6 /
    !     DATA IMACH( 6) /  -64 /
    !     DATA IMACH( 7) /   63 /
    !     DATA IMACH( 8) /   14 /
    !     DATA IMACH( 9) /  -64 /
    !     DATA IMACH(10) /   63 /
    !
    !     MACHINE CONSTANTS FOR THE HARRIS 220.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   23 /
    !     DATA IMACH( 3) / 8388607 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   23 /
    !     DATA IMACH( 6) / -127 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   38 /
    !     DATA IMACH( 9) / -127 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000
    !     AND DPS 8/70 SERIES.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   35 /
    !     DATA IMACH( 3) / 34359738367 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   27 /
    !     DATA IMACH( 6) / -127 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   63 /
    !     DATA IMACH( 9) / -127 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE HP 2100
    !     3 WORD REAL(KIND=r8)    OPTION WITH FTN4
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   15 /
    !     DATA IMACH( 3) / 32767 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   23 /
    !     DATA IMACH( 6) / -128 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   39 /
    !     DATA IMACH( 9) / -128 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE HP 2100
    !     4 WORD REAL(KIND=r8)    OPTION WITH FTN4
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   15 /
    !     DATA IMACH( 3) / 32767 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   23 /
    !     DATA IMACH( 6) / -128 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   55 /
    !     DATA IMACH( 9) / -128 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE HP 9000.
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    24 /
    !     DATA IMACH( 6) /  -126 /
    !     DATA IMACH( 7) /   128 /
    !     DATA IMACH( 8) /    53 /
    !     DATA IMACH( 9) / -1021 /
    !     DATA IMACH(10) /  1024 /
    !
    !     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
    !     THE ICL 2900, THE ITEL AS/6, THE XEROX SIGMA
    !     5/7/9 AND THE SEL SYSTEMS 85/86.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /   16 /
    !     DATA IMACH( 5) /    6 /
    !     DATA IMACH( 6) /  -64 /
    !     DATA IMACH( 7) /   63 /
    !     DATA IMACH( 8) /   14 /
    !     DATA IMACH( 9) /  -64 /
    !     DATA IMACH(10) /   63 /
    !
    !     MACHINE CONSTANTS FOR THE IBM PC.
    !
    !      DATA imach(1)/2/
    !      DATA imach(2)/31/
    !      DATA imach(3)/2147483647/
    !      DATA imach(4)/2/
    !      DATA imach(5)/24/
    !      DATA imach(6)/-125/
    !      DATA imach(7)/128/
    !      DATA imach(8)/53/
    !      DATA imach(9)/-1021/
    !      DATA imach(10)/1024/
    !
    !     MACHINE CONSTANTS FOR THE MACINTOSH II - ABSOFT
    !     MACFORTRAN II.
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    24 /
    !     DATA IMACH( 6) /  -125 /
    !     DATA IMACH( 7) /   128 /
    !     DATA IMACH( 8) /    53 /
    !     DATA IMACH( 9) / -1021 /
    !     DATA IMACH(10) /  1024 /
    !
    !     MACHINE CONSTANTS FOR THE MICROVAX - VMS FORTRAN.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   24 /
    !     DATA IMACH( 6) / -127 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   56 /
    !     DATA IMACH( 9) / -127 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR).
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   35 /
    !     DATA IMACH( 3) / 34359738367 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   27 /
    !     DATA IMACH( 6) / -128 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   54 /
    !     DATA IMACH( 9) / -101 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR).
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   35 /
    !     DATA IMACH( 3) / 34359738367 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   27 /
    !     DATA IMACH( 6) / -128 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   62 /
    !     DATA IMACH( 9) / -128 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE PDP-11 FORTRAN SUPPORTING
    !     32-BIT INTEGER ARITHMETIC.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   24 /
    !     DATA IMACH( 6) / -127 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   56 /
    !     DATA IMACH( 9) / -127 /
    !     DATA IMACH(10) /  127 /
    !
    !     MACHINE CONSTANTS FOR THE SEQUENT BALANCE 8000.
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    24 /
    !     DATA IMACH( 6) /  -125 /
    !     DATA IMACH( 7) /   128 /
    !     DATA IMACH( 8) /    53 /
    !     DATA IMACH( 9) / -1021 /
    !     DATA IMACH(10) /  1024 /
    !
    !     MACHINE CONSTANTS FOR THE SILICON GRAPHICS IRIS-4D
    !     SERIES (MIPS R3000 PROCESSOR).
    !
    !     DATA IMACH( 1) /     2 /
    !     DATA IMACH( 2) /    31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /     2 /
    !     DATA IMACH( 5) /    24 /
    !     DATA IMACH( 6) /  -125 /
    !     DATA IMACH( 7) /   128 /
    !     DATA IMACH( 8) /    53 /
    !     DATA IMACH( 9) / -1021 /
    !     DATA IMACH(10) /  1024 /
    !
    !     MACHINE CONSTANTS FOR IEEE ARITHMETIC MACHINES, SUCH AS THE AT&T
    !     3B SERIES, MOTOROLA 68000 BASED MACHINES (E.G. SUN 3 AND AT&T
    !     PC 7300), AND 8087 BASED MICROS (E.G. IBM PC AND AT&T 6300).
    !
    INTEGER, PARAMETER :: IMACH(10) =(/     2 ,    31 , 2147483647,     2,    24 ,&
         -125 ,   128 ,    53 , -1021 ,  1024 /)
    !
    !     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   35 /
    !     DATA IMACH( 3) / 34359738367 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   27 /
    !     DATA IMACH( 6) / -128 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   60 /
    !     DATA IMACH( 9) /-1024 /
    !     DATA IMACH(10) / 1023 /
    !
    !     MACHINE CONSTANTS FOR THE VAX 11/780.
    !
    !     DATA IMACH( 1) /    2 /
    !     DATA IMACH( 2) /   31 /
    !     DATA IMACH( 3) / 2147483647 /
    !     DATA IMACH( 4) /    2 /
    !     DATA IMACH( 5) /   24 /
    !     DATA IMACH( 6) / -127 /
    !     DATA IMACH( 7) /  127 /
    !     DATA IMACH( 8) /   56 /
    !     DATA IMACH( 9) / -127 /
    !     DATA IMACH(10) /  127 /
    !
    ipmpar = imach(i)
    RETURN

  END  FUNCTION ipmpar






  REAL(KIND=r8)    FUNCTION rexp(x)
    !-----------------------------------------------------------------------
    !            EVALUATION OF THE FUNCTION EXP(X) - 1
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)    x
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    w
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC abs,exp
    !     ..
    !     .. Data statements ..
    REAL(KIND=r8)   , PARAMETER :: p1= 0.914041914819518D-09
    REAL(KIND=r8)   , PARAMETER :: p2= 0.238082361044469D-01
    REAL(KIND=r8)   , PARAMETER :: q1=-0.499999999085958D+00
    REAL(KIND=r8)   , PARAMETER :: q2= 0.107141568980644D+00
    REAL(KIND=r8)   , PARAMETER :: q3=-0.119041179760821D-01
    REAL(KIND=r8)   , PARAMETER :: q4= 0.595130811860248D-03
    !     ..
    !     .. Executable Statements ..
    !-----------------------
    IF (ABS(x).GT.0.15D0) GO TO 10
    rexp = x* (((p2*x+p1)*x+1.0_r8)/ ((((q4*x+q3)*x+q2)*x+q1)*x+1.0_r8))
    RETURN
    !
10  w = EXP(x)
    IF (x.GT.0.0_r8) GO TO 20
    rexp = (w-0.5D0) - 0.5D0
    RETURN

20  rexp = w* (0.5D0+ (0.5D0-1.0_r8/w))
    RETURN

  END FUNCTION rexp





  REAL(KIND=r8)    FUNCTION rlog(x)
    !     -------------------
    !     COMPUTATION OF  X - 1 - LN(X)
    !     -------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    REAL(KIND=r8)   , INTENT(IN   ) :: x
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    :: r,t,u,w,w1
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC dble,dlog
    !     ..
    !     .. Data statements ..
    !     -------------------
    REAL(KIND=r8)   , PARAMETER ::  a= 0.566749439387324D-01
    REAL(KIND=r8)   , PARAMETER ::  b= 0.456512608815524D-01
    REAL(KIND=r8)   , PARAMETER :: p0= 0.333333333333333D+00
    REAL(KIND=r8)   , PARAMETER :: p1=-0.224696413112536D+00
    REAL(KIND=r8)   , PARAMETER :: p2= 0.620886815375787D-02
    REAL(KIND=r8)   , PARAMETER :: q1=-0.127408923933623D+01
    REAL(KIND=r8)   , PARAMETER :: q2= 0.354508718369557D+00
    !     ..
    !     .. Executable Statements ..
    !     -------------------
    IF (x.LT.0.61D0 .OR. x.GT.1.57D0) GO TO 40
    IF (x.LT.0.82D0) GO TO 10
    IF (x.GT.1.18D0) GO TO 20
    !
    !              ARGUMENT REDUCTION
    !
    u = (x-0.5D0) - 0.5D0
    w1 = 0.0_r8
    GO TO 30
    !
10  u = DBLE(x) - 0.7D0
    u = u/0.7D0
    w1 = a - u*0.3D0
    GO TO 30
    !
20  u = 0.75D0*DBLE(x) - 1.D0
    w1 = b + u/3.0_r8
    !
    !             SERIES EXPANSION
    !
30  r = u/ (u+2.0_r8)
    t = r*r
    w = ((p2*t+p1)*t+p0)/ ((q2*t+q1)*t+1.0_r8)
    rlog = 2.0_r8*t* (1.0_r8/ (1.0_r8-r)-r*w) + w1
    RETURN
    !
    !
40  r = (x-0.5D0) - 0.5D0
    rlog = r - dlog(x)
    RETURN

  END  FUNCTION rlog




  REAL(KIND=r8)    FUNCTION spmpar(i)
    !-----------------------------------------------------------------------
    !
    !     SPMPAR PROVIDES THE SINGLE PRECISION MACHINE CONSTANTS FOR
    !     THE COMPUTER BEING USED. IT IS ASSUMED THAT THE ARGUMENT
    !     I IS AN INTEGER HAVING ONE OF THE VALUES 1, 2, OR 3. IF THE
    !     SINGLE PRECISION ARITHMETIC BEING USED HAS M BASE B DIGITS AND
    !     ITS SMALLEST AND LARGEST EXPONENTS ARE EMIN AND EMAX, THEN
    !
    !        SPMPAR(1) = B**(1 - M), THE MACHINE PRECISION,
    !
    !        SPMPAR(2) = B**(EMIN - 1), THE SMALLEST MAGNITUDE,
    !
    !        SPMPAR(3) = B**EMAX*(1 - B**(-M)), THE LARGEST MAGNITUDE.
    !
    !-----------------------------------------------------------------------
    !     WRITTEN BY
    !        ALFRED H. MORRIS, JR.
    !        NAVAL SURFACE WARFARE CENTER
    !        DAHLGREN VIRGINIA
    !-----------------------------------------------------------------------
    !-----------------------------------------------------------------------
    !     MODIFIED BY BARRY W. BROWN TO RETURN REAL(KIND=r8)    MACHINE
    !     CONSTANTS FOR THE COMPUTER BEING USED.  THIS MODIFICATION WAS
    !     MADE AS PART OF CONVERTING BRATIO TO REAL(KIND=r8)   
    !-----------------------------------------------------------------------
    IMPLICIT NONE
    !     .. Scalar Arguments ..
    INTEGER, INTENT(IN   ) :: i
    !     ..
    !     .. Local Scalars ..
    REAL(KIND=r8)    b,binv,bm1,one,w,z
    INTEGER emax,emin,ibeta,m
    !     ..
    !     .. External Functions ..
    !INTEGER ipmpar
    !EXTERNAL ipmpar
    !     ..
    !     .. Intrinsic Functions ..
    !INTRINSIC dble
    !     ..
    !     .. Executable Statements ..
    !
    IF (i.GT.1) GO TO 10
    b = ipmpar(4)
    m = ipmpar(8)
    spmpar = b** (1-m)
    RETURN
    !
10  IF (i.GT.2) GO TO 20
    b = ipmpar(4)
    emin = ipmpar(9)
    one = DBLE(1)
    binv = one/b
    w = b** (emin+2)
    spmpar = ((w*binv)*binv)*binv
    RETURN
    !
20  ibeta = ipmpar(4)
    m = ipmpar(8)
    emax = ipmpar(10)
    !
    b = ibeta
    bm1 = ibeta - 1
    one = DBLE(1)
    z = b** (m-1)
    w = ((z-one)*b+bm1)/ (b*z)
    !
    z = b** (emax-2)
    spmpar = ((w*z)*b)*b
    RETURN

  END FUNCTION spmpar


  !**************************************************************************
  !      FFTPACK: Available at http://www.scd.ucar.edu/softlib/mathlib.html
  !
  !   ***NOTE (C.T.) *** this is a subset of FFTPACK, which only includes
  !            the routines for the complex FFT (forward & inverse).
  !
  ! Modified: November 1999 by Arjan van Dijk to include IMPLICIT NONE and
  !           to convert all routines to REAL(KIND=r8)   .
  !**************************************************************************




  !      * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  !      *                                          *
  !      *                        FFTPACK             *
  !      *                                          *
  !      *     A package of Fortran subprograms for calculating          *
  !      *     fast Fourier transforms for both complex and real         *
  !      *      periodic sequences and certain other symmetric           *
  !      *             sequences that are listed below            *
  !      *               (Version 4.1 November 1988)                *
  !      *                          by                    *
  !      *                     Paul Swarztrauber                   *
  !      *                          of                    *
  !      *         The National Center for Atmospheric Research         *
  !      *               Boulder, Colorado  (80307)  U.S.A.           *
  !      *                  which is sponsored by                  *
  !      *              the National Science Foundation          *
  !      *                                          *
  !      * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  !
  !  Any source code available through this distribution interface at NCAR
  !  is free of charge, but there is no guarantee.
  !
  ! FFTPACK breaks the FORTRAN 77 ANSI Standard
  ! by passing REAL arrays to subroutines and using the arrays within
  ! the subroutines as REAL(KIND=r8)    or other types.  This infraction
  ! may cause data alignment problems when the source code is compiled
  ! and loaded in an executable.




  !****************************************************************************
  !     SUBROUTINE CFFTI(N,WSAVE)
  !
  !     SUBROUTINE CFFTI INITIALIZES THE ARRAY WSAVE WHICH IS USED IN
  !     BOTH CFFTF AND CFFTB. THE PRIME FACTORIZATION OF N TOGETHER WITH
  !     A TABULATION OF THE TRIGONOMETRIC FUNCTIONS ARE COMPUTED AND
  !     STORED IN WSAVE.
  !
  !     INPUT PARAMETER
  !
  !     N       THE LENGTH OF THE SEQUENCE TO BE TRANSFORMED
  !
  !     OUTPUT PARAMETER
  !
  !     WSAVE   A WORK ARRAY WHICH MUST BE DIMENSIONED AT LEAST 4*N+15
  !             THE SAME WORK ARRAY CAN BE USED FOR BOTH CFFTF AND CFFTB
  !             AS LONG AS N REMAINS UNCHANGED. DIFFERENT WSAVE ARRAYS
  !             ARE REQUIRED FOR DIFFERENT VALUES OF N. THE CONTENTS OF
  !             WSAVE MUST NOT BE CHANGED BETWEEN CALLS OF CFFTF OR CFFTB.
  !
  SUBROUTINE CFFTI (N     , &! INTEGER, INTENT(IN        ) :: N
             WSAVE   )! REAL(KIND=r8)   , INTENT(IN   ) ::  WSAVE(*)
    IMPLICIT NONE
    INTEGER, INTENT(IN   ) :: N
    REAL(KIND=r8)   , INTENT(OUT ) ::  WSAVE(*)
    INTEGER      :: IW1
    INTEGER      :: IW2
    !
    IF (N .EQ. 1) RETURN
    IW1 = N+N+1
    IW2 = IW1+N+N
    CALL CFFTI1 (N         , &! INTEGER         , INTENT(IN   ) :: N
                 WSAVE(IW1), &! REAL(KIND=r8)   , INTENT(OUT        )  :: WA   (*)
                 WSAVE(IW2)  )! REAL(KIND=r8)   , INTENT(OUT        )  :: WIFAC(*)
    RETURN
  END SUBROUTINE CFFTI

  !****************************************************************************
  !     SUBROUTINE CFFTF(N,C,WSAVE)
  !
  !     SUBROUTINE CFFTF COMPUTES THE FORWARD COMPLEX DISCRETE FOURIER
  !     TRANSFORM (THE FOURIER ANALYSIS). EQUIVALENTLY , CFFTF COMPUTES
  !     THE FOURIER COEFFICIENTS OF A COMPLEX PERIODIC SEQUENCE.
  !     THE TRANSFORM IS DEFINED BELOW AT OUTPUT PARAMETER C.
  !
  !     THE TRANSFORM IS NOT NORMALIZED. TO OBTAIN A NORMALIZED TRANSFORM
  !     THE OUTPUT MUST BE DIVIDED BY N. OTHERWISE A CALL OF CFFTF
  !     FOLLOWED BY A CALL OF CFFTB WILL MULTIPLY THE SEQUENCE BY N.
  !
  !     THE ARRAY WSAVE WHICH IS USED BY SUBROUTINE CFFTF MUST BE
  !     INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE).
  !
  !     INPUT PARAMETERS
  !
  !
  !     N      THE LENGTH OF THE COMPLEX SEQUENCE C. THE METHOD IS
  !            MORE EFFICIENT WHEN N IS THE PRODUCT OF SMALL PRIMES. N
  !
  !     C      A COMPLEX ARRAY OF LENGTH N WHICH CONTAINS THE SEQUENCE
  !
  !     WSAVE   A REAL WORK ARRAY WHICH MUST BE DIMENSIONED AT LEAST 4N+15
  !             IN THE PROGRAM THAT CALLS CFFTF. THE WSAVE ARRAY MUST BE
  !             INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE) AND A
  !             DIFFERENT WSAVE ARRAY MUST BE USED FOR EACH DIFFERENT
  !             VALUE OF N. THIS INITIALIZATION DOES NOT HAVE TO BE
  !             REPEATED SO LONG AS N REMAINS UNCHANGED THUS SUBSEQUENT
  !             TRANSFORMS CAN BE OBTAINED FASTER THAN THE FIRST.
  !             THE SAME WSAVE ARRAY CAN BE USED BY CFFTF AND CFFTB.
  !
  !     OUTPUT PARAMETERS
  !
  !     C      FOR J=1,...,N
  !
  !              C(J)=THE SUM FROM K=1,...,N OF
  !
  !                    C(K)*EXP(-I*(J-1)*(K-1)*2*PI/N)
  !
  !                       WHERE I=SQRT(-1)
  !
  !     WSAVE   CONTAINS INITIALIZATION CALCULATIONS WHICH MUST NOT BE
  !             DESTROYED BETWEEN CALLS OF SUBROUTINE CFFTF OR CFFTB
  !
  SUBROUTINE CFFTF (NK    , & !INTEGER          , INTENT(IN        ) :: nk
       N     , & !INTEGER          , INTENT(IN        ) :: N
       C     , & !COMPLEX(KIND=r8)  , INTENT(INOUT) :: C    (nk)
       WSAVE   ) !REAL(KIND=r8)   , INTENT(INOUT)  :: WSAVE(nk)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: nk
    INTEGER         , INTENT(IN   ) :: N
    REAL(KIND=r8)    , INTENT(INOUT) :: C    (nk)
    REAL(KIND=r8)   , INTENT(INOUT)  :: WSAVE(nk)
    INTEGER, PARAMETER :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers

    REAL(KIND=r8)  :: CREAL   (nk)
    REAL(KIND=r8)  :: CIMAG   (nk)
    INTEGER           :: IW1,IW2,i
    !
    IF (N .EQ. 1) RETURN
    IW1 = N+N+1
    IW2 = IW1+N+N
    !CREAL=REAL (C,kind=r8)
    !CIMAG=AIMAG(C)
    DO i=1,nk
       !        CALL CPX(c(i))
       !        PRINT*,'pk1',C(i)
       !         pause
    END DO

    CALL CFFTF1 (NK,N         , & !INTEGER          , INTENT(IN        ) :: N 
         C         , & !REAL(KIND=r8)   , INTENT(INOUT) :: C    (*)
         WSAVE     , & !REAL(KIND=r8)   , INTENT(INOUT) :: CH   (*)
         WSAVE(IW1), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA   (*)
         WSAVE(IW2)  ) !REAL(KIND=r8)   , INTENT(IN        ) :: WIFAC(*)

    !        PRINT*,'pk2',CREAL(1),(CIMAG(1))

    DO i=1,N+N
       !CREAL=REAL (C(i),kind=r8)
       !CIMAG=AIMAG(C(i))

       !          PRINT*,'pk2',C(i)!,(CIMAG(i))

       !         C(i)=CMPLX(CREAL(i),CIMAG(i),kind=r8)
    END DO
    !      PRINT*,'pk3',C(1),CONJG(C(1))

    RETURN

  END SUBROUTINE CFFTF

  SUBROUTINE CPX(c)
    IMPLICIT NONE
    REAL(KIND=r8)   , INTENT(IN) :: C
    INTEGER :: i
    PRINT*,C
  END SUBROUTINE CPX
  !****************************************************************************
  !     SUBROUTINE CFFTB(N,C,WSAVE)
  !
  !     SUBROUTINE CFFTB COMPUTES THE BACKWARD COMPLEX DISCRETE FOURIER
  !     TRANSFORM (THE FOURIER SYNTHESIS). EQUIVALENTLY , CFFTB COMPUTES
  !     A COMPLEX PERIODIC SEQUENCE FROM ITS FOURIER COEFFICIENTS.
  !     THE TRANSFORM IS DEFINED BELOW AT OUTPUT PARAMETER C.
  !
  !     A CALL OF CFFTF FOLLOWED BY A CALL OF CFFTB WILL MULTIPLY THE
  !     SEQUENCE BY N.
  !
  !     THE ARRAY WSAVE WHICH IS USED BY SUBROUTINE CFFTB MUST BE
  !     INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE).
  !
  !     INPUT PARAMETERS
  !
  !
  !     N      THE LENGTH OF THE COMPLEX SEQUENCE C. THE METHOD IS
  !            MORE EFFICIENT WHEN N IS THE PRODUCT OF SMALL PRIMES.
  !
  !     C      A COMPLEX ARRAY OF LENGTH N WHICH CONTAINS THE SEQUENCE
  !
  !     WSAVE   A REAL WORK ARRAY WHICH MUST BE DIMENSIONED AT LEAST 4N+15
  !             IN THE PROGRAM THAT CALLS CFFTB. THE WSAVE ARRAY MUST BE
  !             INITIALIZED BY CALLING SUBROUTINE CFFTI(N,WSAVE) AND A
  !             DIFFERENT WSAVE ARRAY MUST BE USED FOR EACH DIFFERENT
  !             VALUE OF N. THIS INITIALIZATION DOES NOT HAVE TO BE
  !             REPEATED SO LONG AS N REMAINS UNCHANGED THUS SUBSEQUENT
  !             TRANSFORMS CAN BE OBTAINED FASTER THAN THE FIRST.
  !             THE SAME WSAVE ARRAY CAN BE USED BY CFFTF AND CFFTB.
  !
  !     OUTPUT PARAMETERS
  !
  !     C      FOR J=1,...,N
  !
  !              C(J)=THE SUM FROM K=1,...,N OF
  !
  !                    C(K)*EXP(I*(J-1)*(K-1)*2*PI/N)
  !
  !                       WHERE I=SQRT(-1)
  !
  !     WSAVE   CONTAINS INITIALIZATION CALCULATIONS WHICH MUST NOT BE
  !             DESTROYED BETWEEN CALLS OF SUBROUTINE CFFTF OR CFFTB
  !
  SUBROUTINE CFFTB (N    , &!INTEGER         , INTENT(IN   ) :: N
       C    , &!REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
       WSAVE  )!REAL(KIND=r8)   , INTENT(INOUT) :: WSAVE(*)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: N
    REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
    REAL(KIND=r8)   , INTENT(INOUT) :: WSAVE(*)
    INTEGER IW1,IW2
    !
    IF (N .EQ. 1) RETURN
    IW1 = N+N+1
    IW2 = IW1+N+N
    CALL CFFTB1 (N         , &!INTEGER        , INTENT(IN   ) :: N
         C         , &!REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
         WSAVE     , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH(*)
         WSAVE(IW1), &!REAL(KIND=r8)   , INTENT(IN  ) :: WA(*)
         WSAVE(IW2)  )!REAL(KIND=r8)   , INTENT(IN  ) :: WIFAC(*)
    RETURN
  END SUBROUTINE CFFTB

  !****************************************************************************
  SUBROUTINE CFFTI1 (N    , &! INTEGER, INTENT(IN        ) :: N
       WA   , &! REAL(KIND=r8)   , INTENT(OUT   )  :: WA   (*)
       WIFAC  )! REAL(KIND=r8)   , INTENT(OUT   )  :: WIFAC(*)
    IMPLICIT NONE
    INTEGER, INTENT(IN   ) :: N
    REAL(KIND=r8)   , INTENT(OUT   )  :: WA   (*)
    REAL(KIND=r8)   , INTENT(OUT   )  :: WIFAC(*)
    INTEGER           :: NL
    INTEGER           :: NF
    INTEGER           :: J
    INTEGER           :: NTRY
    INTEGER           :: NQ
    INTEGER           :: NR
    INTEGER           :: IB
    INTEGER           :: I
    INTEGER           :: L1
    INTEGER           :: K1
    INTEGER           :: IP
    INTEGER           :: LD
    INTEGER           :: L2
    INTEGER           :: IDO
    INTEGER           :: IDOT
    INTEGER           :: IPM
    INTEGER           :: I1
    INTEGER           :: II
    REAL(KIND=r8)     :: TPI
    REAL(KIND=r8)     :: ARGH
    REAL(KIND=r8)     :: FI
    REAL(KIND=r8)     :: ARGLD
    REAL(KIND=r8)     :: ARG
    INTEGER, PARAMETER :: NTRYH(1:4)=(/3,4,2,5/)
    NL = N
    NF = 0
    J = 0
101 J = J+1
    IF (J-4) 102,102,103
102 NTRY = NTRYH(J)
    GO TO 104
103 NTRY = NTRY+2
104 NQ = NL/NTRY
    NR = NL-NTRY*NQ
    IF (NR) 101,105,101
105 NF = NF+1
    WIFAC(NF+2) = DBLE(NTRY)
    NL = NQ
    IF (NTRY .NE. 2) GO TO 107
    IF (NF .EQ. 1) GO TO 107
    DO  I=2,NF
       IB = NF-I+2
       WIFAC(IB+2) = WIFAC(IB+1)
    END DO
    WIFAC(3) = 2.D0
107 IF (NL .NE. 1) GO TO 104
    WIFAC(1) = DBLE(N)
    WIFAC(2) = DBLE(NF)
    TPI = 2.D0*(4.D0*ATAN(1.D0))
    ARGH = TPI/DBLE(N)
    I = 2
    L1 = 1
    DO  K1=1,NF
       IP = NINT(WIFAC(K1+2))
       LD = 0
       L2 = L1*IP
       IDO = N/L2
       IDOT = IDO+IDO+2
       IPM = IP-1
       DO  J=1,IPM
          I1 = I
          WA(I-1) = 1.D0
          WA(I) = 0.D0
          LD = LD+L1
          FI = 0.D0
          ARGLD = DBLE(LD)*ARGH
          DO  II=4,IDOT,2
             I = I+2
             FI = FI+1.D0
             ARG = FI*ARGLD
             WA(I-1) = COS(ARG)
             WA(I) = SIN(ARG)
          END DO
          IF (IP .GT. 5) THEN
             WA(I1-1) = WA(I-1)
             WA(I1) = WA(I)
          END IF
       END DO
       L1 = L2
    END DO
    RETURN
  END SUBROUTINE CFFTI1

  !****************************************************************************
  SUBROUTINE CFFTF1 (NK,N        , &!INTEGER              , INTENT(IN   ) :: N 
       C        , &!REAL(KIND=r8)   , INTENT(INOUT) :: C    (*)
       CH       , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH   (*)
       WA       , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA   (*)
       WIFAC      )!REAL(KIND=r8)   , INTENT(IN   ) :: WIFAC(*)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: NK
    INTEGER         , INTENT(IN   ) :: N 
    REAL(KIND=r8)   , INTENT(INOUT) :: C    (NK) 
    REAL(KIND=r8)   , INTENT(INOUT) :: CH   (NK) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA   (NK) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WIFAC(NK)
    REAL(KIND=r8)    :: CC    (NK)

    INTEGER :: NF,NA,L1,IW,K1,IP,L2,IDO,IDOT,IDL1,IX2,IX3,IX4,N2, &
         I,NAC
    !C=REAL(CC)
    NF = NINT(WIFAC(2))
    NA = 0
    L1 = 1
    IW = 1
    DO  K1=1,NF
       IP = NINT(WIFAC(K1+2))
       L2 = IP*L1
       IDO = N/L2
       IDOT = IDO+IDO
       IDL1 = IDOT*L1
       IF (IP .NE. 4) GO TO 103
       IX2 = IW+IDOT
       IX3 = IX2+IDOT
       IF (NA .NE. 0) GO TO 101
       CALL PASSF4 (IDOT   , & !INTEGER, INTENT(IN   )          :: IDO
            L1     , & !INTEGER, INTENT(IN   )          :: L1
            C      , & !REAL(KIND=r8)   , INTENT(IN        ) :: CC    (IDO,4,L1)
            CH     , & !REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,4)  
            WA(IW) , & !REAL(KIND=r8)   , INTENT(IN        ) :: WA1   (*) 
            WA(IX2), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA2   (*) 
            WA(IX3)  ) !REAL(KIND=r8)   , INTENT(IN        ) :: WA3   (*) 
       GO TO 102
101    CALL PASSF4 (IDOT   , &!INTEGER, INTENT(IN   )          :: IDO
            L1     , &!INTEGER, INTENT(IN   )          :: L1
            CH     , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC    (IDO,4,L1)
            C      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,4)  
            WA (IW), &!REAL(KIND=r8)   , INTENT(IN   ) :: WA1   (*) 
            WA(IX2), &!REAL(KIND=r8)   , INTENT(IN   ) :: WA2   (*) 
            WA(IX3)  )!REAL(KIND=r8)   , INTENT(IN   ) :: WA3   (*) 
102    NA = 1-NA
       GO TO 115
103    IF (IP .NE. 2) GO TO 106
       IF (NA .NE. 0) GO TO 104
       CALL PASSF2 (IDOT   , &!INTEGER         , INTENT(IN   ) :: IDO   
            L1     , &!INTEGER         , INTENT(IN   ) :: L1    
            C      , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC (IDO,2,L1)   
            CH     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,2)
            WA(IW)   )!REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)  
       GO TO 105
104    CALL PASSF2 (IDOT   , &!INTEGER         , INTENT(IN   ) :: IDO   
            L1     , &!INTEGER         , INTENT(IN   ) :: L1    
            CH     , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC (IDO,2,L1)   
            C      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,2)
            WA(IW)   )!REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)  
105    NA = 1-NA
       GO TO 115
106    IF (IP .NE. 3) GO TO 109
       IX2 = IW+IDOT
       IF (NA .NE. 0) GO TO 107
       CALL PASSF3 (IDOT   , & ! INTEGER           ,INTENT(IN  ) :: IDO
            L1     , & ! INTEGER           ,INTENT(IN  ) :: L1
            C      , & ! REAL(KIND=r8)   ,INTENT(IN  ) :: CC (IDO,3,L1)
            CH     , & ! REAL(KIND=r8)   ,INTENT(OUT ) :: CH (IDO,L1,3)
            WA(IW) , & ! REAL(KIND=r8)   ,INTENT(IN  ) :: WA1(*)
            WA(IX2)  ) ! REAL(KIND=r8)   ,INTENT(IN  ) :: WA2(*)
       GO TO 108
107    CALL PASSF3 (IDOT   , &! INTEGER          ,INTENT(IN  ) :: IDO
            L1     , &! INTEGER          ,INTENT(IN  ) :: L1
            CH     , &! REAL(KIND=r8)   ,INTENT(IN  ) :: CC (IDO,3,L1)
            C      , &! REAL(KIND=r8)   ,INTENT(OUT ) :: CH (IDO,L1,3)
            WA(IW) , &! REAL(KIND=r8)   ,INTENT(IN  ) :: WA1(*)
            WA(IX2)  )! REAL(KIND=r8)   ,INTENT(IN  ) :: WA2(*)
108    NA = 1-NA
       GO TO 115
109    IF (IP .NE. 5) GO TO 112
       IX2 = IW+IDOT
       IX3 = IX2+IDOT
       IX4 = IX3+IDOT
       IF (NA .NE. 0) GO TO 110
       CALL PASSF5 (IDOT    , &!INTEGER          , INTENT(IN   ) :: IDO
            L1      , &!INTEGER          , INTENT(IN   ) :: L1
            C       , &!REAL(KIND=r8)   , INTENT(IN        ) :: CC(IDO,5,L1)
            CH      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5)
            WA(IW)  , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA1(*)
            WA(IX2) , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA2(*)
            WA(IX3) , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA3(*)
            WA(IX4)   )!REAL(KIND=r8)   , INTENT(IN        ) :: WA4(*)
       GO TO 111
110    CALL PASSF5 (IDOT    , &!INTEGER          , INTENT(IN   ) :: IDO
            L1      , &!INTEGER          , INTENT(IN   ) :: L1
            CH      , &!REAL(KIND=r8)   , INTENT(IN        ) :: CC(IDO,5,L1)
            C       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5)
            WA(IW)  , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA1(*)
            WA(IX2) , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA2(*)
            WA(IX3) , &!REAL(KIND=r8)   , INTENT(IN        ) :: WA3(*)
            WA(IX4)   )!REAL(KIND=r8)   , INTENT(IN        ) :: WA4(*)
111    NA = 1-NA
       GO TO 115
112    IF (NA .NE. 0) GO TO 113
       CALL PASSF (NAC    , & !INTEGER, INTENT(OUT  ) :: NAC
            IDOT   , & !INTEGER, INTENT(IN   ) :: IDO
            IP     , & !INTEGER, INTENT(IN   ) :: IP
            L1     , & !INTEGER, INTENT(IN   ) :: L1
            IDL1   , & !INTEGER, INTENT(IN   ) :: IDL1
            C      , & !REAL(KIND=r8)   , INTENT(IN  ) :: CC(IDO,IP,L1)
            C      , & !REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
            C      , & !REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
            CH     , & !REAL(KIND=r8)   , INTENT(INOUT) :: CH(IDO,L1,IP)
            CH     , & !REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
            WA(IW)   ) !REAL(KIND=r8)   , INTENT(IN  ) :: WA(*)
       GO TO 114
113    CALL PASSF (NAC    , &!INTEGER, INTENT(OUT  ) :: NAC
            IDOT   , &!INTEGER, INTENT(IN   ) :: IDO
            IP     , &!INTEGER, INTENT(IN   ) :: IP
            L1     , &!INTEGER, INTENT(IN   ) :: L1
            IDL1   , &!INTEGER, INTENT(IN   ) :: IDL1
            CH     , &!REAL(KIND=r8)   , INTENT(IN  ) :: CC(IDO,IP,L1)
            CH     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
            CH     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
            C      , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH(IDO,L1,IP)
            C      , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
            WA(IW)   )!REAL(KIND=r8)   , INTENT(IN  ) :: WA(*)
114    IF (NAC .NE. 0) NA = 1-NA
115    L1 = L2
       IW = IW+(IP-1)*IDOT
    END DO
    IF (NA .EQ. 0) RETURN
    N2 = N+N
    DO I=1,N2
       C(I) = CH(I)
       !CC(I)=CMPLX(C(i))
       !PRINT*, C(I)
    END DO
    RETURN
  END  SUBROUTINE CFFTF1

  !****************************************************************************
  SUBROUTINE CFFTB1 (N    , & !INTEGER          , INTENT(IN        ) :: N
       C    , & !REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
       CH   , & !REAL(KIND=r8)   , INTENT(INOUT) :: CH(*)
       WA   , & !REAL(KIND=r8)   , INTENT(IN        ) :: WA(*)
       WIFAC)   !REAL(KIND=r8)   , INTENT(IN        ) :: WIFAC(*)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: N
    REAL(KIND=r8)   , INTENT(INOUT) :: C(*)
    REAL(KIND=r8)   , INTENT(INOUT) :: CH(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WIFAC(*)

    INTEGER NF,NA,L1,IW,K1,IP,L2,IDO,IDOT,IDL1,IX2,IX3,IX4,N2, &
         I,NAC

    NF = NINT(WIFAC(2))
    NA = 0
    L1 = 1
    IW = 1
    DO K1=1,NF
       IP = NINT(WIFAC(K1+2))
       L2 = IP*L1
       IDO = N/L2
       IDOT = IDO+IDO
       IDL1 = IDOT*L1
       IF (IP .NE. 4) GO TO 103
       IX2 = IW+IDOT
       IX3 = IX2+IDOT
       IF (NA .NE. 0) GO TO 101
       CALL PASSB4 (IDOT     , &!  INTEGER             , INTENT(IN   ) :: IDO
            L1       , &!  INTEGER             , INTENT(IN   ) :: L1
            C        , &!  REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,4,L1)
            CH       , &!  REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,4)
            WA(IW)   , &!  REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
            WA(IX2)  , &!  REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
            WA(IX3)    )!  REAL(KIND=r8)   , INTENT(IN   ) :: WA3(*)
       GO TO 102
101    CALL PASSB4 (IDOT     , &!  INTEGER            , INTENT(IN   ) :: IDO
            L1       , &!  INTEGER            , INTENT(IN   ) :: L1
            CH       , &!  REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,4,L1)
            C        , &!  REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,4)
            WA(IW)   , &!  REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
            WA(IX2)  , &!  REAL(KIND=r8)    , INTENT(IN   ):: WA2(*)
            WA(IX3)    )!  REAL(KIND=r8)    , INTENT(IN   ):: WA3(*)
102    NA = 1-NA
       GO TO 115
103    IF (IP .NE. 2) GO TO 106
       IF (NA .NE. 0) GO TO 104
       CALL PASSB2 (IDOT    , & ! INTEGER           , INTENT(IN   ) :: IDO
            L1      , & ! INTEGER           , INTENT(IN   ) :: L1
            C       , & ! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,2,L1) 
            CH      , & ! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,2) 
            WA(IW)    ) ! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(1)
       GO TO 105
104    CALL PASSB2 (IDOT   , & ! INTEGER          , INTENT(IN        ) :: IDO
            L1     , & ! INTEGER          , INTENT(IN        ) :: L1
            CH     , & ! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,2,L1) 
            C      , & ! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,2) 
            WA(IW)   ) ! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(1)
105    NA = 1-NA
       GO TO 115
106    IF (IP .NE. 3) GO TO 109
       IX2 = IW+IDOT
       IF (NA .NE. 0) GO TO 107
       CALL PASSB3 (IDOT    , &! INTEGER, INTENT(IN        ) :: IDO
            L1      , &! INTEGER, INTENT(IN        ) :: L1
            C       , &! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,3,L1)
            CH      , &! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,3)
            WA(IW)  , &! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
            WA(IX2)   )! REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
       GO TO 108
107    CALL PASSB3 (IDOT    , & ! INTEGER, INTENT(IN   ) :: IDO
            L1      , & ! INTEGER, INTENT(IN   ) :: L1
            CH      , & ! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,3,L1)
            C       , & ! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,3)
            WA(IW)  , & ! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
            WA(IX2)   ) ! REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
108    NA = 1-NA
       GO TO 115
109    IF (IP .NE. 5) GO TO 112
       IX2 = IW+IDOT
       IX3 = IX2+IDOT
       IX4 = IX3+IDOT
       IF (NA .NE. 0) GO TO 110
       CALL PASSB5 (IDOT   , & !INTEGER          , INTENT(IN        ) :: IDO
            L1     , & !INTEGER          , INTENT(IN        ) :: L1
            C      , & !REAL(KIND=r8)   , INTENT(IN        ) :: CC(IDO,5,L1)
            CH     , & !REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5) 
            WA(IW) , & !REAL(KIND=r8)   , INTENT(IN        ) :: WA1(*)
            WA(IX2), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA2(*)
            WA(IX3), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA3(*)
            WA(IX4)  ) !REAL(KIND=r8)   , INTENT(IN        ) :: WA4(*)
       GO TO 111
110    CALL PASSB5 (IDOT   , & !INTEGER          , INTENT(IN        ) :: IDO
            L1     , & !INTEGER          , INTENT(IN        ) :: L1
            CH     , & !REAL(KIND=r8)   , INTENT(IN        ) :: CC(IDO,5,L1)
            C      , & !REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5) 
            WA(IW) , & !REAL(KIND=r8)   , INTENT(IN        ) :: WA1(*)
            WA(IX2), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA2(*)
            WA(IX3), & !REAL(KIND=r8)   , INTENT(IN        ) :: WA3(*)
            WA(IX4)  ) !REAL(KIND=r8)   , INTENT(IN        ) :: WA4(*)
111    NA = 1-NA
       GO TO 115
112    IF (NA .NE. 0) GO TO 113
       CALL PASSB (NAC  , &!INTEGER             , INTENT(OUT  ) :: NAC
            IDOT , &!INTEGER             , INTENT(IN   ) :: IDO
            IP   , &!INTEGER             , INTENT(IN   ) :: IP
            L1   , &!INTEGER             , INTENT(IN   ) :: L1
            IDL1 , &!INTEGER             , INTENT(IN   ) :: IDL1
            C    , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,IP,L1)
            C    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
            C    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
            CH   , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,IP)
            CH   , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
            WA(IW) )!REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)
       GO TO 114
113    CALL PASSB (NAC  , &!INTEGER             , INTENT(OUT  ) :: NAC
            IDOT , &!INTEGER             , INTENT(IN   ) :: IDO
            IP   , &!INTEGER             , INTENT(IN   ) :: IP
            L1   , &!INTEGER             , INTENT(IN   ) :: L1
            IDL1 , &!INTEGER             , INTENT(IN   ) :: IDL1
            CH   , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,IP,L1)
            CH   , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
            CH   , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
            C    , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,IP)
            C    , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
            WA(IW) )!REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)
114    IF (NAC .NE. 0) NA = 1-NA
115    L1 = L2
       IW = IW+(IP-1)*IDOT
    END DO
    IF (NA .EQ. 0) RETURN
    N2 = N+N
    DO I=1,N2
       C(I) = CH(I)
    END DO
    RETURN
  END SUBROUTINE CFFTB1

  !****************************************************************************
  SUBROUTINE PASSB (NAC  , & !INTEGER         , INTENT(OUT  ) :: NAC
       IDO  , & !INTEGER         , INTENT(IN   ) :: IDO
       IP   , & !INTEGER         , INTENT(IN   ) :: IP
       L1   , & !INTEGER         , INTENT(IN   ) :: L1
       IDL1 , & !INTEGER         , INTENT(IN   ) :: IDL1
       CC   , & !REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,IP,L1)
       C1   , & !REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
       C2   , & !REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
       CH   , & !REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,IP)
       CH2  , & !REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
       WA     ) !REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)
    IMPLICIT NONE
    INTEGER         , INTENT(OUT  ) :: NAC
    INTEGER         , INTENT(IN   ) :: IDO
    INTEGER         , INTENT(IN   ) :: IP
    INTEGER         , INTENT(IN   ) :: L1
    INTEGER         , INTENT(IN   ) :: IDL1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,IP,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
    REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,IP)
    REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)

    REAL(KIND=r8)    :: WAR
    REAL(KIND=r8)    :: WAI
    INTEGER :: IDOT,IPP2,IPPH,IDP,J,K,I,IDL,LC,L, &
         IK,IDLJ,JC,INC,IDIJ,IDJ

    IDOT = IDO/2
    IPP2 = IP+2
    IPPH = (IP+1)/2
    IDP = IP*IDO
    !
    IF (IDO .LT. L1) GO TO 106
    DO J=2,IPPH
       JC = IPP2-J
       DO K=1,L1
          DO I=1,IDO
             CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
             CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
          END DO
       END DO
    END DO
    DO K=1,L1
       DO I=1,IDO
          CH(I,K,1) = CC(I,1,K)
       END DO
    END DO
    GO TO 112
106 CONTINUE
    DO J=2,IPPH
       JC = IPP2-J
       DO I=1,IDO
          DO K=1,L1
             CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
             CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
          END DO
       END DO
    END DO
    DO I=1,IDO
       DO K=1,L1
          CH(I,K,1) = CC(I,1,K)
       END DO
    END DO
112 IDL = 2-IDO
    INC = 0
    DO L=2,IPPH
       LC = IPP2-L
       IDL = IDL+IDO
       DO IK=1,IDL1
          C2(IK,L) = CH2(IK,1)+WA(IDL-1)*CH2(IK,2)
          C2(IK,LC) = WA(IDL)*CH2(IK,IP)
       END DO
       IDLJ = IDL
       INC = INC+IDO
       DO J=3,IPPH
          JC = IPP2-J
          IDLJ = IDLJ+INC
          IF (IDLJ .GT. IDP) IDLJ = IDLJ-IDP
          WAR = WA(IDLJ-1)
          WAI = WA(IDLJ)
          DO IK=1,IDL1
             C2(IK,L) = C2(IK,L)+WAR*CH2(IK,J)
             C2(IK,LC) = C2(IK,LC)+WAI*CH2(IK,JC)
          END DO
       END DO
    END DO
    DO J=2,IPPH
       DO IK=1,IDL1
          CH2(IK,1) = CH2(IK,1)+CH2(IK,J)
       END DO
    END DO
    DO J=2,IPPH
       JC = IPP2-J
       DO IK=2,IDL1,2
          CH2(IK-1,J) = C2(IK-1,J)-C2(IK,JC)
          CH2(IK-1,JC) = C2(IK-1,J)+C2(IK,JC)
          CH2(IK,J) = C2(IK,J)+C2(IK-1,JC)
          CH2(IK,JC) = C2(IK,J)-C2(IK-1,JC)
       END DO
    END DO
    NAC = 1
    IF (IDO .EQ. 2) RETURN
    NAC = 0
    DO IK=1,IDL1
       C2(IK,1) = CH2(IK,1)
    END DO
    DO  J=2,IP
       DO  K=1,L1
          C1(1,K,J) = CH(1,K,J)
          C1(2,K,J) = CH(2,K,J)
       END DO
123 END DO
    IF (IDOT .GT. L1) GO TO 127
    IDIJ = 0
    DO J=2,IP
       IDIJ = IDIJ+2
       DO I=4,IDO,2
          IDIJ = IDIJ+2
          DO K=1,L1
             C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
             C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)
          END DO
       END DO
    END DO
    RETURN
127 IDJ = 2-IDO
    DO J=2,IP
       IDJ = IDJ+IDO
       DO K=1,L1
          IDIJ = IDJ
          DO I=4,IDO,2
             IDIJ = IDIJ+2
             C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
             C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)
          END DO
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSB

  !****************************************************************************
  SUBROUTINE PASSB2 (IDO   , & ! INTEGER            , INTENT(IN   ) :: IDO
       L1    , & ! INTEGER            , INTENT(IN   ) :: L1
       CC    , & ! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,2,L1) 
       CH    , & ! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,2) 
       WA1     ) ! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(1)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: IDO
    INTEGER         , INTENT(IN   ) :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,2,L1) 
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,2) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)

    REAL(KIND=r8)    :: TR2,TI2

    INTEGER :: K,I

    IF (IDO .GT. 2) GO TO 102
    DO  K=1,L1
       CH(1,K,1) = CC(1,1,K)+CC(1,2,K)
       CH(1,K,2) = CC(1,1,K)-CC(1,2,K)
       CH(2,K,1) = CC(2,1,K)+CC(2,2,K)
       CH(2,K,2) = CC(2,1,K)-CC(2,2,K)
    END DO
    RETURN
102 DO  K=1,L1
       DO I=2,IDO,2
          CH(I-1,K,1) = CC(I-1,1,K)+CC(I-1,2,K)
          TR2         = CC(I-1,1,K)-CC(I-1,2,K)
          CH(I,K,1)   = CC(I,1,K)+CC(I,2,K)
          TI2         = CC(I,1,K)-CC(I,2,K)
          CH(I,K,2)   = WA1(I-1)*TI2+WA1(I)*TR2
          CH(I-1,K,2) = WA1(I-1)*TR2-WA1(I)*TI2
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSB2

  !****************************************************************************
  SUBROUTINE PASSB3 (IDO     , &! INTEGER, INTENT(IN   ) :: IDO
       L1      , &! INTEGER, INTENT(IN   ) :: L1
       CC      , &! REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,3,L1)
       CH      , &! REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,3)
       WA1     , &! REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
       WA2       )! REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
    IMPLICIT NONE
    INTEGER, INTENT(IN   ) :: IDO
    INTEGER, INTENT(IN   ) :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,3,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,3)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
    REAL(KIND=r8)          :: TR2,CR2,TI2,CI2,CR3,CI3, &
         DR2,DR3,DI2,DI3
    INTEGER :: K
    INTEGER :: I

    REAL(KIND=r8)   , PARAMETER :: TAUR=-0.5D0
    REAL(KIND=r8)   , PARAMETER :: TAUI=0.866025403784439D0

    IF (IDO .NE. 2) GO TO 102
    DO  K=1,L1
       TR2       = CC(1,2,K)+CC(1,3,K)
       CR2       = CC(1,1,K)+TAUR*TR2
       CH(1,K,1) = CC(1,1,K)+TR2
       TI2       = CC(2,2,K)+CC(2,3,K)
       CI2       = CC(2,1,K)+TAUR*TI2
       CH(2,K,1) = CC(2,1,K)+TI2
       CR3       = TAUI*(CC(1,2,K)-CC(1,3,K))
       CI3       = TAUI*(CC(2,2,K)-CC(2,3,K))
       CH(1,K,2) = CR2-CI3
       CH(1,K,3) = CR2+CI3
       CH(2,K,2) = CI2+CR3
       CH(2,K,3) = CI2-CR3
    END DO
    RETURN
102 DO K=1,L1
       DO  I=2,IDO,2
          TR2 = CC(I-1,2,K)+CC(I-1,3,K)
          CR2 = CC(I-1,1,K)+TAUR*TR2
          CH(I-1,K,1) = CC(I-1,1,K)+TR2
          TI2 = CC(I,2,K)+CC(I,3,K)
          CI2 = CC(I,1,K)+TAUR*TI2
          CH(I,K,1) = CC(I,1,K)+TI2
          CR3 = TAUI*(CC(I-1,2,K)-CC(I-1,3,K))
          CI3 = TAUI*(CC(I,2,K)-CC(I,3,K))
          DR2 = CR2-CI3
          DR3 = CR2+CI3
          DI2 = CI2+CR3
          DI3 = CI2-CR3
          CH(I,K,2)   = WA1(I-1)*DI2+WA1(I)*DR2
          CH(I-1,K,2) = WA1(I-1)*DR2-WA1(I)*DI2
          CH(I,K,3)   = WA2(I-1)*DI3+WA2(I)*DR3
          CH(I-1,K,3) = WA2(I-1)*DR3-WA2(I)*DI3
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSB3

  !****************************************************************************
  SUBROUTINE PASSB4 (IDO    , &!  INTEGER             , INTENT(IN   ) :: IDO
       L1     , &!  INTEGER             , INTENT(IN   ) :: L1
       CC     , &!  REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,4,L1)
       CH     , &!  REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,4)
       WA1    , &!  REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
       WA2    , &!  REAL(KIND=r8)    , INTENT(IN   ):: WA2(*)
       WA3      )!  REAL(KIND=r8)    , INTENT(IN   ):: WA3(*)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: IDO
    INTEGER         , INTENT(IN   ) :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,4,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,4)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
    REAL(KIND=r8)    , INTENT(IN   ):: WA2(*)
    REAL(KIND=r8)    , INTENT(IN   ):: WA3(*)

    REAL(KIND=r8)    :: TI1,TI2,TI3,TI4,TR1,TR2,TR3, &
         TR4,CR3,CR2,CI3,CR4,CI4,CI2
    INTEGER :: K,I

    IF (IDO .NE. 2) GO TO 102
    DO K=1,L1
       TI1 = CC(2,1,K)-CC(2,3,K)
       TI2 = CC(2,1,K)+CC(2,3,K)
       TR4 = CC(2,4,K)-CC(2,2,K)
       TI3 = CC(2,2,K)+CC(2,4,K)
       TR1 = CC(1,1,K)-CC(1,3,K)
       TR2 = CC(1,1,K)+CC(1,3,K)
       TI4 = CC(1,2,K)-CC(1,4,K)
       TR3 = CC(1,2,K)+CC(1,4,K)
       CH(1,K,1) = TR2+TR3
       CH(1,K,3) = TR2-TR3
       CH(2,K,1) = TI2+TI3
       CH(2,K,3) = TI2-TI3
       CH(1,K,2) = TR1+TR4
       CH(1,K,4) = TR1-TR4
       CH(2,K,2) = TI1+TI4
       CH(2,K,4) = TI1-TI4
    END DO
    RETURN
102 DO K=1,L1
       DO I=2,IDO,2
          TI1 = CC(I,1,K)-CC(I,3,K)
          TI2 = CC(I,1,K)+CC(I,3,K)
          TI3 = CC(I,2,K)+CC(I,4,K)
          TR4 = CC(I,4,K)-CC(I,2,K)
          TR1 = CC(I-1,1,K)-CC(I-1,3,K)
          TR2 = CC(I-1,1,K)+CC(I-1,3,K)
          TI4 = CC(I-1,2,K)-CC(I-1,4,K)
          TR3 = CC(I-1,2,K)+CC(I-1,4,K)
          CH(I-1,K,1) = TR2+TR3
          CR3 = TR2-TR3
          CH(I,K,1) = TI2+TI3
          CI3 = TI2-TI3
          CR2 = TR1+TR4
          CR4 = TR1-TR4
          CI2 = TI1+TI4
          CI4 = TI1-TI4
          CH(I-1,K,2) = WA1(I-1)*CR2-WA1(I)*CI2
          CH(I,K,2)   = WA1(I-1)*CI2+WA1(I)*CR2
          CH(I-1,K,3) = WA2(I-1)*CR3-WA2(I)*CI3
          CH(I,K,3)   = WA2(I-1)*CI3+WA2(I)*CR3
          CH(I-1,K,4) = WA3(I-1)*CR4-WA3(I)*CI4
          CH(I,K,4)   = WA3(I-1)*CI4+WA3(I)*CR4
       END DO
    END DO
    RETURN
  END  SUBROUTINE PASSB4

  !****************************************************************************
  SUBROUTINE PASSB5 (IDO    , &!INTEGER            , INTENT(IN   ) :: IDO
       L1     , &!INTEGER            , INTENT(IN   ) :: L1
       CC     , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,5,L1)
       CH     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5) 
       WA1    , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
       WA2    , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
       WA3    , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA3(*)
       WA4      )!REAL(KIND=r8)   , INTENT(IN   ) :: WA4(*)

    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: IDO
    INTEGER         , INTENT(IN   ) :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,5,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA3(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA4(*)

    REAL(KIND=r8)    :: TI2,TI3,TI4,TI5,TR2,TR3,TR4,TR5,CR2,CI2,CR3,CI3,CR4,CI4,CR5,CI5, &
         DR3,DI3,DR4,DI4,DR5,DI5,DR2,DI2

    REAL(KIND=r8)   , PARAMETER:: TR11 =  .309016994374947D0
    REAL(KIND=r8)   , PARAMETER:: TI11 =  .951056516295154D0
    REAL(KIND=r8)   , PARAMETER:: TR12 =  -.809016994374947D0
    REAL(KIND=r8)   , PARAMETER:: TI12 =  .587785252292473D0




    INTEGER ::  K,I
    IF (IDO .NE. 2) GO TO 102
    DO K=1,L1
       TI5 = CC(2,2,K)-CC(2,5,K)
       TI2 = CC(2,2,K)+CC(2,5,K)
       TI4 = CC(2,3,K)-CC(2,4,K)
       TI3 = CC(2,3,K)+CC(2,4,K)
       TR5 = CC(1,2,K)-CC(1,5,K)
       TR2 = CC(1,2,K)+CC(1,5,K)
       TR4 = CC(1,3,K)-CC(1,4,K)
       TR3 = CC(1,3,K)+CC(1,4,K)
       CH(1,K,1) = CC(1,1,K)+TR2+TR3
       CH(2,K,1) = CC(2,1,K)+TI2+TI3
       CR2 = CC(1,1,K)+TR11*TR2+TR12*TR3
       CI2 = CC(2,1,K)+TR11*TI2+TR12*TI3
       CR3 = CC(1,1,K)+TR12*TR2+TR11*TR3
       CI3 = CC(2,1,K)+TR12*TI2+TR11*TI3
       CR5 = TI11*TR5+TI12*TR4
       CI5 = TI11*TI5+TI12*TI4
       CR4 = TI12*TR5-TI11*TR4
       CI4 = TI12*TI5-TI11*TI4
       CH(1,K,2) = CR2-CI5
       CH(1,K,5) = CR2+CI5
       CH(2,K,2) = CI2+CR5
       CH(2,K,3) = CI3+CR4
       CH(1,K,3) = CR3-CI4
       CH(1,K,4) = CR3+CI4
       CH(2,K,4) = CI3-CR4
       CH(2,K,5) = CI2-CR5
    END DO
    RETURN
102 DO  K=1,L1
       DO  I=2,IDO,2
          TI5 = CC(I,2,K)-CC(I,5,K)
          TI2 = CC(I,2,K)+CC(I,5,K)
          TI4 = CC(I,3,K)-CC(I,4,K)
          TI3 = CC(I,3,K)+CC(I,4,K)
          TR5 = CC(I-1,2,K)-CC(I-1,5,K)
          TR2 = CC(I-1,2,K)+CC(I-1,5,K)
          TR4 = CC(I-1,3,K)-CC(I-1,4,K)
          TR3 = CC(I-1,3,K)+CC(I-1,4,K)
          CH(I-1,K,1) = CC(I-1,1,K)+TR2+TR3
          CH(I,K,1) = CC(I,1,K)+TI2+TI3
          CR2 = CC(I-1,1,K)+TR11*TR2+TR12*TR3
          CI2 = CC(I,1,K)+TR11*TI2+TR12*TI3
          CR3 = CC(I-1,1,K)+TR12*TR2+TR11*TR3
          CI3 = CC(I,1,K)+TR12*TI2+TR11*TI3
          CR5 = TI11*TR5+TI12*TR4
          CI5 = TI11*TI5+TI12*TI4
          CR4 = TI12*TR5-TI11*TR4
          CI4 = TI12*TI5-TI11*TI4
          DR3 = CR3-CI4
          DR4 = CR3+CI4
          DI3 = CI3+CR4
          DI4 = CI3-CR4
          DR5 = CR2+CI5
          DR2 = CR2-CI5
          DI5 = CI2-CR5
          DI2 = CI2+CR5
          CH(I-1,K,2) = WA1(I-1)*DR2-WA1(I)*DI2
          CH(I,K,2)   = WA1(I-1)*DI2+WA1(I)*DR2
          CH(I-1,K,3) = WA2(I-1)*DR3-WA2(I)*DI3
          CH(I,K,3)   = WA2(I-1)*DI3+WA2(I)*DR3
          CH(I-1,K,4) = WA3(I-1)*DR4-WA3(I)*DI4
          CH(I,K,4)   = WA3(I-1)*DI4+WA3(I)*DR4
          CH(I-1,K,5) = WA4(I-1)*DR5-WA4(I)*DI5
          CH(I,K,5)   = WA4(I-1)*DI5+WA4(I)*DR5
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSB5

  !****************************************************************************
  SUBROUTINE PASSF (NAC    , &!INTEGER, INTENT(OUT  ) :: NAC
       IDO    , &!INTEGER, INTENT(IN   ) :: IDO
       IP     , &!INTEGER, INTENT(IN   ) :: IP
       L1     , &!INTEGER, INTENT(IN   ) :: L1
       IDL1   , &!INTEGER, INTENT(IN   ) :: IDL1
       CC     , &!REAL(KIND=r8)   , INTENT(IN        ) :: CC(IDO,IP,L1)
       C1     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
       C2     , &!REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
       CH     , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH(IDO,L1,IP)
       CH2    , &!REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
       WA       )!REAL(KIND=r8)   , INTENT(IN        ) :: WA(*)
    IMPLICIT NONE
    INTEGER, INTENT(OUT  ) :: NAC
    INTEGER, INTENT(IN   ) :: IDO
    INTEGER, INTENT(IN   ) :: IP
    INTEGER, INTENT(IN   ) :: L1
    INTEGER, INTENT(IN   ) :: IDL1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,IP,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: C1(IDO,L1,IP)
    REAL(KIND=r8)   , INTENT(OUT  ) :: C2(IDL1,IP)
    REAL(KIND=r8)   , INTENT(INOUT) :: CH(IDO,L1,IP)
    REAL(KIND=r8)   , INTENT(INOUT) :: CH2(IDL1,IP)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA(*)

    REAL(KIND=r8)    :: WAR,WAI
    INTEGER :: IDOT,IPP2,IPPH,IDP,J,JC,K,I,IDL, &
         INC,L,LC,IK,IDLJ,IDIJ,IDJ

    IDOT = IDO/2
    IPP2 = IP+2
    IPPH = (IP+1)/2
    IDP = IP*IDO
    !
    IF (IDO .LT. L1) GO TO 106
    DO J=2,IPPH
       JC = IPP2-J
       DO  K=1,L1
          DO  I=1,IDO
             CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
             CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
          END DO
       END DO
    END DO
    DO K=1,L1
       DO  I=1,IDO
          CH(I,K,1) = CC(I,1,K)
       END DO
    END DO
    GO TO 112
106 CONTINUE
    DO J=2,IPPH
       JC = IPP2-J
       DO I=1,IDO
          DO K=1,L1
             CH(I,K,J) = CC(I,J,K)+CC(I,JC,K)
             CH(I,K,JC) = CC(I,J,K)-CC(I,JC,K)
          END DO
       END DO
    END DO
    DO I=1,IDO
       DO K=1,L1
          CH(I,K,1) = CC(I,1,K)
       END DO
    END DO
112 IDL = 2-IDO
    INC = 0
    DO L=2,IPPH
       LC = IPP2-L
       IDL = IDL+IDO
       DO IK=1,IDL1
          C2(IK,L) = CH2(IK,1)+WA(IDL-1)*CH2(IK,2)
          C2(IK,LC) = -WA(IDL)*CH2(IK,IP)
       END DO
       IDLJ = IDL
       INC = INC+IDO
       DO J=3,IPPH
          JC = IPP2-J
          IDLJ = IDLJ+INC
          IF (IDLJ .GT. IDP) IDLJ = IDLJ-IDP
          WAR = WA(IDLJ-1)
          WAI = WA(IDLJ)
          DO IK=1,IDL1
             C2(IK,L) = C2(IK,L)+WAR*CH2(IK,J)
             C2(IK,LC) = C2(IK,LC)-WAI*CH2(IK,JC)
          END DO
       END DO
    END DO
    DO J=2,IPPH
       DO IK=1,IDL1
          CH2(IK,1) = CH2(IK,1)+CH2(IK,J)
       END DO
    END DO
    DO J=2,IPPH
       JC = IPP2-J
       DO IK=2,IDL1,2
          CH2(IK-1,J) = C2(IK-1,J)-C2(IK,JC)
          CH2(IK-1,JC) = C2(IK-1,J)+C2(IK,JC)
          CH2(IK,J) = C2(IK,J)+C2(IK-1,JC)
          CH2(IK,JC) = C2(IK,J)-C2(IK-1,JC)
       END DO
    END DO
    NAC = 1
    IF (IDO .EQ. 2) RETURN
    NAC = 0
    DO IK=1,IDL1
       C2(IK,1) = CH2(IK,1)
    END DO
    DO J=2,IP
       DO K=1,L1
          C1(1,K,J) = CH(1,K,J)
          C1(2,K,J) = CH(2,K,J)
       END DO
    END DO
    IF (IDOT .GT. L1) GO TO 127
    IDIJ = 0
    DO J=2,IP
       IDIJ = IDIJ+2
       DO I=4,IDO,2
          IDIJ = IDIJ+2
          DO K=1,L1
             C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)+WA(IDIJ)*CH(I,K,J)
             C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)-WA(IDIJ)*CH(I-1,K,J)
          END DO
       END DO
    END DO
    RETURN
127 IDJ = 2-IDO
    DO J=2,IP
       IDJ = IDJ+IDO
       DO K=1,L1
          IDIJ = IDJ
          DO I=4,IDO,2
             IDIJ = IDIJ+2
             C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)+WA(IDIJ)*CH(I,K,J)
             C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)-WA(IDIJ)*CH(I-1,K,J)
          END DO
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSF

  !****************************************************************************
  SUBROUTINE PASSF2 (IDO   , &!  INTEGER             , INTENT(IN   ) :: IDO   
       L1    , &!  INTEGER             , INTENT(IN   ) :: L1    
       CC    , &!  REAL(KIND=r8)   , INTENT(IN   ) :: CC (IDO,2,L1)   
       CH    , &!  REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,2)
       WA1     )!  REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)        
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: IDO   
    INTEGER         , INTENT(IN   ) :: L1    
    REAL(KIND=r8)   , INTENT(IN   ) :: CC (IDO,2,L1)   
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,2)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)   
    INTEGER          :: K,I
    REAL(KIND=r8)    :: TR2,TI2

    IF (IDO .GT. 2) GO TO 102
    DO K=1,L1
       CH(1,K,1) = CC(1,1,K)+CC(1,2,K)
       CH(1,K,2) = CC(1,1,K)-CC(1,2,K)
       CH(2,K,1) = CC(2,1,K)+CC(2,2,K)
       CH(2,K,2) = CC(2,1,K)-CC(2,2,K)
    END DO
    RETURN
102 DO K=1,L1
       DO I=2,IDO,2
          CH(I-1,K,1) = CC(I-1,1,K)+CC(I-1,2,K)
          TR2         = CC(I-1,1,K)-CC(I-1,2,K)
          CH(I,K,1)   = CC(I,1,K)+CC(I,2,K)
          TI2         = CC(I,1,K)-CC(I,2,K)
          CH(I,K,2)   = WA1(I-1)*TI2-WA1(I)*TR2
          CH(I-1,K,2) = WA1(I-1)*TR2+WA1(I)*TI2
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSF2

  !****************************************************************************
  SUBROUTINE PASSF3 (IDO   , &! INTEGER            ,INTENT(IN        ) :: IDO
       L1    , &! INTEGER            ,INTENT(IN        ) :: L1
       CC    , &! REAL(KIND=r8)   ,INTENT(IN        ) :: CC (IDO,3,L1)
       CH    , &! REAL(KIND=r8)   ,INTENT(OUT ) :: CH (IDO,L1,3)
       WA1   , &! REAL(KIND=r8)   ,INTENT(IN        ) :: WA1(*)
       WA2     )! REAL(KIND=r8)   ,INTENT(IN        ) :: WA2(*)
    IMPLICIT NONE
    INTEGER         ,INTENT(IN   ) :: IDO
    INTEGER         ,INTENT(IN   ) :: L1
    REAL(KIND=r8)   ,INTENT(IN   ) :: CC (IDO,3,L1)
    REAL(KIND=r8)   ,INTENT(OUT  ) :: CH (IDO,L1,3)
    REAL(KIND=r8)   ,INTENT(IN   ) :: WA1(*)
    REAL(KIND=r8)   ,INTENT(IN   ) :: WA2(*)

    REAL(KIND=r8)         :: TR2,CR2,TI2,CI2,CR3,CI3, &
         DR2,DR3,DI2,DI3
    REAL(KIND=r8)   , PARAMETER :: TAUR=-.5D0
    REAL(KIND=r8)   , PARAMETER :: TAUI=-.866025403784439D0
    INTEGER K,I

    IF (IDO .NE. 2) GO TO 102
    DO K=1,L1
       TR2 = CC(1,2,K)+CC(1,3,K)
       CR2 = CC(1,1,K)+TAUR*TR2
       CH(1,K,1) = CC(1,1,K)+TR2
       TI2 = CC(2,2,K)+CC(2,3,K)
       CI2 = CC(2,1,K)+TAUR*TI2
       CH(2,K,1) = CC(2,1,K)+TI2
       CR3 = TAUI*(CC(1,2,K)-CC(1,3,K))
       CI3 = TAUI*(CC(2,2,K)-CC(2,3,K))
       CH(1,K,2) = CR2-CI3
       CH(1,K,3) = CR2+CI3
       CH(2,K,2) = CI2+CR3
       CH(2,K,3) = CI2-CR3
    END DO
    RETURN
102 DO K=1,L1
       DO I=2,IDO,2
          TR2 = CC(I-1,2,K)+CC(I-1,3,K)
          CR2 = CC(I-1,1,K)+TAUR*TR2
          CH(I-1,K,1) = CC(I-1,1,K)+TR2
          TI2 = CC(I,2,K)+CC(I,3,K)
          CI2 = CC(I,1,K)+TAUR*TI2
          CH(I,K,1) = CC(I,1,K)+TI2
          CR3 = TAUI*(CC(I-1,2,K)-CC(I-1,3,K))
          CI3 = TAUI*(CC(I,2,K)-CC(I,3,K))
          DR2 = CR2-CI3
          DR3 = CR2+CI3
          DI2 = CI2+CR3
          DI3 = CI2-CR3
          CH(I,K,2) = WA1(I-1)*DI2-WA1(I)*DR2
          CH(I-1,K,2) = WA1(I-1)*DR2+WA1(I)*DI2
          CH(I,K,3) = WA2(I-1)*DI3-WA2(I)*DR3
          CH(I-1,K,3) = WA2(I-1)*DR3+WA2(I)*DI3
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSF3

  !****************************************************************************
  SUBROUTINE PASSF4 (IDO     , &!INTEGER, INTENT(IN   )             :: IDO
       L1      , &!INTEGER, INTENT(IN   )             :: L1
       CC      , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC    (IDO,4,L1)
       CH      , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,4)  
       WA1     , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA1   (*) 
       WA2     , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA2   (*) 
       WA3       )!REAL(KIND=r8)   , INTENT(IN   ) :: WA3   (*) 
    IMPLICIT NONE
    INTEGER, INTENT(IN   )          :: IDO
    INTEGER, INTENT(IN   )          :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC    (IDO,4,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH    (IDO,L1,4) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1   (*) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA2   (*) 
    REAL(KIND=r8)   , INTENT(IN   ) :: WA3   (*) 

    REAL(KIND=r8)    :: TI1,TI2,TI3,TI4,TR1,TR2,TR3, &
         TR4,CR2,CR3,CR4,CI2,CI3,CI4
    INTEGER K,I

    IF (IDO .NE. 2) GO TO 102
    DO K=1,L1
       TI1 = CC(2,1,K)-CC(2,3,K)
       TI2 = CC(2,1,K)+CC(2,3,K)
       TR4 = CC(2,2,K)-CC(2,4,K)
       TI3 = CC(2,2,K)+CC(2,4,K)
       TR1 = CC(1,1,K)-CC(1,3,K)
       TR2 = CC(1,1,K)+CC(1,3,K)
       TI4 = CC(1,4,K)-CC(1,2,K)
       TR3 = CC(1,2,K)+CC(1,4,K)
       CH(1,K,1) = TR2+TR3
       CH(1,K,3) = TR2-TR3
       CH(2,K,1) = TI2+TI3
       CH(2,K,3) = TI2-TI3
       CH(1,K,2) = TR1+TR4
       CH(1,K,4) = TR1-TR4
       CH(2,K,2) = TI1+TI4
       CH(2,K,4) = TI1-TI4
    END DO
    RETURN
102 DO K=1,L1
       DO I=2,IDO,2
          TI1 = CC(I,1,K)-CC(I,3,K)
          TI2 = CC(I,1,K)+CC(I,3,K)
          TI3 = CC(I,2,K)+CC(I,4,K)
          TR4 = CC(I,2,K)-CC(I,4,K)
          TR1 = CC(I-1,1,K)-CC(I-1,3,K)
          TR2 = CC(I-1,1,K)+CC(I-1,3,K)
          TI4 = CC(I-1,4,K)-CC(I-1,2,K)
          TR3 = CC(I-1,2,K)+CC(I-1,4,K)
          CH(I-1,K,1) = TR2+TR3
          CR3 = TR2-TR3
          CH(I,K,1) = TI2+TI3
          CI3 = TI2-TI3
          CR2 = TR1+TR4
          CR4 = TR1-TR4
          CI2 = TI1+TI4
          CI4 = TI1-TI4
          CH(I-1,K,2) = WA1(I-1)*CR2+WA1(I)*CI2
          CH(I,K,2) = WA1(I-1)*CI2-WA1(I)*CR2
          CH(I-1,K,3) = WA2(I-1)*CR3+WA2(I)*CI3
          CH(I,K,3) = WA2(I-1)*CI3-WA2(I)*CR3
          CH(I-1,K,4) = WA3(I-1)*CR4+WA3(I)*CI4
          CH(I,K,4) = WA3(I-1)*CI4-WA3(I)*CR4
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSF4

  !****************************************************************************
  SUBROUTINE PASSF5 (IDO      , &!INTEGER             , INTENT(IN   ) :: IDO
       L1       , &!INTEGER             , INTENT(IN   ) :: L1
       CC       , &!REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,5,L1)
       CH       , &!REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5)
       WA1      , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
       WA2      , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
       WA3      , &!REAL(KIND=r8)   , INTENT(IN   ) :: WA3(*)
       WA4        )!REAL(KIND=r8)   , INTENT(IN   ) :: WA4(*)
    IMPLICIT NONE
    INTEGER         , INTENT(IN   ) :: IDO
    INTEGER         , INTENT(IN   ) :: L1
    REAL(KIND=r8)   , INTENT(IN   ) :: CC(IDO,5,L1)
    REAL(KIND=r8)   , INTENT(OUT  ) :: CH(IDO,L1,5)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA1(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA2(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA3(*)
    REAL(KIND=r8)   , INTENT(IN   ) :: WA4(*)
    INTEGER          :: K,I

    REAL(KIND=r8)    :: TI2,TI3,TI4,TI5,TR2,TR3,TR4,TR5,CI2,CI3,CI4,CI5,CR2,CR3,CR4, &
         CR5,DI2,DI3,DI4,DI5,DR2,DR3,DR4,DR5

    REAL(KIND=r8)   ,  PARAMETER :: TR11= .309016994374947D0
    REAL(KIND=r8)   ,  PARAMETER :: TI11=-.951056516295154D0
    REAL(KIND=r8)   ,  PARAMETER :: TR12=-.809016994374947D0
    REAL(KIND=r8)   ,  PARAMETER :: TI12=-.587785252292473D0

    IF (IDO .NE. 2) GO TO 102
    DO K=1,L1
       TI5 = CC(2,2,K)-CC(2,5,K)
       TI2 = CC(2,2,K)+CC(2,5,K)
       TI4 = CC(2,3,K)-CC(2,4,K)
       TI3 = CC(2,3,K)+CC(2,4,K)
       TR5 = CC(1,2,K)-CC(1,5,K)
       TR2 = CC(1,2,K)+CC(1,5,K)
       TR4 = CC(1,3,K)-CC(1,4,K)
       TR3 = CC(1,3,K)+CC(1,4,K)
       CH(1,K,1) = CC(1,1,K)+TR2+TR3
       CH(2,K,1) = CC(2,1,K)+TI2+TI3
       CR2 = CC(1,1,K)+TR11*TR2+TR12*TR3
       CI2 = CC(2,1,K)+TR11*TI2+TR12*TI3
       CR3 = CC(1,1,K)+TR12*TR2+TR11*TR3
       CI3 = CC(2,1,K)+TR12*TI2+TR11*TI3
       CR5 = TI11*TR5+TI12*TR4
       CI5 = TI11*TI5+TI12*TI4
       CR4 = TI12*TR5-TI11*TR4
       CI4 = TI12*TI5-TI11*TI4
       CH(1,K,2) = CR2-CI5
       CH(1,K,5) = CR2+CI5
       CH(2,K,2) = CI2+CR5
       CH(2,K,3) = CI3+CR4
       CH(1,K,3) = CR3-CI4
       CH(1,K,4) = CR3+CI4
       CH(2,K,4) = CI3-CR4
       CH(2,K,5) = CI2-CR5
    END DO
    RETURN
102 DO  K=1,L1
       DO  I=2,IDO,2
          TI5 = CC(I,2,K)-CC(I,5,K)
          TI2 = CC(I,2,K)+CC(I,5,K)
          TI4 = CC(I,3,K)-CC(I,4,K)
          TI3 = CC(I,3,K)+CC(I,4,K)
          TR5 = CC(I-1,2,K)-CC(I-1,5,K)
          TR2 = CC(I-1,2,K)+CC(I-1,5,K)
          TR4 = CC(I-1,3,K)-CC(I-1,4,K)
          TR3 = CC(I-1,3,K)+CC(I-1,4,K)
          CH(I-1,K,1) = CC(I-1,1,K)+TR2+TR3
          CH(I,K,1) = CC(I,1,K)+TI2+TI3
          CR2 = CC(I-1,1,K)+TR11*TR2+TR12*TR3
          CI2 = CC(I,1,K)+TR11*TI2+TR12*TI3
          CR3 = CC(I-1,1,K)+TR12*TR2+TR11*TR3
          CI3 = CC(I,1,K)+TR12*TI2+TR11*TI3
          CR5 = TI11*TR5+TI12*TR4
          CI5 = TI11*TI5+TI12*TI4
          CR4 = TI12*TR5-TI11*TR4
          CI4 = TI12*TI5-TI11*TI4
          DR3 = CR3-CI4
          DR4 = CR3+CI4
          DI3 = CI3+CR4
          DI4 = CI3-CR4
          DR5 = CR2+CI5
          DR2 = CR2-CI5
          DI5 = CI2-CR5
          DI2 = CI2+CR5
          CH(I-1,K,2) = WA1(I-1)*DR2+WA1(I)*DI2
          CH(I,K,2)   = WA1(I-1)*DI2-WA1(I)*DR2
          CH(I-1,K,3) = WA2(I-1)*DR3+WA2(I)*DI3
          CH(I,K,3)   = WA2(I-1)*DI3-WA2(I)*DR3
          CH(I-1,K,4) = WA3(I-1)*DR4+WA3(I)*DI4
          CH(I,K,4)   = WA3(I-1)*DI4-WA3(I)*DR4
          CH(I-1,K,5) = WA4(I-1)*DR5+WA4(I)*DI5
          CH(I,K,5)   = WA4(I-1)*DI5-WA4(I)*DR5
       END DO
    END DO
    RETURN
  END SUBROUTINE PASSF5



END MODULE Class_Wavelet


